function controls=makeuicontrolsboundaryexplorer(src,ev)
%Boundary Explorer
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

%if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %filtering uicontrols:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    parameterheader=text(.15-.10317,.70*1.0317-.0317,'Genetics','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    poptext=text(.30-.10317,.665*1.0317-.0317,'Population Size','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    gentext=text(.30-.10317,.620*1.0317-.0317,'Number of Generations','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    popedit=uicontrol('position',[origin(1)+190 origin(2)+400 50 22],...
        'style','edit','string','300','callback',[],...
        'backgroundcolor',[1 1 1],'tag','popedit');
    
    genedit=uicontrol('position',[origin(1)+190 origin(2)+370 50 22],...
        'style','edit','string','100','callback',[],...
        'backgroundcolor',[1 1 1],'tag','genedit');
    
    filterbutton=uicontrol('position',[origin(1)+65 origin(2)+320 165 30],...
        'style','pushbutton','string','Find Boundary','backgroundcolor',handles.colors.buttonbg,...
        'callback',@findbounds,'enable','on','tag','filterbutton');
    
    %             genzdone=text(.30,.450,'0 Generations Completed','horizontalalignment','left',...
    %                 'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
    %                 'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
    %                 'tag','genzdone','visible','off');
    
    exportbutton=uicontrol('position',[origin(1)+65 origin(2)+220 165 30],...
        'style','pushbutton','string','Export as Data File','backgroundcolor',handles.colors.buttonbg,...
        'callback',@ravensgaii_bounds_export,'enable','off','tag','exportbutton');
    
    
    if isempty(handles.constrainttext{udata.view.dataset}{udata.view.analysis})
        conststring='No constraints have been created yet.';
    else
        conststring=[cellstr('') handles.constrainttext{udata.view.dataset}{udata.view.analysis}];
    end
    
    
    constraintlist=uicontrol('position',[origin(1)+40 origin(2)+450 210 100],'style','listbox','max',100,...
        'string',conststring,'tag','constraintlist');
    
    
    
    controls=[parameterheader constraintlist    ...
        filterbutton poptext gentext popedit genedit exportbutton];
    
% else
%     %explain that they can't do that.
%     nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
%         'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
%         'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
%         'fontweight','bold','backgroundcolor','none');
%     
%     handles.tabobjects(end+1)=nonotext;
% end
% 


%guidata(src,handles);


function findbounds(src,ev)

handles=guidata(src);
getudata;


genedit=findobj(handles.tabobjects,'tag','genedit');
popedit=findobj(handles.tabobjects,'tag','popedit');

paretotarget={[-inf -inf];[-inf inf];[ inf inf];[inf -inf]};

d=udata.view.dataset;
x=udata.view.x;
y=udata.view.y;

N=floor(str2double(get(popedit,'string')));
T=floor(str2double(get(genedit,'string')));
M=2; %always 2 objectives for 2-D scatterplot pareto front.


pcross=0.9;
pmutation=0.9;
nbits=8; %number of bits to discretize each input var

plotvars=[udata.view.x udata.view.y];

funvars=handles.funvars

settings.animation=false;
settings.gif =false;
settings.fps = 0;
settings.loop = 0;
settings.pcross = .9;
settings.pmutation = .9;
settings.nbits = 8;
settings.method='nsgaii';
settings.pop= N;
settings.gens = T;
settings.plotprogress = true;
settings.plotvars = plotvars;
settings.targets = paretotarget{1};
settings.d = d;
settings.a = udata.view.analysis;
settings.x = [x y];
settings.activeconstraints = 0;
settings.startbutton = 0;
settings.stopbuttonpos = 0;

indexx=handles.funvars{d}{settings.x(1)};
for k=2:numel(settings.x)
    indexx=union(indexx,handles.funvars{d}{settings.x(k)});
end
settings.funvars=indexx;

ravensgaii_bounds(handles.fig,[],settings);
