function controls=makeuicontrolsoregontrail(src,ev)
%Frontier Explorer (2D)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

%check that both variables are functions:
if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %filtering uicontrols:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    targetstext=text(.275-.10317,.870*1.0317-.0317,'Enter target values:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    targ1text=text(.30-.10317,.835*1.0317-.0317,handles.allnames{udata.view.dataset}{udata.view.x},'horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    targ2text=text(.30-.10317,.790*1.0317-.0317,handles.allnames{udata.view.dataset}{udata.view.y},'horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    targ1edit=uicontrol('position',[origin(1)+190 origin(2)+510 50 22],...
        'style','edit','string','-inf','callback',[],...
        'backgroundcolor',[1 1 1],'tag','targ1edit');
    
    targ2edit=uicontrol('position',[origin(1)+190 origin(2)+480 50 22],...
        'style','edit','string','-inf','callback',[],...
        'backgroundcolor',[1 1 1],'tag','targ2edit');
    
    parameterheader=text(.15-.10317,.74*1.0317-.0317,'Genetics','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,...
        'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    poptext=text(.25-.10317,.7*1.0317-.0317,'Population Size','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    gentext=text(.25-.10317,.650*1.0317-.0317,'Number of Generations','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','normal','backgroundcolor','none');
    
    popedit=uicontrol('position',[origin(1)+190 origin(2)+425 50 22],...
        'style','edit','string','300','callback',[],...
        'backgroundcolor',[1 1 1],'tag','popedit');
    
    genedit=uicontrol('position',[origin(1)+190 origin(2)+395 50 22],...
        'style','edit','string','100','callback',[],...
        'backgroundcolor',[1 1 1],'tag','genedit');
    
    genzdone=text(.30-.10317,.560*1.0317-.0317,'0 Generations Completed','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',9,...
        'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none',...
        'tag','genzdone','visible','off');
    
    gobutton=uicontrol('position',[origin(1)+65 origin(2)+350 165 30],...
        'style','pushbutton','string','Find Frontier','backgroundcolor',handles.colors.buttonbg,...
        'callback',@otrail_frontier,'enable','off','tag','startbutton');
    
    
    anicheck=uicontrol('position',[origin(1)+40 origin(2)+215 120 22],...
        'style','checkbox','string','Export Animation','callback',@anicheck_callback,...
        'backgroundcolor',[1 1 1],'tag','anicheck','value',0,'fontsize',9);
    

    
    %text headers:
    varordertext=text(.15-.10317,.460*1.0317-.0317,'Order in which to change variables:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    %get variables for these two functions:
    funvars=union(handles.funvars{udata.view.dataset}{udata.view.x},handles.funvars{udata.view.dataset}{udata.view.y});
    varnames=handles.allnames{udata.view.dataset}(funvars);
    
    
    %make a listbox of the variables, and two arrow buttons to move
    %them up and down to select order in which to make changes.
    varordermenu=uicontrol('position',[origin(1)+30 origin(2)+60 180 210],...
        'style','listbox','string',varnames,'callback',[],...
        'backgroundcolor',[1 1 1],'tag','varordermenu','max',1000);
    
    upbutton=uicontrol('position',[origin(1)+213 origin(2)+170 35 80],...
        'style','pushbutton','string','^','backgroundcolor',handles.colors.buttonbg,...
        'callback',@upbutton_Callback);
    downbutton=uicontrol('position',[origin(1)+213 origin(2)+88 35 80],...
        'style','pushbutton','string','v','backgroundcolor',handles.colors.buttonbg,...
        'callback',@downbutton_Callback);
    
    %preserve colors checkbox:
    preservecolors=uicontrol('position',[origin(1)+30 origin(2)+10 200 30],...
        'style','checkbox','string','Preserve Variable Colors','backgroundcolor',handles.colors.tabbg,...
        'callback','','value',1,'tag','keepcolors');
        

    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    %slide bar axes:
    
    axpos=[origin(1)+30 origin(2)+295 200 20];
    %create the axes:
    slideax=axes('units',get(handles.fig,'units'),'position',axpos,...
        'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],...
        'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','on',...
        'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7],'tag','otrailax');
    
    
    set(slideax,'visible','off')
    L=line(0:.1:1,[.5 .5 .5 .5 .5 .5 .5 .5 .5 .5 .5],'linewidth',1,'color',[0 0 0],'marker','+','parent',slideax); %#ok<NASGU>
    blobs=line(0.50,.5,'linestyle','none','markerfacecolor',handles.colors.text,'marker','o',...
        'markeredgecolor',[0 0 0],'buttondownfcn',@otrail_click,'userdata','','tag','otrailgrabber','parent',slideax); %#ok<NASGU>
    
    
    
    %set graph callback to allow point selection
    set(handles.activegraph,'hittest','on','buttondownfcn',@otrail_select)
    
    
        
    controls=[parameterheader targetstext targ1text targ2text targ1edit...
        targ2edit gobutton poptext gentext popedit genedit genzdone ...
        anicheck varordertext varordermenu upbutton downbutton ...
        preservecolors slideax];
    
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    
    controls=nonotext;
end
guidata(src,handles);




function otrail_frontier(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

genedit=findobj(handles.tabobjects,'tag','genedit');
popedit=findobj(handles.tabobjects,'tag','popedit');

d=udata.view.dataset;
x=udata.view.x;
y=udata.view.y;
N=floor(str2double(get(popedit,'string'))); %population size
T=floor(str2double(get(genedit,'string'))); %population size

settings.animation=false;
settings.gif =false;
settings.fps = 0;
settings.loop = 0;
settings.pcross = .9;
settings.pmutation = .9;
settings.nbits = 8;
settings.method='nsgaii';
settings.pop= N;
settings.gens = T;
settings.plotprogress = true;
settings.plotvars = [x y];
settings.targets = handles.targetvalues{1}([x y]);
settings.d = d;
settings.a = udata.view.analysis;
settings.x = [x y];
settings.activeconstraints = 0;
settings.startbutton = src;
settings.stopbuttonpos = [];

%first make the frontier
ravensgaii(src,ev,settings)

%now get change-in-x data:
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

isx=~handles.isresponse{udata.view.dataset};

exes=handles.temp.finalpop;
whys=handles.temp.finalresp;

[B,index]=sortrows(whys);
sortedexes=exes(index,isx);

grabby=findobj('tag','otrailgrabber');

frontierposition=get(grabby,'xdata');

popsize=size(sortedexes,1);

index=ceil(popsize*frontierposition);

udata.explore.enddata=sortedexes(index,:);


%update userdata:
set(handles.fakeax(handles.activeindex),'userdata',udata);

%draw path:
trackchanges_helper(src,ev);



function otrail_select(src,ev)
%like point 2point select, but only pick a starting point. (no ctrl clikcing)

%select the current graph:
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
xcol=udata.view.x;
ycol=udata.view.y;

nrows=size(handles.alldata{d},1);

cc = get(handles.activegraph,'CurrentPoint');

xclick=cc(1,1);
yclick=cc(1,2);

%reformat data so it is all of the same order of magnitude:
%(based on values in row 1)
omagx=10.^(floor(log10(handles.alldata{d}(1,xcol))));
omagy=10.^(floor(log10(handles.alldata{d}(1,ycol))));

tesselationdata=zeros(nrows,2);
tesselationdata(:,1)=handles.alldata{d}(:,xcol)./omagx;
tesselationdata(:,2)=handles.alldata{d}(:,ycol)./omagy;

xclick=xclick/omagx;
yclick=yclick/omagy;

tri=delaunay(tesselationdata(:,1),tesselationdata(:,2));

%find index of closest point to where you clicked.
index = dsearch(tesselationdata(:,1),tesselationdata(:,2),tri,xclick,yclick);

%get coords for this point:
x=handles.alldata{d}(index,xcol);
y=handles.alldata{d}(index,ycol);



%if a point was already selected, delete it:
if ~isempty(findobj(handles.activegraph,'tag','startpoint'))
    delete(findobj(handles.activegraph,'tag','startpoint'))
end

udata.explore.start=index;
udata.explore.startdata=handles.alldata{d}(index,:);

%plot it:
handles.tempplots(1)=line(x,y,'marker','p','markersize',14,'markerfacecolor',[0 1 0],...
    'markeredgecolor',[0 0 0],'tag','startpoint','hittest','off');

%update userdata
set(handles.fakeax(handles.activeindex),'userdata',udata);

% if they've selected start and endpoints, enable the go button:
gobutton=findobj(handles.tabobjects,'tag','startbutton');

%if ~isempty(findobj(handles.activegraph,'tag','endpoint')) && ~isempty(findobj(handles.activegraph,'tag','startpoint'))
set(gobutton,'enable','on')
%end

guidata(src, handles);


function otrail_click(src,ev)
%for clicking on the slide bar.
handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn',{@otrailslide,handles.fig})
set(handles.fig,'windowbuttonupfcn',{@otraildone,handles.fig})

function otrailslide(hObject,eventdata,fig)
handles=guidata(fig);
udata=get(handles.fakeax(handles.activeindex),'userdata');

slideax=findobj('tag','otrailax');

if ~isempty(hObject) %if this function is called as a callback: (get mouse location and slide bars)
    clicked=get(slideax,'currentpoint');
    val=clicked(1,1);
    
    %move marker within bounds
    if val<0
        val=0;
    elseif val>1
        val=1;
    end
    set(gco,'xdata',val)
end


%% UPDATE GRAPH:

isx=~handles.isresponse{udata.view.dataset};
exes=handles.temp.finalpop;
whys=handles.temp.finalresp;

[B,index]=sortrows(whys);
sortedexes=exes(index,isx);

frontierposition=get(gco,'xdata');

popsize=size(sortedexes,1);

index=ceil(popsize*frontierposition);
if index==0
    index=1;
end
udata.explore.enddata=sortedexes(index,:);

%update userdata:
set(handles.fakeax(handles.activeindex),'userdata',udata);

%draw path:
trackchanges_helper(hObject,[]);


function otraildone(hObject,eventdata,fig)
handles=guidata(fig);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')


function upbutton_Callback(src,eventdata)
handles=guidata(src);

varordermenu=findobj(handles.fig,'tag','varordermenu');
currentstring=get(varordermenu,'string');
selectedvars=get(varordermenu,'value');

beforethese=1:min(selectedvars)-2;
afterthese=setdiff(min(selectedvars)-1:numel(currentstring),selectedvars);
afterthese=afterthese(afterthese>0);

neworder=[beforethese selectedvars afterthese];
newstring=currentstring(neworder);
newvalue=find(ismember(neworder,selectedvars));

set(varordermenu,'string',newstring,'value',newvalue)

function downbutton_Callback(src,eventdata)
handles=guidata(src);

varordermenu=findobj(handles.fig,'tag','varordermenu');
currentstring=get(varordermenu,'string');
selectedvars=get(varordermenu,'value');

beforethese=setdiff(1:max(selectedvars)+1,selectedvars);
beforethese=beforethese(beforethese<=numel(currentstring));
afterthese=max(selectedvars)+2:numel(currentstring);

neworder=[beforethese selectedvars afterthese];
newstring=currentstring(neworder);
newvalue=find(ismember(neworder,selectedvars));

set(varordermenu,'string',newstring,'value',newvalue)
