function controls=makeuicontrolspoint2point(src,ev)
%Point-to-Point
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

%check that both variables are functions:
if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    %text headers:
    varordertext=text(.15-.10317,.850*1.0317-.0317,'Order in which to change variables:','horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    %get variables for these two functions:
    funvars=union(handles.funvars{udata.view.dataset}{udata.view.x},handles.funvars{udata.view.dataset}{udata.view.y});
    varnames=handles.allnames{udata.view.dataset}(funvars);
    
    %make a listbox of the variables, and two arrow buttons to move
    %them up and down to select order in which to make changes.
    varordermenu=uicontrol('position',[origin(1)+30 origin(2)+300 180 210],...
        'style','listbox','string',varnames,'callback',[],...
        'backgroundcolor',[1 1 1],'tag','varordermenu','max',1000);
    
    upbutton=uicontrol('position',[origin(1)+213 origin(2)+410 35 80],...
        'style','pushbutton','string','^','backgroundcolor',handles.colors.buttonbg,...
        'callback',@upbutton_Callback);
    downbutton=uicontrol('position',[origin(1)+213 origin(2)+328 35 80],...
        'style','pushbutton','string','v','backgroundcolor',handles.colors.buttonbg,...
        'callback',@downbutton_Callback);
    
    % text explaining what to do:
    dothistext=text(.20-.10317,.420*1.0317-.0317,{'Click a point to make it the';'starting point. Ctrl+click a point';'to make it the end point. After';'selecting two points, click below.'},'horizontalalignment','left',...
        'backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
    
    % go button
    gobutton=uicontrol('position',[origin(1)+30 origin(2)+180 200 30],...
        'style','pushbutton','string','Connect the Dots','backgroundcolor',handles.colors.buttonbg,...
        'callback',@trackchanges_helper,'enable','off','tag','gobutton');
    
    %preserve colors checkbox:
    preservecolors=uicontrol('position',[origin(1)+30 origin(2)+140 200 30],...
        'style','checkbox','string','Preserve Variable Colors','backgroundcolor',handles.colors.tabbg,...
        'callback','','value',1,'tag','keepcolors');
    
    %update tabobjects:
    controls=[varordertext varordermenu upbutton downbutton dothistext gobutton...
        preservecolors];
    
    %set graph callback to allow point selection
    set(handles.activegraph,'hittest','on','buttondownfcn',@point2point_select)
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    
    controls=nonotext;
end



guidata(src,handles);


%% point 2 point callbacks:

function upbutton_Callback(src,eventdata)
handles=guidata(src);

varordermenu=findobj(handles.fig,'tag','varordermenu');
currentstring=get(varordermenu,'string');
selectedvars=get(varordermenu,'value');

beforethese=1:min(selectedvars)-2;
afterthese=setdiff(min(selectedvars)-1:numel(currentstring),selectedvars);
afterthese=afterthese(afterthese>0);

neworder=[beforethese selectedvars afterthese];
newstring=currentstring(neworder);
newvalue=find(ismember(neworder,selectedvars));

set(varordermenu,'string',newstring,'value',newvalue)

function downbutton_Callback(src,eventdata)
handles=guidata(src);

varordermenu=findobj(handles.fig,'tag','varordermenu');
currentstring=get(varordermenu,'string');
selectedvars=get(varordermenu,'value');

beforethese=setdiff(1:max(selectedvars)+1,selectedvars);
beforethese=beforethese(beforethese<=numel(currentstring));
afterthese=max(selectedvars)+2:numel(currentstring);

neworder=[beforethese selectedvars afterthese];
newstring=currentstring(neworder);
newvalue=find(ismember(neworder,selectedvars));

set(varordermenu,'string',newstring,'value',newvalue)
