function point2point_select(src,eventdata)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%select the current graph:
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
xcol=udata.view.x;
ycol=udata.view.y;

nrows=size(handles.alldata{d},1);

cc = get(handles.activegraph,'CurrentPoint');
maindata=findobj(handles.activegraph,'tag','main');

xclick=cc(1,1);
yclick=cc(1,2);

index=ravefindclosestpoint2D(xclick,yclick,maindata);


%{
%reformat data so it is all of the same order of magnitude:
%(based on values in row 1)
omagx=10.^(floor(log10(handles.alldata{d}(1,xcol))));
omagy=10.^(floor(log10(handles.alldata{d}(1,ycol))));

tesselationdata=zeros(nrows,2);
tesselationdata(:,1)=handles.alldata{d}(:,xcol)./omagx;
tesselationdata(:,2)=handles.alldata{d}(:,ycol)./omagy;

xclick=xclick/omagx;
yclick=yclick/omagy;

tri=delaunay(tesselationdata(:,1),tesselationdata(:,2));

%find index of closest point to where you clicked.
index = dsearch(tesselationdata(:,1),tesselationdata(:,2),tri,xclick,yclick);
%}


%get coords for this point:
x=handles.alldata{d}(index,xcol);
y=handles.alldata{d}(index,ycol);

%determine if this was start point or end point selection:
if strcmp(get(handles.fig,'selectiontype'),'alt') %end point

    %if a point was already selected, delete it:
    if ~isempty(findobj(handles.activegraph,'tag','endpoint'))
        delete(findobj(handles.activegraph,'tag','endpoint'))
    end

    udata.explore.end=index;
    udata.explore.enddata=handles.alldata{d}(index,:);

    %plot it:
    handles.tempplots(2)=line(x,y,'marker','p','markersize',14,'markerfacecolor',[1 0 0],...
        'markeredgecolor',[0 0 0],'tag','endpoint','hittest','off');

else %start point
    %if a point was already selected, delete it:
    if ~isempty(findobj(handles.activegraph,'tag','startpoint'))
        delete(findobj(handles.activegraph,'tag','startpoint'))
    end

    udata.explore.start=index;
    udata.explore.startdata=handles.alldata{d}(index,:);

    %plot it:
    handles.tempplots(1)=line(x,y,'marker','p','markersize',14,'markerfacecolor',[0 1 0],...
        'markeredgecolor',[0 0 0],'tag','startpoint','hittest','off');
end

%update userdata
set(handles.fakeax(handles.activeindex),'userdata',udata);

% if they've selected start and endpoints, enable the go button:
gobutton=findobj(handles.tabobjects,'tag','gobutton');

if ~isempty(findobj(handles.activegraph,'tag','endpoint')) && ~isempty(findobj(handles.activegraph,'tag','startpoint'))
    set(gobutton,'enable','on')
end

guidata(src, handles);
