function trackchanges_Callback(src,eventdata)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
% x=udata.view.x;
% y=udata.view.y;

% start=udata.explore.start;
% finish=udata.explore.end;

startdata=udata.explore.startdata;
enddata=udata.explore.enddata;


%delete existing lines: (lineseries without a pentagram marker)
linez=findobj(handles.activegraph,'tag','point2point');
if ~isempty(linez)
    delete(linez)
end

%determine which inputs variables are different between these two rows:
%isdifferent=and(~handles.isresponse{d},handles.alldata{d}(start,:)~=handles.alldata{d}(finish,:));
isdifferent=startdata(1:length(enddata))~=enddata;
isdifferent=isdifferent(~handles.isresponse{d});
allnames=handles.allnames{d};

%remove spaces from these to match varnames format (ravejmp2m removes
%spaces in variable names)
for j=1:length(allnames)
    allnames{j}=allnames{j}(allnames{j}~=char(32));
end

%get names of different variables:
diffnames=allnames(isdifferent);

%put names in order in which to make changes:
varordermenu=findobj(handles.tabobjects,'tag','varordermenu');
sortnames=get(varordermenu,'string');
includesorted=ismember(sortnames,diffnames);
sortnames=sortnames(includesorted);

%includethisvar=zeros(1,numel(sortnames));
realindex=zeros(1,numel(sortnames));

%find the index into allnames of sortnames.
for jj=1:numel(sortnames)
    realindex(jj)=find(strcmp(allnames,sortnames{jj}));
end

%generate "DOE" table of inputs to evaluate:
%use 7 runs for each variable: two near each end point, and three in the
%middle, to calculate value and 1st derivatives at each end and in the
%middle. note, rundata will contain incorrect values for response
%variables, but these are never used anywhere.

npaths=7;

rundata=repmat(startdata,[npaths*length(realindex) 1]); %start with matrix of starting point



for v=1:length(realindex)
    thisvar=realindex(v);
    %     startvalue=handles.alldata{d}(start,thisvar);
    %     endvalue=handles.alldata{d}(finish,thisvar);

    startvalue=startdata(thisvar);
    endvalue=enddata(thisvar);

    if startvalue~=endvalue
        values=(startvalue:(endvalue-startvalue)/(npaths-1):endvalue)';
    else
        values=startvalue*ones(npaths,1);
    end

    newcolumn=[startvalue*ones((v-1)*npaths,1);values;endvalue*ones((length(realindex)-v)*npaths,1)];
    rundata(:,realindex(v))=newcolumn;

end

% evaluate functions at this value:
outs=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],rundata);

xvalues=outs(:,1);
yvalues=outs(:,2);


%yvalues=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.y},rundata,handles.nargout{udata.view.dataset}(udata.view.y));



%make plots:

colors=handles.variablecolors{udata.view.dataset}(realindex,:);

h=zeros(length(realindex),1);
for kk=1:length(realindex)

    h(kk)=line(xvalues(npaths*(kk-1)+1:npaths*kk),yvalues(npaths*(kk-1)+1:npaths*kk),'color',colors(kk,:),'linewidth',3,...
        'parent',handles.activegraph,'hittest','off','tag','point2point');
    line(xvalues(npaths*kk),yvalues(npaths*kk),'markerfacecolor',colors(kk,:),'linestyle','none',...
        'parent',handles.activegraph,'marker','o','markeredgecolor',[0 0 0],'markersize',8,'hittest','off','tag','point2point');

end

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

%make legend:

f=numel(sortnames);

% sortnames=cell(f,1);
% for e=1:f
%     sortnames{e}=['Design Var ',num2str(e)];
% end


L=legend(h,sortnames);



legdata=get(handles.fakerax(handles.activeindex),'userdata');

legdata=union(legdata,L);
legdata=legdata(ishandle(legdata));

set(handles.fakerax(handles.activeindex),'userdata',legdata);

guidata(src,handles);
