function controls=makeuicontrolsspaceshift(src,ev)
%Data Shift
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');


%note, need 2 functions to use this method.

if handles.isfunction{udata.view.dataset}(udata.view.x) && handles.isfunction{udata.view.dataset}(udata.view.y)
    %do stuff
    
    d=udata.view.dataset;
    fieldname='magic';
    
    %get variables for these two functions:
    varnames=union(handles.funvars{udata.view.dataset}{udata.view.x},handles.funvars{udata.view.dataset}{udata.view.y});
    constvars=find(logical(handles.isconstant{udata.view.dataset}));
    varnames=setdiff(varnames,constvars);
    varnames=handles.allnames{udata.view.dataset}(varnames);
    allnames=handles.allnames{d};
    
    
    ranges=[min(handles.alldata{d})' max(handles.alldata{d})'];
    
    %format position:
    pos=[10 300 230 220]; %250
    axpos=[origin(1)+pos(1) origin(2)+pos(2) pos(3)-18 pos(4)];
    slpos=[origin(1)+pos(1)+pos(3)-17 origin(2)+pos(2)-1 17 pos(4)+1];
    
    %create the axes:
    handles.(fieldname).ax=axes('units',get(handles.fig,'units'),'position',axpos,...
        'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],...
        'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','on',...
        'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7]);
    
    %create the slidebar:
    handles.(fieldname).slider=uicontrol('units',get(handles.fig,'units'),'position',slpos,...
        'style','slider');
    
    nvars=numel(varnames);
    
    ndisp=floor(pos(4)/14.7); %number of rows of text that will fit in the box
    
    %make box:
    L=zeros(1,nvars);
    l=L;
    
    if nvars>ndisp
        for t=1:nvars
            text(0.38,t/ndisp,varnames{end-t+1},'horizontalalignment','right','clipping','on','verticalalignment','top');
            
            set(handles.(fieldname).slider,'max',nvars/ndisp-1,'value',nvars/ndisp-1);
            set(handles.(fieldname).ax,'ylim',[nvars/ndisp-1 nvars/ndisp]);
            range=get(handles.(fieldname).slider,'max')-get(handles.(fieldname).slider,'min');
            set(handles.(fieldname).slider,'sliderstep',[1/(ndisp*range)-.0001 (1-1/ndisp)/range])
            
            L(t)=line([0.43 .73],[t/ndisp-.03 t/ndisp-.03],'linewidth',1,'color',[0 0 0],'marker','+');
            
            l(t)=line(0.58,t/ndisp-.03,'linestyle','none','markerfacecolor',[.9 .9 .9],'marker','o',...
                'markeredgecolor',[0 0 0],'buttondownfcn',{@spaceshiftclick,fieldname},'userdata',nvars-t+1);
            
            handles.(fieldname).valtexts(nvars-t+1)=text(0.78,t/ndisp,num2str(mean(ranges(t,:))),'horizontalalignment','left','clipping','on','verticalalignment','top');
        end
        
    else %small amount of variables... don't need slider
        for t=1:nvars
            thisvar=find(strcmp(allnames,varnames{end-t+1}));
            text(0.38,(t+ndisp-nvars)/ndisp,varnames{end-t+1},'horizontalalignment','right','clipping','off','verticalalignment','top');
            
            set(handles.(fieldname).slider,'enable','off','visible','off')
            set(handles.(fieldname).ax,'visible','off')
            
            L(t)=line([0.43 .73],[(t+ndisp-nvars)/ndisp-.029 (t+ndisp-nvars)/ndisp-.029],...
                'linewidth',1,'color',[0 0 0],'marker','+');
            
            l(t)=line(0.58,(t+ndisp-nvars)/ndisp-.029,'linestyle','none','markerfacecolor',[.9 .9 .9],'marker','o',...
                'markeredgecolor',[0 0 0],'buttondownfcn',{@spaceshiftclick,fieldname},'userdata',thisvar);
            
            handles.(fieldname).valtexts(thisvar)=text(0.78,(t+ndisp-nvars)/ndisp,num2str(mean(ranges(thisvar,:))),'horizontalalignment','left','clipping','on','verticalalignment','top');
        end
        
    end
    
    
    %store ranges for future use.
    handles.(fieldname).ranges=ranges;
    
    %on/off switches:
    handles.(fieldname).onoff=false(numel(handles.allnames{d}),1);
    
    %IDEA: save last slide bar position/onoff and use this as default the next time user chooses
    %this option.
    
    methodtext=text(.15-.10317,.85*1.0317-.0317,'Variable Values','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    formattext=text(.15-.10317,.45*1.0317-.0317,'Style','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);
    
    
    drawlinescheck=uicontrol('style','checkbox','position',[origin(1)+40 origin(2)+300-60 120 22],...
        'string','Draw trajectories','tag','drawlinescheck','backgroundcolor',handles.colors.tabbg);
    hidedata=uicontrol('style','checkbox','position',[origin(1)+40 origin(2)+275-60 120 22],...
        'string','Hide original data','tag','hidedatacheck','backgroundcolor',handles.colors.tabbg);
    
    
    
   
    handles.temp.varorder=zeros(1,numel(handles.allnames{d}));
    handles.temp.lastvar=0;
    
    main=findobj(handles.activegraph,'tag','main');
    %hhh=copyobj(main,handles.activegraph);
    %set(hhh,'tag','crap'); %dont want it named 'main'.
    
    set(main,'visible','off');
    
    handles.temp.shiftdata=line(get(main,'xdata'),get(main,'ydata'),'linestyle','none','marker','o','markeredgecolor',[0 0 0],...
        'markerfacecolor',handles.colors.text,'parent',handles.activegraph);
    
    controls=[handles.(fieldname).ax handles.(fieldname).slider handles.(fieldname).valtexts(handles.(fieldname).valtexts>0) L l,...
        methodtext drawlinescheck formattext hidedata];
    
else
    %explain that they can't do that.
    nonotext=text(.25-.10317,.820*1.0317-.0317,{'This feature is only available';'if both variables are modeled';'by regressions.'},...
        'horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
        'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],...
        'fontweight','bold','backgroundcolor','none');
    
    controls=nonotext;
end

guidata(src,handles);


function spaceshiftclick(hObject,eventdata,fieldname)
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.fig,'windowbuttonmotionfcn',{@spaceshiftslide,fieldname})
set(handles.fig,'windowbuttonupfcn',@slidebardone)

%double click to turn variables on/off:
if ~strcmp(get(handles.fig,'selectiontype'),'open') %double click
    return
end


thisvar=get(gco,'userdata');
if handles.(fieldname).onoff(get(gco,'userdata')) %if on, turn off
    
    handles.(fieldname).onoff(get(gco,'userdata'))=0;
    
    %delete lines for this marker:
    %linez=findobj(handles.activegraph,'type','line','userdata',thisvar);
    linez=findobj(handles.activegraph,'tag','trajectory');
    delete(linez);
    
    %remove this variable from turned on list:
    oldspot=handles.temp.varorder(thisvar);
    handles.temp.varorder(thisvar)=0;
    handles.temp.varorder=handles.temp.varorder-double(handles.temp.varorder>oldspot);
    
    handles.temp.lastvar=0;
    
    set(gco,'markerfacecolor',[.9 .9 .9])
    
    %fix axes limits:
    set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
        'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))
    
    guidata(handles.fig,handles);
    
else %turn this variable on
    
    cmap=handles.variablecolors{udata.view.dataset};
    
    %get old values:
    values=handles.magic.valtexts;
    
    thisvar=get(gco,'userdata');
    set(gco,'markerfacecolor',cmap(thisvar,:))
    handles.(fieldname).onoff(get(gco,'userdata'))=1;
    
    
    %handles.temp.lastvar=thisvar;
    
    guidata(handles.fig,handles);
    
    
    
    %{
    
    
    %draw lines connecting endpoints:
    d=udata.view.dataset;

    %mark this variable as most recently turned on:
    handles.temp.varorder(thisvar)=max(handles.temp.varorder)+1;
    
    
    

    
    
    currentins=handles.alldata{d}(:,~handles.isresponse{d});
    npoints=14; %number of points to generate smooth curve
    rundata=repmat(currentins,[1 1 npoints]); %start with 3d array of starting points
    guidata(handles.fig,handles);

    %for each variable that's on:
    for j=1:max(handles.temp.varorder)
        handles=guidata(handles.fig);
        currentvar=find(handles.temp.varorder==j);

        if j>1
            %set(handles.temp.tictacs,'markerfacecolor',[.5 .5 .5])
            linez=findobj(get(handles.allgraphs(handles.activeindex),'children'),'type','line','marker','none');
            delete(linez);
            
            try %#ok<TRYNC>
                set(handles.temp.tictacs,'markerfacecolor',[.7 .7 .7]);
                delete(handles.temp.tictacs);

            end
        end

        %THIS METHOD DRAWS CURVE FROM MIN TO MAX VALUE OF THISVAR
        thisvar=currentvar;

        startvalue=min(handles.alldata{d}(:,thisvar));

        nrows=size(handles.alldata{d}(:,:),1);
        endvalue=max(handles.alldata{d}(:,thisvar));
        if startvalue==endvalue;continue;end; %avoids error when double clicking a constant.

        thediff=endvalue-startvalue;
        stepsize=thediff/(npoints-1);
        newvalues=repmat(startvalue:stepsize:endvalue,[nrows 1]);

        %insert slice into 3D array:
        rundata(:,thisvar,:)=newvalues;

        xvalues=zeros(nrows,npoints);
        yvalues=xvalues;

        for u=1:npoints
            newins=rundata(:,:,u);

            xvalues=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.x},newins,handles.nargout{udata.view.dataset}(udata.view.x));
            yvalues=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.y},newins,handles.nargout{udata.view.dataset}(udata.view.y));



        end

        jkl=zeros(1,nrows);
        %draw line from old point to new point


        if j==max(handles.temp.varorder);
            %draw lines: (for last var turned on only)
            for k=1:nrows
                jkl(k)=line(xvalues(k,:),yvalues(k,:),...
                    'parent',handles.activegraph,'marker','none','color',get(gco,'markerfacecolor'),...
                    'markerfacecolor',get(gco,'markerfacecolor'),'userdata',thisvar,...
                    'linewidth',1,'linestyle','-');
            end
        end

        %bring original scatter data to top:
        %scattr=findobj(get(handles.allgraphs(handles.activeindex),'children'),'type','hggroup');
        uistack(handles.temp.shiftdata,'top');

%         handles.temp.tictacs=line(xvalues(:,end),yvalues(:,end),...
%             'parent',handles.activegraph,'marker','o','markerfacecolor',get(gco,'markerfacecolor'),...
%             'markeredgecolor',[0 0 0],'linestyle','none','markersize',5,'color',get(gco,'markerfacecolor'),'userdata',thisvar);

        guidata(handles.fig,handles);
    %}
    
    
end

%move marker to correct spot for value set on slide bar.
spaceshiftslide(gco,[],fieldname);

%SET RUNDATA FOR THIS COLUMN TO THE VALUE SET BY SLIDEBAR, TO ENSURE SUBSEQUENT CLICKS
%occur at this value:
%rundata(:,thisvar,:)=str2double(get(values(thisvar),'string'));

%fix axes limits:
%         set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
%             'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

%end

%
%guidata(handles.fig,handles);


function spaceshiftslide(hObject,eventdata,fieldname) %#ok<INUSL>
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
clicked=get(handles.(fieldname).ax,'currentpoint');
val=clicked(1,1);
thisvar=get(gco,'userdata');



%move marker within bounds
if val>0.73
    val=0.73;
elseif val<0.43
    val=0.43;
end
set(gco,'xdata',val)

%update text values

range=handles.(fieldname).ranges(thisvar,:);
if range(1)-range(2)==0;return;end %avoids errors with constant variables.

newval=range(1)+((val-0.43)/0.3)*(range(2)-range(1));

set(handles.(fieldname).valtexts(thisvar),'string',num2str(newval));


% %if this variable is turned off, do not update graph:
% if ~handles.(fieldname).onoff(thisvar)
%     return
% end



%update scatterplot with lines:
d=udata.view.dataset;
% x=udata.view.x;
% y=udata.view.y;

ons=find(handles.(fieldname).onoff);

%currentouts=[handles.alldata{d}(:,x) handles.alldata{d}(:,y)];
currentins=handles.alldata{d};

%replace value of all currently-on variables with their slider value:
for j=ons'
    thisvarsvalue=str2double(get(handles.(fieldname).valtexts(j),'string')) ;
    currentins(:,j)=thisvarsvalue;
end


%if this slider was not the last one you changed, update trajectories:
if thisvar~=handles.temp.lastvar && handles.(fieldname).onoff(thisvar)
    
    delete(findobj(handles.activegraph,'tag','trajectory'));
    numpts=14;
    
    datarep=repmat(currentins,[numpts 1]); %tile this matrix numpts times.
    
    steps=linspace(handles.(fieldname).ranges(thisvar,1),handles.(fieldname).ranges(thisvar,2),numpts);
    
    numrows=size(currentins,1);
    for g=1:numpts
        datarep((g-1)*numrows+1:(g)*numrows,thisvar)=steps(g);
    end
    
    
    %evaulate these points:
    outs=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],datarep);
    
    xvalues=outs(:,1);
    yvalues=outs(:,2);
    
    %reshape into something that makes sense:
    
    xright=reshape(xvalues,[],numpts);
    yright=reshape(yvalues,[],numpts);
    
    cmap=handles.variablecolors{udata.view.dataset};
    
    
    for h=1:size(xright,1)
        line(xright(h,:),yright(h,:),'parent',handles.activegraph,'tag','trajectory','color',cmap(thisvar,:));
    end
    
    uistack(handles.temp.shiftdata,'top');
    
end



handles.temp.lastvar=thisvar;
guidata(handles.fig,handles);



%update scatter data to reflect changes:

% xcoord=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.x},currentins,handles.nargout{udata.view.dataset}(udata.view.x));
% ycoord=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.y},currentins,handles.nargout{udata.view.dataset}(udata.view.y));

currentouts=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],currentins);

xcoord=currentouts(:,1);
ycoord=currentouts(:,2);



set(handles.temp.shiftdata,'xdata',xcoord','ydata',ycoord');

return






for j=1:sum(handles.(fieldname).onoff)
    currentvar=ons(j);
    
    %copy current points:
    newins=currentins;
    %change column corresponding to currentvar:
    newins(:,currentvar)=str2double(get(handles.(fieldname).valtexts(currentvar),'string'));
    
    %evaluate functions at newins:

    %     xvalues=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.x},newins,handles.nargout{udata.view.dataset}(udata.view.x));
    %     yvalues=raveevaluatefunction(handles.funnames{udata.view.dataset}{udata.view.y},newins,handles.nargout{udata.view.dataset}(udata.view.y));
    %
    
    newouts=raveevaluatefunction(handles.fig,udata.view.dataset,[udata.view.x udata.view.y],newins);
    
    
    set(handles.temp.shiftdata,'xdata',newouts(:,1),'ydata',newouts(:,2));
    
    
    % currentouts=newouts;
    currentins=newins;
    
end

guidata(handles.fig,handles);


function slidebardone(varargin)
handles=guidata(varargin{1});
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')