function ravecreatecalculatedvalue(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
targetindex=find(handles.allgraphs==targetaxes);

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%hide axes:
set(targetaxes,'visible','off','xticklabel','','yticklabel','')
set(handles.fakeax(targetindex),'visible','off')
set(handles.fakerax(targetindex),'visible','off')

cla(targetaxes);

%process expression:



indata=udata.view.expression;

if any(indata==char(58)) %if any colons
    errordlg('Vectorized expressions are not allowed (get rid of the colons).','You can''t do that')
end

varnames=handles.allnames{d};
%replace longest names first in case a shorter name is a subset of a longer name.
varlengths=cellfun(@length,varnames);
[sortednames varlengthorder]=sort(varlengths,'descend');
isinput=false(1,numel(varnames));
for g=1:numel(varnames)
    thisvar=varnames{varlengthorder(g)};
    thisindex=num2str(find(ismember(varnames,thisvar)));
    isinput(g)=any(strfind(indata,thisvar));
    indata=strrep(indata,thisvar,['data(:,',thisindex,')']);
end
udata.view.vars=sort(varlengthorder(isinput)); %variables that appear in this expression


%remove newlines:
indata=indata(indata~=char(15));

%replace retarded "+-" business with just "-"
indata=strrep(indata,'+-','-');

%remove tabs:
%indata=strrep(indata,char(9),'');

%replace vector multiply/divide with scalar: (NOTE, KEEP ALL 6 OF THESE STATMENTS)
indata=strrep(indata,'.*','*');
indata=strrep(indata,'./','/');
indata=strrep(indata,'.^','^');
indata=strrep(indata,'*','.*');
indata=strrep(indata,'/','./');
indata=strrep(indata,'^','.^');

%convert funtion names to matlabese:
indata=strrep(indata,'Exp','exp');
indata=strrep(indata,'Sine','sin');
indata=strrep(indata,'Squish','logsig');
indata=strrep(indata,'Cosine','cos');
indata=strrep(indata,'Tangent','tan');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Abs','abs');
indata=strrep(indata,'Round','round');
indata=strrep(indata,'Floor','floor');
indata=strrep(indata,'Ceiling','ceil');

%% determine which variables appear in this constraint:

thisstring=indata;
indata=handles.xdials{d}{a};
varnumbers=udata.view.vars;

if ~isempty(varnumbers)
    %evaluate all regressions necessary for this constraint:
    respdata=raveevaluatefunction(handles.fig,udata.view.dataset,varnumbers,indata);
    indata(:,varnumbers)=respdata;
end

%solve evaluate this constraint for y variable:
try
    eval(['newhandle=@(data) ',thisstring,';']);
    g=newhandle(indata);
    
    if isempty(udata.format.format)
        numstring=num2str(g);
    else
        numstring=num2str(g,udata.format.format);
    end
    
catch %#ok<CTCH>
    g=nan;
    numstring='(ERROR)';
end

if udata.format.fontstyle==3
    style={'fontweight','bold','fontangle','italic'};
elseif udata.format.fontstyle==1
    style={'fontweight','bold','fontangle','normal'};
elseif udata.format.fontstyle==2
    style={'fontweight','normal','fontangle','italic'};
else
    style={'fontweight','normal','fontangle','normal'};
end


%conditional coloration:
activecondition=0;

if udata.format.usecondition2 && ~isempty(udata.format.condition2)
    strng=udata.format.condition2;
    try
        condish=eval(['g' strng]);
    catch
        condish=false;
    end
    
    
    
    if islogical(condish)
        if condish
            activecondition=2;
        end
    end
end

if udata.format.usecondition1 && ~isempty(udata.format.condition1)
    strng=udata.format.condition1;
    try
        condish=eval(['g' strng]);
    catch
        condish=false;
    end
    
    if islogical(condish)
        if condish
            activecondition=1;
        end
    end
    
end



if activecondition==1
    color=udata.format.c1color; 
    set(handles.allblockers(handles.activeindex),'facecolor',udata.format.c1bg)
elseif activecondition==2
    color=udata.format.c2color;
    set(handles.allblockers(handles.activeindex),'facecolor',udata.format.c2bg)
else
    color=udata.format.color;
    set(handles.allblockers(handles.activeindex),'facecolor',udata.format.bgcolor)
end


%make text:
text(0,0.5,[udata.format.prefix numstring udata.format.suffix],'parent',targetaxes,...
    'hittest','off','fontname',udata.format.fontname,'fontsize',udata.format.fontsize,...
    'color',color,'ver','middle',style{:});






