function ravecreatederivativeprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

set(handles.fakeax(handles.activeindex),'userdata',udata);

targetaxes=handles.activegraph; %=handles.activegraph
ycol=udata.view.y;
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

xdials=handles.xdials{d}{a};

numy=length(ycol);
numx=length(xcol);
numpts=udata.view.numpts;

if numx==0 %this can happen when switching to a new yvar that shares no vars with previous xvar selection.
    numx=1;
    xcol=1;
end

if numy==0
    set(handles.infobar,'string','Error: No variables are modeled by functions.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})

handles.subax{handles.activeindex}=[];
handles.subfakeax{handles.activeindex}=[];
handles.subfakerax{handles.activeindex}=[];
guidata(handles.fig,handles);

%get data
xnames=handles.allnames{d}(xcol);
ynames=handles.allnames{d}(ycol);

set(targetaxes,'Visible','off')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graphtitle,'fontsize',handles.prefs.fonts.graphtitlesize,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=25;
end

if ~udata.view.singleyaxis
    rows=numy;
else
    rows=1;
end
cols=numx;

% Create and plot into small axes
ax = zeros(rows,cols);
fakeax = zeros(rows,cols);
fakerax = zeros(rows,cols);

pos = get(targetaxes,'Position');

hspace = handles.prefs.profilerhspacing;
vspace = handles.prefs.profilervspacing; % 5 percent space between subaxes

width = floor((pos(3)-hspace*numx)/numx);
height = floor((pos(4)-titoffset-vspace*numy)/numy);

xlim = zeros([numy numx 2]);
ylim = zeros([numy numx 2]);


%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end

%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);
ydatatypes=handles.datatype{d}(ycol);

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

rowz=sum(totalrows);
xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:numx
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
    
else
    for k=1:numx
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:
yvalues=raveevaluatefunction(src,d,udata.view.y,xfixed);

%update axis limits only if needed:
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

ycurrent=raveevaluatefunction(src,d,udata.view.y,xdials);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));
ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

if udata.view.singleyaxis && udata.view.usevariablecolors
    linecolors=handles.variablecolors{d}(ycol,:);
elseif udata.view.singleyaxis
    linecolors=ravemakelinecolors(handles.fig,[],numel(ycol),1);
end

%% set up grid of axes.
for r=rows:-1:1,
    for c=cols:-1:1,
        axPos = single(floor([round(pos(1))+(c-1)*(width+hspace) pos(2)+(rows-r)*(height+vspace) ...
            width height]));
        
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,'color',bgcolor,...
            'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid,...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnderivativeprofiler,...
            'userdata',handles.activeindex,'color','none',...
            'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor,...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize); %userdata contains the index of the bigax for this subgraph.
        
        set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ticklength',[.02 .02])
        if strcmp(ydatatypes{r},'logical')
            set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ytick',[0 1],'yticklabel',{handles.prefs.logicalfalse handles.prefs.logicaltrue})
        end
        
        thisxcolor=udata.format.haircolor;
        thisycolor=udata.format.haircolor;
        if udata.format.smarthaircolors
            if handles.isconstant{udata.view.dataset}(xcol(c))
                thisxcolor=get(handles.fakeax(handles.activeindex),'xcolor');
                set(ax(r,c),'buttondownfcn','') %this var is constant so shouldnt update profiler when clicking it.
            end
            
            if ~ismember(xcol(c),handles.funvars{udata.view.dataset}{ycol(r)})
                thisycolor=get(handles.fakeax(handles.activeindex),'xcolor');
            end
        end
        %tracer data series:
        h.xhair(r,c)=line([xdials(xcol(c)) xdials(xcol(c))],[0 1],'visible','on','linestyle',udata.format.linestyle,'color',thisxcolor,...
            'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1,'linewidth',udata.format.tracersize);
        %response tracer series:
        if ~udata.view.singleyaxis
            h.yhair(r,c)=line([-10000000 1000000000],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',udata.format.linestyle,'color',thisycolor,...
                'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1,'linewidth',udata.format.tracersize);
        else
            for R=1:numy
                thisycolor=linecolors(R,:);
                h.yhair(R,c)=line([-10000000 1000000000],[ycurrent{R} ycurrent{R}],'visible','on','linestyle',udata.format.linestyle,'color',thisycolor,...
                    'parent',ax(r,c),'hittest','off','tag','hair','linewidth',udata.format.tracersize);
            end
        end
        %main data series:
        
        if  ~any(strcmp(datatypes{c},{'continuous';'constant'})) %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(c)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{c}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            if ~udata.view.singleyaxis
                h.main(r,c)=line(xvals(xmap),yvalues{r}{c}(ymap),'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                    'color',udata.format.linecolor,'hittest','off','tag','keeper','parent',ax(r,c));
            else
                
                for R=1:numy
                    h.main(R,c)=line(xvals(xmap),yvalues{R}{c}(ymap),'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                        'color',linecolors(R,:),'hittest','off','tag','keeper','parent',ax(r,c));
                end
            end
        else
            if ~udata.view.singleyaxis
                h.main(r,c)=line(xsteps{c},yvalues{r}{c},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                    'color',udata.format.linecolor,'hittest','off','tag','keeper','parent',ax(r,c));
            else
                for R=1:numy
                    h.main(R,c)=line(xsteps{c},yvalues{R}{c},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                        'color',linecolors(R,:),'hittest','off','tag','keeper','parent',ax(r,c));
                end
            end
        end
        
        if udata.format.showpoints && ~udata.view.singleyaxis
            nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
            xdata=handles.alldata{udata.view.dataset}(nothidden,udata.view.x(c));
            ydata=handles.alldata{udata.view.dataset}(nothidden,udata.view.y(r));
            
            h.points(r,c)=line(xdata,ydata,-ones(size(ydata)),'parent',ax(r,c),'tag','points','linestyle',...
                'none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
                'marker',udata.format.marker,'markersize',udata.format.markersize,'hittest','on','buttondownfcn',@buttondownfcnderivativeprofiler);
            uistack(h.points(r,c),'bottom');
        end
        
        if handles.minlimit{d}{a}(ycol(r)) ~= handles.maxlimit{d}{a}(ycol(r))
            set(ax(r,c),'xlimmode','auto','ylim',[handles.minlimit{d}{a}(ycol(r)) handles.maxlimit{d}{a}(ycol(r))],'ylimmode','manual','xgrid','off','ygrid','off')
        else
            set(ax(r,c),'xlimmode','auto','ylim',[handles.minlimit{d}{a}(ycol(r)) handles.maxlimit{d}{a}(ycol(r))+0.001],'ylimmode','manual','xgrid','off','ygrid','off')
        end
        
        xlim(r,c,:) = get(ax(r,c),'xlim');
        ylim(r,c,:) = get(ax(r,c),'ylim');
    end
end

%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
guidata(handles.fig,handles);

%make listeners:
delete(handles.axeslisteners{handles.activeindex});
handles.axeslisteners{handles.activeindex}=[];

%must store first listener outside of loop to initialize the variable type
%as "listener";

handles.axeslisteners{handles.activeindex}=addlistener(ax(1,1),'YTick','PreSet',@subaxeschangelistener);
for r=2:rows
    try %these dont work on 2007b,
        handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(r,1),'YTick','PreSet',@subaxeschangelistener);
    end
end

for c=1:cols
    try %these dont work on 2007b,
        handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(end,c),'XTick','PreSet',@subaxeschangelistener);
    end
end
guidata(handles.fig,handles);


if ~udata.format.showxtracer
    set(h.xhair,'visible','off')
end
if ~udata.format.showytracer
    set(h.yhair,'visible','off')
end


if numel(ycol)>1 && udata.view.singleyaxis
    leg1=legend(handles.activegraph,h.main(:,1),handles.allnames{d}(ycol),'interpreter',handles.prefs.interpreter);
    set(handles.fakerax(handles.activeindex),'userdata',leg1)
    set(leg1,'color',[1 1 1])
else
    leg1=[];
end

xlimmin = xmin(xcol);
xlimmax = xmax(xcol);

if udata.format.globallimits
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
else
    for r=1:numel(ycol);
        allyvalues=cell2mat(yvalues{r});
        ylimmin(r)=min(allyvalues);
        ylimmax(r)=max(allyvalues);
        if ylimmin(r)==ylimmax(r)
            ylimmin(r)=ylimmin(r)-0.01;
            ylimmax(r)=ylimmax(r)+0.01;
        end
    end
end

for r=1:numel(ycol);
    if ylimmin(r)==ylimmax(r)
        ylimmin(r)=ylimmin(r)-0.01;
        ylimmax(r)=ylimmax(r)+0.01;
    end
end

for u=1:cols
    if xlimmin(u)==xlimmax(u)
        xlimmin(u)=xlimmin(u)-0.01;
        xlimmax(u)=xlimmax(u)+0.01;
    end
end

inset = .10;
%fix y limits
for r=1:rows,
    if ~udata.view.singleyaxis
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
        set(h.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
    else
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
        set(h.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
    end
    
    %force non exponential labels:
    if ~strcmp(ydatatypes{r},'logical')
        set(ax(r,1),'YTickLabel',get(ax(r,1),'YTick'))
    end
    set(fakeax(r,1),'YTickLabel','')
    set(fakerax(r,1),'YTickLabel','')
    
    maxchars=floor(height./7);
    
    %make axes labels:
    if ~udata.view.singleyaxis && udata.view.showcurrentpoint %label and value
        set(get(ax(r,1),'ylabel'),'string',[linewrap(ynames{r},maxchars);{'';'';''}],'hittest','off','interpreter','none',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(r,1),'ylabel'),'string',{['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(ycurrent{r},5)];'';''},...
            'interpreter','tex','hittest','off','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    elseif ~udata.view.showcurrentpoint && ~udata.view.singleyaxis
        set(get(ax(r,1),'ylabel'),'string',[linewrap(ynames{r},maxchars);{'';''}],'hittest','off','interpreter','none',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(r,1),'ylabel'),'string',' ','interpreter','tex','hittest','off','fontname',handles.prefs.fonts.graph,...
            'fontsize',handles.prefs.fonts.graphsize);
    else %nothing
        set(get(ax(r,1),'ylabel'),'string','','hittest','off','interpreter','none',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(r,1),'ylabel'),'string','','interpreter','tex','hittest','off',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    end
    
    
end
dx = zeros(1,cols);

%fix x limits
for c=1:cols,
    try %gives error if xlimmin==xlimmax
        set([ax(1,c) fakeax(1,c) fakerax(1,c)],'xlim',[xlimmin(c) xlimmax(c)])
        dx(c) = diff(get(ax(1,c),'xlim'))*inset;
        set([ax(:,c) fakeax(:,c) fakerax(:,c)],'xlim',[xlimmin(c)-dx(c) xlimmax(c)+dx(c)])
    end
    %force non exponential labels:
    set(ax(end,c),'XTickLabel',get(ax(end,c),'XTick'))
    set(fakeax(end,c),'XTickLabel','')
    set(fakerax(end,c),'XTickLabel','')
    
    %make axes labels:
    maxchars=floor(width./8);
    if udata.format.xlabelabove
        set(get(ax(1,c),'title'),'string',linewrap(xnames{c},maxchars),...
            'interpreter','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    end
    
    if udata.format.xlabelbelow
        if udata.view.showcurrentpoint
            set(get(ax(end,c),'xlabel'),'string',[{''};{''};{''};linewrap(xnames{c},maxchars)],'verticalalignment','top',...
                'hittest','off','interpreter','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
            set(get(fakeax(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(xdials(xcol(c)),5)]},...
                'verticalalignment','top','interpreter','tex','hittest','on','buttondownfcn',@derivativeprofilerchangevaltexts,...
                'userdata',handles.activeindex,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        else
            set(get(ax(end,c),'xlabel'),'string',[{''};{''};linewrap(xnames{c},maxchars)],'verticalalignment','top',...
                'hittest','off','interpreter','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
            set(get(fakeax(end,c),'xlabel'),'string',{' ';' ';' '},...
                'verticalalignment','top','interpreter','tex','hittest','on','buttondownfcn',@derivativeprofilerchangevaltexts,...
                'userdata',handles.activeindex,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        end
    end
end

set(ax(1:rows-1,:),'xticklabel','')
set(ax(:,2:cols),'yticklabel','')

set(fakeax(1:rows-1,:),'xticklabel','')
set(fakeax(:,2:cols),'yticklabel','')

set(fakerax(1:rows-1,:),'xticklabel','')
set(fakerax(:,2:cols),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','derivativeprofiler','tag','DerivativeProfilerBigAx');

set(ax,'xlimmode','manual','ylimmode','manual')

if udata.format.xlabelbelow
    for c=1:numx
        ravemaketextticks(ax(end,c),[],xcol(c),[],[]);
    end
else
    set(ax,'xticklabel','')
    set(fakeax,'xticklabel','')
    set(fakerax,'xticklabel','')
end

%also replace yticklabels with strings, this allows y (axis) labels to be
%aligned with each other.
if udata.view.singleyaxis
    yticks=get(ax(1,1),'ytick');
    yticklabels=num2cell(yticks);
    yticklabels=cellfun(@num2str,yticklabels,'UniformOutput',false);
    ravemaketextticks(ax(1,1),[],[],0,[],[],[],yticklabels);
else
    for r=1:rows
        ravemaketextticks(ax(r,1),[],[],ycol(r),[]);
    end
end

%resize ticks:
%shrink tick labels:
xticklabels=findall(ax,'tag','xticklabel');
if iscell(xticklabels);xticklabels=cell2mat(xticklabels);end
set(xticklabels,'fontsize',handles.prefs.fonts.matrixsize);

%shrink tick labels:
yticklabels=findall(ax,'tag','yticklabel');
if iscell(yticklabels);yticklabels=cell2mat(yticklabels);end
set(yticklabels,'fontsize',handles.prefs.fonts.matrixsize);

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')

%select the subaxes in position (1,2) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%store handles to the data sets:
udata.view.mainhandles=h.main;
udata.view.xhair=h.xhair;
udata.view.yhair=h.yhair;

set(handles.fakeax(handles.activeindex),'userdata',udata);


%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

%update da handles
guidata(src,handles)

%shrink axes to encompass labels
raveresizeaxes(handles.fig,[],handles.activeindex)
handles=guidata(src);

%store axes labels so don't need to waste time get-ing them later:
ylabels=get(handles.subfakeax{handles.activeindex}(:,1),'ylabel');
if iscell(ylabels)
    ylabels=cell2mat(ylabels);
end
udata.view.ylabels=ylabels;
set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




