function ravecreateprefprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.fakeax(handles.activeindex),'userdata',udata);

targetaxes=handles.activegraph; %=handles.activegraph
ycol=udata.view.y;
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

xdials=handles.xdials{d}{a};

numy=length(ycol);
numx=length(xcol);
numpts=udata.view.numpts;

if numx==0 %this can happen when switching to a new yvar that shares no vars with previous xvar selection.
    numx=1;
    xcol=1;
end

if numy==0
    set(handles.infobar,'string','Error: No variables are modeled by Equations.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})

handles.subax{handles.activeindex}=[];
handles.subfakeax{handles.activeindex}=[];
handles.subfakerax{handles.activeindex}=[];
guidata(handles.fig,handles);


%get data
xnames=handles.allnames{d}(xcol);
ynames=handles.allnames{d}(ycol);

set(targetaxes,'Visible','off')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graphtitle,'fontsize',handles.prefs.fonts.graphtitlesize,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=25;
end

rows=numy;
cols=numx;

% Create and plot into small axes
ax = zeros(rows,cols);
fakeax = zeros(rows,cols);
fakerax = zeros(rows,cols);

pos = get(targetaxes,'Position');

hspace = handles.prefs.profilerhspacing;
vspace = handles.prefs.profilervspacing; % 5 percent space between subaxes

width = floor((pos(3)-hspace*numx)/numx);
height = floor((pos(4)-titoffset-vspace*numy)/numy);

xlim = zeros([numy numx 2]);
ylim = zeros([numy numx 2]);

%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end


%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);
ydatatypes=handles.datatype{d}(ycol);

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

pfixed=handles.preferences{d};

npts=udata.view.numpts;
nvars=numel(udata.view.x);

pgrid=repmat(pfixed,npts*numx,1);

for g=1:numx
    newpfs=linspace(0,1,npts);
    pgrid(npts*(g-1)+1:npts*g,xcol(g))=linspace(0,1,npts);
    
    for c=1:npts
        %normalize other prefs:
        index=1:length(pfixed)~=xcol(g);
        allprefs=pfixed; %if preference=0, keep it at zero.
        multiplier=(1-newpfs(c))./sum(allprefs(index));
        if isnan(multiplier) || isinf(multiplier);multiplier=0;end
        newprefs=pfixed(index).*multiplier;
        pgrid(npts*(g-1)+c,index)=newprefs;
    end
end

xfixed=xdials;
xdials=pfixed;
psteps=pgrid(:,udata.view.x);


%evaluate the functions:
yvalues=zeros(size(pgrid,1),numel(ycol));

for p=1:size(pgrid,1);
    handles.preferences{d}=pgrid(p,:);
    guidata(handles.fig,handles);
    yvalues(p,:)=raveevaluatefunction(src,d,udata.view.y,xfixed);
end

handles.preferences{d}=pfixed;
guidata(handles.fig,handles);

%update axis limits only if needed:
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

ycurrent=raveevaluatefunction(src,d,udata.view.y,xdials);

%put results of each response in a seperate cell:
psteps=mat2cell(psteps,repmat(npts,numx,1),size(psteps,2));
yvalues=mat2cell(yvalues,repmat(npts,nvars,1),ones(1,numy));
ycurrent=num2cell(ycurrent,1);

%and put each x for each resposne in its own cell too:
% for zz=1:numy
%     yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
% end

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

if udata.view.singleyaxis && udata.view.usevariablecolors
    %[linecolors]=ravemakelinecolors(handles.fig,[],V,2*J); %still need these markercolors
    linecolors=handles.variablecolors{d}(ycol,:);
elseif udata.view.singleyaxis
    [linecolors]=ravemakelinecolors(handles.fig,[],numy,1);
end

%% set up grid of axes.
for r=rows:-1:1,
    for c=cols:-1:1,
        axPos = single(floor([round(pos(1))+(c-1)*(width+hspace) pos(2)+(rows-r)*(height+vspace) ...
            width height]));
        
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,'color',bgcolor,...
            'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@raveupdateprefprofiler,...
            'userdata',handles.activeindex,'color','none',...
            'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %userdata contains the index of the bigax for this subgraph.
        set(ax(r,c),'visible','on');
        set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on');
        
        set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ticklength',[.02 .02])
        if strcmp(ydatatypes{r},'logical')
            set([ax(r,c) fakeax(r,c) fakerax(r,c)],'ytick',[0 1],'yticklabel',{handles.prefs.logicalfalse handles.prefs.logicaltrue})
        end
        
        thisxcolor=udata.format.haircolor;
        thisycolor=udata.format.haircolor;
        if udata.format.smarthaircolors
            if handles.isconstant{udata.view.dataset}(xcol(c))
                thisxcolor=get(handles.fakeax(handles.activeindex),'xcolor');
                set(ax(r,c),'buttondownfcn','') %this var is constant so shouldnt update profiler when clicking it.
            end
            
            if ~ismember(xcol(c),handles.funvars{udata.view.dataset}{ycol(r)})
                thisycolor=get(handles.fakeax(handles.activeindex),'xcolor');
            end
        end
        %tracer data series:
        h.xhair(r,c)=line([xdials(xcol(c)) xdials(xcol(c))],[0 1],'visible','on','linestyle',':','color',thisxcolor,...
            'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1);
        %response tracer series:

            h.yhair(r,c)=line([-10000000 1000000000],[ycurrent{r} ycurrent{r}],'visible','on','linestyle',':','color',thisycolor,...
                'parent',ax(r,c),'hittest','off','tag','hair','linewidth',1);

        %main data series:
            h.main(r,c)=line(psteps{c}(:,c),yvalues{c,r},'linewidth',udata.format.linesize,'linestyle','-','marker','none',...
                'color',udata.format.linecolor,'hittest','off','tag','keeper','parent',ax(r,c));
    end

    if handles.minlimit{d}{a}(ycol(r)) ~= handles.maxlimit{d}{a}(ycol(r))
        set(ax(r,c),'xlimmode','auto','ylim',[handles.minlimit{d}{a}(ycol(r)) handles.maxlimit{d}{a}(ycol(r))],'ylimmode','manual','xgrid','off','ygrid','off')
    else
        set(ax(r,c),'xlimmode','auto','ylim',[handles.minlimit{d}{a}(ycol(r)) handles.maxlimit{d}{a}(ycol(r))+0.001],'ylimmode','manual','xgrid','off','ygrid','off')
    end
    
    xlim(r,c,:) = get(ax(r,c),'xlim');
    ylim(r,c,:) = get(ax(r,c),'ylim');
end

%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;

%select the subaxes in position (1,2) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%store handles to the data sets:
% handles.profilers.main{handles.activeindex}=h.main;
% handles.profilers.xhair{handles.activeindex}=h.xhair;
% handles.profilers.yhair{handles.activeindex}=h.yhair;

udata.view.mainhandles=h.main;
udata.view.xhair=h.xhair;
udata.view.yhair=h.yhair;

set(handles.fakeax(handles.activeindex),'userdata',udata);

guidata(handles.fig,handles);

%make listeners:
delete(handles.axeslisteners{handles.activeindex});
handles.axeslisteners{handles.activeindex}=[];

%must store first listener outside of loop to initialize the variable type
%as "listener";

handles.axeslisteners{handles.activeindex}=addlistener(ax(1,1),'YTick','PreSet',@subaxeschangelistener);
for r=2:rows
    try %these dont work on 2007b,
        handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(r,1),'YTick','PreSet',@subaxeschangelistener);
    end
end

for c=1:cols
    try %these dont work on 2007b,
        handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(end,c),'XTick','PreSet',@subaxeschangelistener);
    end
end
guidata(handles.fig,handles);

% if numel(ycol)>1 && udata.view.singleyaxis
%     leg1=legend(handles.activegraph,h.main(:,1),handles.allnames{d}(ycol),'interpreter',handles.prefs.interpreter);
%     set(handles.fakerax(handles.activeindex),'userdata',leg1)
%     set(leg1,'color',[1 1 1])
% else
    leg1=[];
%end

if ~udata.view.showcurrentpoint
    set(h.yhair,'visible','off')
    set(h.xhair,'visible','off');
end

xlimmin = xmin(xcol);
xlimmax = xmax(xcol);

if udata.format.globallimits
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
else
    for r=1:numy;
        allyvalues=cell2mat(yvalues{r});
        ylimmin(r)=min(allyvalues);
        ylimmax(r)=max(allyvalues);
    end
end

for r=1:numy;
    if ylimmin(r)==ylimmax(r)
        ylimmin(r)=ylimmin(r)-0.01;
        ylimmax(r)=ylimmax(r)+0.01;
    end
end


for u=1:numx
    if xlimmin(u)==xlimmax(u)
        xlimmin(u)=xlimmin(u)-0.01;
        xlimmax(u)=xlimmax(u)+0.01;
    end
end

inset = .10;
%fix y limits
for r=1:numy,
    if ~udata.view.singleyaxis
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
        set(h.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
    else
        set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
        dy = diff(get(ax(r,1),'ylim'))*inset;
        set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
        set(h.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
    end
    
    %force non exponential labels:
    if ~strcmp(ydatatypes{r},'logical')
        set(ax(r,1),'YTickLabel',get(ax(r,1),'YTick'))
    end
    set(fakeax(r,1),'YTickLabel','')
    set(fakerax(r,1),'YTickLabel','')
    
    maxchars=floor(height./7);
    
    %make axes labels:
    if ~udata.view.singleyaxis && udata.view.showcurrentpoint %label and value
        set(get(ax(r,1),'ylabel'),'string',[linewrap(ynames{r},maxchars);{'';'';''}],'hittest','off','interpreter','none','ver','bottom',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(r,1),'ylabel'),'string',{['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(ycurrent{r},5)];'';''},...
            'interpreter','tex','hittest','off','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    elseif ~udata.view.showcurrentpoint && ~udata.view.singleyaxis
        set(get(ax(r,1),'ylabel'),'string',[linewrap(ynames{r},maxchars);{'';''}],'hittest','off','interpreter','none','vert','bottom',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(r,1),'ylabel'),'string',' ','interpreter','tex','hittest','off',...
            'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    else %nothing
        set(get(ax(r,1),'ylabel'),'string','','hittest','off','interpreter','none','vert','bottom');
        set(get(fakeax(r,1),'ylabel'),'string','','interpreter','tex','hittest','off');
    end
end
dx = zeros(1,cols);

%fix x limits
for c=1:numx,
    try %gives error if xlimmin==xlimmax
        set([ax(1,c) fakeax(1,c) fakerax(1,c)],'xlim',[0 1])
        dx(c) = diff(get(ax(1,c),'xlim'))*inset;
        set([ax(:,c) fakeax(:,c) fakerax(:,c)],'xlim',[0 1])
    end
    %force non exponential labels:
    set(ax(end,c),'XTickLabel',get(ax(end,c),'XTick'))
    set(fakeax(end,c),'XTickLabel','')
    set(fakerax(end,c),'XTickLabel','')
    
    %make axes labels:
    maxchars=floor(width./8);
    set(get(ax(1,c),'title'),'string',linewrap(xnames{c},maxchars),'interpreter','none');
    
    if udata.view.showcurrentpoint
        set(get(ax(end,c),'xlabel'),'string',[{''};{''};{''};linewrap(['P(',xnames{c},')'],maxchars)],'verticalalignment','top','hittest','off',...
            'interpreter','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',num2str(udata.format.linecolor),'} ',num2str(xdials(xcol(c)),5)]},...
            'verticalalignment','top','interpreter','tex','hittest','on','buttondownfcn',@prefprofilerchangevaltexts,...
            'userdata',handles.activeindex,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    else
        set(get(ax(end,c),'xlabel'),'string',[{''};{''};linewrap(xnames{c},maxchars)],'verticalalignment','top','hittest','off',...
            'interpreter','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
        set(get(fakeax(end,c),'xlabel'),'string',{' ';' ';' '},...
            'verticalalignment','top','interpreter','tex','hittest','on','buttondownfcn',@prefprofilerchangevaltexts,...
            'userdata',handles.activeindex,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize);
    end
    
    
end

set(ax(1:numy-1,:),'xticklabel','')
set(ax(:,2:numx),'yticklabel','')

set(fakeax(1:numy-1,:),'xticklabel','')
set(fakeax(:,2:numx),'yticklabel','')

set(fakerax(1:numy-1,:),'xticklabel','')
set(fakerax(:,2:numx),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','prefprofiler','tag','DerivativeProfilerBigAx');

set(ax,'xlimmode','manual','ylimmode','manual')

for c=1:numx 
    ravemaketextticks(ax(end,c),[],xcol(c),[],[]);
end


%also replace yticklabels with strings, this allows y (axis) labels to be aligned with each other.
if udata.view.singleyaxis
    yticks=get(ax(1,1),'ytick');
    yticklabels=num2cell(yticks);
    yticklabels=cellfun(@num2str,yticklabels,'UniformOutput',false);
    ravemaketextticks(ax(1,1),[],[],0,[],[],[],yticklabels);
else
    for r=1:rows
       ravemaketextticks(ax(r,1),[],[],ycol(r),[]); 
    end
end

%resize ticks:
%shrink tick labels:
xticklabels=findall(ax,'tag','xticklabel');
if iscell(xticklabels);xticklabels=cell2mat(xticklabels);end
set(xticklabels,'fontsize',handles.prefs.fonts.matrixsize);

%shrink tick labels:
yticklabels=findall(ax,'tag','yticklabel');
if iscell(yticklabels);yticklabels=cell2mat(yticklabels);end
set(yticklabels,'fontsize',handles.prefs.fonts.matrixsize);

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

%update da handles
guidata(src,handles)

%shrink axes to encompass labels
raveresizeaxes(handles.fig,[],handles.activeindex)

handles=guidata(src);

%store axes labels so don't need to waste time "get"ing them later:
ylabels=get(handles.subfakeax{handles.activeindex}(:,1),'ylabel');
if iscell(ylabels)
    ylabels=cell2mat(ylabels);
end
udata.view.ylabels=ylabels;
set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




