function raveupdateprefprofiler(src,eventdata)
%this is the buttondownfcn for derivative profiler axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

thistype=get(src,'tag');
if strcmp(thistype,'points');
    points=src;
    src=get(src,'parent');
else
    points=[];
end

ai=get(src,'userdata');
udata=get(handles.fakeax(ai),'userdata');


if strcmp(get(handles.fig,'selectiontype'),'alt') && udata.format.showpoints && ~udata.view.singleyaxis %ctrl click or right click = select point closest to clicked point.
    %need to check if this graph is active. if not , return and do nothing.
    thisindex=get(src,'userdata');
    if thisindex==handles.activeindex
        if strcmp(handles.activetab,'format')
            raveselectsubgraph(src,eventdata)
            return
        end
    end
    %
    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    
    cc=get(src,'currentpoint');
    xclick=cc(1,1);
    yclick=cc(1,2);
    
    try %gives error if you didnt click on a point (eg clicked on background)
        index=ravefindclosestpoint2D(xclick,yclick,points);
    catch
        return
    end
    
    %figure out
    
    oldselects=handles.isselected{udata.view.dataset}{udata.view.analysis};
    oldselects=false(size(oldselects));
    oldselects(index)=true;
    handles.isselected{udata.view.dataset}{udata.view.analysis}=oldselects;
    
    handles.xdials{udata.view.dataset}{udata.view.analysis}=handles.alldata{udata.view.dataset}(index,:);
    
    guidata(src,handles);
    
    raveupdateanalysis(src,[],'isselected',[]);
    raveupdateanalysis(src,[],'xdials',[]);
    return
    
else
    %if format tab is active, instead of doing this, make this graph the active subgraph to allow axes
    %changes and such:
    thisindex=get(src,'userdata');
    if thisindex~=handles.activeindex
        if strcmp(handles.activetab,'format')
            raveselectsubgraph(src,eventdata)
            return
        end
    end
    raveslideprofiler(handles.fig,[],ai);
    %make sliding along graph update continuously:
    set(handles.fig,'windowbuttonupfcn',{@raveprofdone,ai})
    set(handles.fig,'windowbuttonmotionfcn',{@raveslideprofiler,ai})
    set(handles.fig,'interruptible','off')
end


function raveslideprofiler(src,ev,ai)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

xmin=0;
xmax=1;

xcol=udata.view.x;
ycol=udata.view.y;

numx=numel(xcol);
numy=numel(ycol);

rows=numy;
cols=numx;

%find this plot's location in the matrix:
[crap,index]=find(handles.subax{ai}==gca);

if isempty(index);
    return
end

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
val=clicked(1,1);

%move marker within bounds
if val>xmax
    val=xmax;
elseif val<xmin
    val=xmin;
end

%if normalize preferences, adjust all other blobs too:
if handles.normalizepreferences{d}
    othervars=udata.view.x(udata.view.x~=udata.view.x(index));
    allprefs=handles.preferences{d}; %if preference=0, keep it at zero.
    multiplier=(1-val)./sum(allprefs(othervars));
    if isnan(multiplier) || isinf(multiplier);multiplier=0;end
    newprefs=handles.preferences{d}(udata.view.x).*multiplier;
    handles.preferences{d}(udata.view.x)=newprefs;
    handles.preferences{d}(udata.view.x(index))=val;
end

for c=1:numel(xcol)
    thisval=handles.preferences{d}(udata.view.x(c));
    valtext=strtrim(sprintf('%5.5g',thisval));
    set(udata.view.xhair(:,c),'xdata',[thisval thisval])
    set(get(handles.subfakeax{ai}(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',udata.format.linecolorString,'} ',valtext]},'verticalalignment','top','interpreter','tex');
end


preferences=handles.preferences{d};
preferences(xcol(index))=val;

handles.preferences{d}=preferences;
guidata(handles.fig,handles);


%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);
ydatatypes=handles.datatype{d}(ycol);

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

pfixed=handles.preferences{d};

npts=udata.view.numpts;
nvars=numel(udata.view.x);

pgrid=repmat(pfixed,npts*numx,1);

for g=1:numx
    newpfs=linspace(0,1,npts);
    pgrid(npts*(g-1)+1:npts*g,xcol(g))=linspace(0,1,npts);
    
    for c=1:npts
        %normalize other prefs:
        index=1:length(pfixed)~=xcol(g);
        allprefs=pfixed; %if preference=0, keep it at zero.
        multiplier=(1-newpfs(c))./sum(allprefs(index));
        if isnan(multiplier) || isinf(multiplier);multiplier=0;end
        newprefs=pfixed(index).*multiplier;
        pgrid(npts*(g-1)+c,index)=newprefs;
    end
end


xdials=handles.xdials{d}{a};
xfixed=xdials;
xdials=pfixed;
psteps=pgrid(:,udata.view.x);

%evaluate the functions:
yvalues=zeros(size(pgrid,1),numel(ycol));

for p=1:size(pgrid,1);
    handles.preferences{d}=pgrid(p,:);
    guidata(handles.fig,handles);
    yvalues(p,:)=raveevaluatefunction(src,d,udata.view.y,xfixed);
end

handles.preferences{d}=pfixed;
guidata(handles.fig,handles);

doupdate=0;
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

if doupdate;
    ax=handles.subax{ai};
    fakeax=handles.subfakeax{ai};
    fakerax=handles.subfakerax{ai};
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
    ydatatypes=handles.datatype{d}(ycol);
    inset = .06;
    
    %fix y limits
    for r=1:numel(ycol),
        if ~udata.view.singleyaxis
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
            set(udata.view.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
        else
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
            set(udata.view.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
        end
        
    end
    
    for c=1:numel(xcol)
        ylimits=get(ax(end,c),'Ylim');
        ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
        ticks=findall(ax(end,c),'tag','xticklabel');
        
        posses=get(ticks,'position');
        if iscell(posses);posses=cell2mat(posses);end
        posses(:,2)=ypos;
        posses=mat2cell(posses,ones(size(posses,1),1),3);
        set(ticks,{'position'},posses);
    end
    
    guidata(src,handles)
end

ycurrent=raveevaluatefunction(src,d,ycol,xdials);

%put results of each response in a seperate cell:
psteps=mat2cell(psteps,repmat(npts,nvars,1),size(psteps,2));
yvalues=mat2cell(yvalues,repmat(npts,nvars,1),ones(1,numy));
ycurrent=num2cell(ycurrent,1);
% 
% %and put each x for each resposne in its own cell too:
% for zz=1:numy
%     yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
% end

%update current-value texts for responses:
for r=1:numel(ycol)
    %set y label:
    g=udata.view.ylabels(r);
    if ~udata.view.singleyaxis
        set(g,'string',{['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{r}))];'';''});
    end

    for c=1:numel(xcol)
        
%         if ~udata.view.singleyaxis
            set(udata.view.yhair(r,c),'ydata',[ycurrent{r} ycurrent{r}])
            set(udata.view.mainhandles(r,c),'ydata',yvalues{c,r},'xdata',psteps{c}(:,c))
%         else
%             for R=1:numy
%                 set(handles.profilers.yhair{ai}(R,cc),'ydata',[ycurrent{R} ycurrent{R}])
%                 set(handles.profilers.main{ai}(R,cc),'ydata',yvalues{R}{cc},'xdata',xsteps{cc})
%             end
%         end
        
    end
end

if any(udata.view.constraints~=0)
    ravedrawconstraint(handles.allgraphs(ai),ai,udata.view.constraints);
end


%set xdials for rave wing demo:
%yvalues(p,:)=raveevaluatefunction(src,d,udata.view.y,xfixed);
newxdials=raveevaluatefunction(src,d,[9:13],xdials);


handles.xdials{1}{1}(1:5)=newxdials;
guidata(handles.fig,handles);

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdatedataset(src,[],'preferences',ai,d)
end



function raveprofdone(src,ev,ai) 

handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')
set(handles.fig,'interruptible','on')

%update other analyses to reflect new values:
raveupdatedataset(handles.fig,[],'preferences',ai)
