function handles=raveupdateprefprofilerpreferences(src,eventdata,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles

if nargin<3 %generally nargin==2 because this is called by raveudpatedataset, but when this is called by raveupdateprefprofilerxdials, nargin==3
    handles=guidata(src);
end

ai=handles.activeindex;
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

xmin=0;
xmax=1;

xcol=udata.view.x;
ycol=udata.view.y;

numx=numel(xcol);
numy=numel(ycol);

rows=numy;
cols=numx;

%% calculate profiler data for each graph:

%get type of each variable:
datatypes=handles.datatype{d}(xcol);
ydatatypes=handles.datatype{d}(ycol);

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

pfixed=handles.preferences{d};

npts=udata.view.numpts;
nvars=numel(udata.view.x);

pgrid=repmat(pfixed,npts*numx,1);

for g=1:numx
    newpfs=linspace(0,1,npts);
    pgrid(npts*(g-1)+1:npts*g,xcol(g))=linspace(0,1,npts);
    
    for c=1:npts
        %normalize other prefs:
        index=1:length(pfixed)~=xcol(g);
        allprefs=pfixed; %if preference=0, keep it at zero.
        multiplier=(1-newpfs(c))./sum(allprefs(index));
        if isnan(multiplier) || isinf(multiplier);multiplier=0;end
        newprefs=pfixed(index).*multiplier;
        pgrid(npts*(g-1)+c,index)=newprefs;
    end
end

xdials=handles.xdials{d}{a};
xfixed=xdials;
xdials=pfixed;
psteps=pgrid(:,udata.view.x);

%evaluate the functions:
yvalues=zeros(size(pgrid,1),numel(ycol));

for p=1:size(pgrid,1);
    handles.preferences{d}=pgrid(p,:);
    guidata(handles.fig,handles);
    yvalues(p,:)=raveevaluatefunction(src,d,udata.view.y,xfixed);
end

handles.preferences{d}=pfixed;
guidata(handles.fig,handles);

doupdate=0;
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

if doupdate;
    ax=handles.subax{ai};
    fakeax=handles.subfakeax{ai};
    fakerax=handles.subfakerax{ai};
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
    ydatatypes=handles.datatype{d}(ycol);
    inset = .06;
    
    %fix y limits
    for r=1:numel(ycol),
        if ~udata.view.singleyaxis
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
            set(udata.view.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
        else
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
            set(udata.view.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
        end
        
    end
    
    for c=1:numel(xcol)
        ylimits=get(ax(end,c),'Ylim');
        ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
        ticks=findall(ax(end,c),'tag','xticklabel');
        
        posses=get(ticks,'position');
        if iscell(posses);posses=cell2mat(posses);end
        posses(:,2)=ypos;
        posses=mat2cell(posses,ones(size(posses,1),1),3);
        set(ticks,{'position'},posses);
    end
    
    guidata(src,handles)
end

ycurrent=raveevaluatefunction(src,d,ycol,xdials);

%put results of each response in a seperate cell:
psteps=mat2cell(psteps,repmat(npts,nvars,1),size(psteps,2));
yvalues=mat2cell(yvalues,repmat(npts,nvars,1),ones(1,numy));
ycurrent=num2cell(ycurrent,1);
%
% %and put each x for each resposne in its own cell too:
% for zz=1:numy
%     yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
% end

%update current-value texts for responses:
for r=1:numel(ycol)
    %set y label:
    g=udata.view.ylabels(r);
    if ~udata.view.singleyaxis
        set(g,'string',{['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{r}))];'';''});
    end
    
    for c=1:numel(xcol)
        
        %         if ~udata.view.singleyaxis
        set(udata.view.xhair(r,c),'xdata',[xdials(xcol(c)) xdials(xcol(c))])
        set(udata.view.yhair(r,c),'ydata',[ycurrent{r} ycurrent{r}])
        set(udata.view.mainhandles(r,c),'ydata',yvalues{c,r},'xdata',psteps{c}(:,c))
        %         else
        %             for R=1:numy
        %                 set(handles.profilers.yhair{ai}(R,cc),'ydata',[ycurrent{R} ycurrent{R}])
        %                 set(handles.profilers.main{ai}(R,cc),'ydata',yvalues{R}{cc},'xdata',xsteps{cc})
        %             end
        %         end
        
    end
end


for c=1:numel(xcol)
    set(get(handles.subfakeax{ai}(end,c),'xlabel'),'string',{' ';' ';['\color[rgb]{',num2str(udata.format.linecolor),'} ',strtrim(sprintf('%5.5g',xdials(xcol(c))))]});
end

if any(udata.view.constraints~=0)
    ravedrawconstraint(handles.allgraphs(ai),ai,udata.view.constraints);
end


