function handles=raveupdatecontourxdials(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%handles=guidata(src);
getudata;
ai=handles.activeindex;
targetaxes=handles.allgraphs(ai);
xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;


%prepare data for gridding:
numpts=udata.view.smooth;
xdials=handles.xdials{d}{a};

set(udata.view.xhair,'xdata',[xdials(xcol) xdials(xcol)])
set(udata.view.yhair,'ydata',[xdials(ycol) xdials(ycol)])

xcol=[xcol ycol];

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(1,3);


thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,ycol)=ycalldata;

%evaluate the functions:

zvalues=raveevaluatefunction(src,d,udata.view.z,constants);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata{3};
zdataz=zdataz(:);

doupdate=0;
if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

% contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
contourplot(C,udata.view.mainhandles,'update');


%delete(handles.profilers.main{ai})
%mainhandles=contourplot(C,'style','line','parent',handles.allgraphs(ai),'linewidth',udata.format.linesize,'hittest','off');
% uistack(mainhandles,'bottom');

%uistack([handles.profilers.xhair{ai} handles.profilers.yhair{ai}],'top');
 
%handles.profilers.main{ai}=mainhandles;
if doupdate
    guidata(src,handles); %save now in case an error happens later.
end

%udpate colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[lowestvalue highestvalue];
    udata.view.clim=newlim;
    ylim=get(cbar,'ylim');
    if ~all(oldlim==newlim)
        raveupdatecolorbar(handles.fig,[],ai);
        legdata=get(handles.fakerax(ai),'userdata');
        cbar=findobj(legdata,'tag','Colorbar');
    end

    %determine current c value:
    thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});
    thiscoord=interp1(newlim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    if ~isempty(cvtext)
        pos=get(cvtext,'pos');
        set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
    end
end

%% draw constraints:
constraints=udata.view.constraints(udata.view.constraints~=0);

oldconstraints=findobj(handles.allgraphs(ai),'tag','constraint');
for i=1:numel(oldconstraints)
    ldata=get(oldconstraints(i),'userdata');
    delete(ldata.handles)
end

for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end

