function ravecreatehyperslice(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;

if numel(xcol)<2;set(handles.infobar,'You must select at least two variables for the contour plot matrix');return;end



a=udata.view.analysis;
d=udata.view.dataset; %data set index


%clear existing graph
%cla(targetaxes);

delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})

xnames=handles.allnames{d}(xcol);
ynames=handles.allnames{d}(ycol);

set(targetaxes,'Visible','off','color','none')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graphtitle,'fontsize',handles.prefs.fonts.graphtitlesize,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=10;
end


ax = zeros(numel(ycol),numel(xcol));
fakeax = zeros(numel(ycol),numel(xcol));
fakerax = zeros(numel(ycol),numel(xcol));

rows=numel(ycol);
cols=numel(xcol);

pos = get(targetaxes,'Position');%+[30 30 -45 -30]; %shrink a little to fit tick labels. (now handled by raveresizeaxes)

space=handles.prefs.matrixspacing; %pixels between subaxes;

width = floor((pos(3)-numel(xcol)*space)/numel(xcol));
height = floor((pos(4)-titoffset-numel(ycol)*space)/numel(ycol));

xlim = zeros([rows cols 2]);
ylim = zeros([rows cols 2]);

%format for fillstyle:

%initialize handles
h.main=zeros(rows,cols,10);

%prepare data for gridding:
datatypes=handles.datatype{d}(xcol);
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
xdials=handles.xdials{d}{a};

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);

constraints=udata.view.constraints(udata.view.constraints~=0);
if ~isempty(constraints)
    alltheconstraints=cell(numel(xcol),numel(xcol),numel(constraints));
end


for r=numel(ycol):-1:1,
    for c=numel(xcol):-1:1,
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};

        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);

        xcalldata=xgrid(:);
        ycalldata=ygrid(:);

        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;

        %evaluate the functions:

        zvalues=raveevaluatefunction(src,d,udata.view.z,constants);

        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

        for q=1:numel(constraints)
            constraintvalues=raveevaluateconstraint(handles,d,a,constraints(q),constants,false(1,size(constants,2)));
            alltheconstraints{r,c,q}=reshape(constraintvalues,numel(thisydata),numel(thisxdata));
        end

    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=zdataz(~cellfun(@isempty,zdataz));

lowestvalue=min(zdataz{1}(:));
highestvalue=max(zdataz{1}(:));
for b=2:numel(zdataz)
    lowestvalue=min(lowestvalue,min(zdataz{b}(:)));
    highestvalue=max(highestvalue,max(zdataz{b}(:)));
end

%check against existing limits:
if lowestvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=lowestvalue;end
if highestvalue>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=highestvalue;end
guidata(src,handles);

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

udata.view.clim=[lowestvalue highestvalue];
set(handles.fakeax(handles.activeindex),'userdata',udata);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end


%set up grid of axes.
for r=numel(ycol):-1:1
    for c=numel(xcol):-1:1
        axPos = [pos(1)+(c-1)*(width+space) pos(2)+(rows-r)*(height+space) ...
            width height];

        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,'color',bgcolor,...
            'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,'visible','on');
        fakeax(r,c) = axes('units','pixels','Position',axPos+[5 10 -10 -20],'parent',parent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid,'visible','off');
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@raveupdatehyperslice,...
            'userdata',handles.activeindex,'color','none','drawmode','normal',...
            'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor,...
            'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]); %userdata contains the index of the bigax for this subgraph.

        if r==c %make prediction profiler plot:
            thisxdata=xsteps{c};
            constants=repmat(xdials,[length(thisxdata),1]);
            constants(:,xcol(c))=thisxdata;
            vvalues=raveevaluatefunction(src,d,udata.view.z,constants);
            %normalize vvalues to fit scale of box:
            vvalues=((vvalues-lowestvalue)./(highestvalue-lowestvalue))*(max(xsteps{r})-min(xsteps{r}))+min(xsteps{r});
            
            set(fakerax(r,c),'visible','off') %don't show grid lines

            set(ax(r,c),'xlim',single([xmin(xcol(c)) xmax(xcol(c))]),'ylim',single([xmin(ycol(r)) xmax(ycol(r))]),'hittest','off')

            line(thisxdata,vvalues,'parent',fakeax(r,c),'tag','junkk')
        else
            zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            % contourf(ax(r,c),single(allthedata{r,c,1}),single(allthedata{r,c,2}),single(zplotdata),single(levelz),'linecolor','none');

            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            
            %make empty lines:
            for i=numel(levelz):-1:1
                mains(i)=line(nan,nan,'parent',ax(r,c),'linewidth',udata.format.linesize,'hittest','off');
            end
            mainhandles{r,c}=mains;      
            contourplot(C,mains,'update');

            %draw constraints:
            for q=1:numel(constraints)
                constraintdata=alltheconstraints{r,c,q};
                if ~udata.format.reversecolors
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                else
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=128-(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                end

                C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),constraintdata,[clevelz clevelz]);
                ravedrawconstraintcore(ax(r,c),handles.activeindex,C,constraints(q),xcol(c),xcol(r),1);
            end


            %use single precision axes limits for correct positioning on screen:
            xlimz=single([allthedata{r,c,1}(1,1) allthedata{r,c,1}(1,end)]);
            ylimz=single([allthedata{r,c,2}(1,1) allthedata{r,c,2}(end,1)]);

            set(ax(r,c),'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual','xgrid','off','ygrid','off','layer','top')

            %make tracers
            if udata.format.usevariablecolors
                L1=line([xdials(xcol(c)) xdials(xcol(c))],[xmin(xcol(r)) xmax(xcol(r))],...
                    'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,...
                    'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(c),:));
                L2=line([xmin(xcol(c)) xmax(xcol(c))],[xdials(xcol(r)) xdials(xcol(r))],...
                    'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,...
                    'linewidth',1+udata.format.bold,'color',handles.variablecolors{d}(xcol(r),:));
            else
                L1=line([xdials(xcol(c)) xdials(xcol(c))],[xmin(xcol(r)) xmax(xcol(r))],...
                    'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,...
                    'linewidth',1+udata.format.bold,'color',udata.format.tracercolor);
                L2=line([xmin(xcol(c)) xmax(xcol(c))],[xdials(xcol(r)) xdials(xcol(r))],...
                    'parent',ax(r,c),'hittest','off','linestyle',udata.format.linestyle,...
                    'linewidth',1+udata.format.bold,'color',udata.format.tracercolor);

            end

            xtracerhandles{r,c}=L1;
            ytracerhandles{r,c}=L2;

        end

        xlim(r,c,:) = get(ax(r,c),'xlim');
        ylim(r,c,:) = get(ax(r,c),'ylim');
    end
end

clear contourplot %clear persistent vars;

udata.view.mainhandles=mainhandles;
udata.view.xhair=xtracerhandles;
udata.view.yhair=ytracerhandles;

set(handles.fakeax(handles.activeindex),'userdata',udata);



set(fakeax(:),{'xlim'},get(ax,'xlim'),{'ylim'},get(ax,'ylim'));

set(ax(1,1),'xcolor',[1 1 1],'zcolor',[0 0 0],'box','off')
set(ax(numel(xcol),numel(xcol)),'ycolor',[1 1 1],'zcolor',[0 0 0],'box','off')
for c=2:numel(xcol)-1
    set(ax(c,c),'visible','off')
end


%% fix axes limits:

for r=1:rows,
    %force non exponential labels:
    set(ax(r,1),'YTickLabel',get(ax(r,1),'YTick'))
    set(fakeax(r,1),'YTickLabel','')
    set(fakerax(r,1),'YTickLabel','')
end


for c=1:cols,
    %force non exponential labels:
    set(ax(end,c),'XTickLabel',get(ax(end,c),'XTick'))
    set(fakeax(end,c),'XTickLabel','')
    set(fakerax(end,c),'XTickLabel','')
end

set(ax(1:rows-1,:),'xticklabel','')
set(ax(:,2:cols),'yticklabel','')

set(fakeax(1:rows-1,:),'xticklabel','')
set(fakeax(:,2:cols),'yticklabel','')

set(fakerax(1:rows-1,:),'xticklabel','')
set(fakerax(:,2:cols),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','hyperslice','tag','');

set(ax,'xlimmode','manual','ylimmode','manual')

%% fix tick labels:



for c=1:numel(xcol)
    switch handles.datatype{d}{xcol(c)}
        case {'continuous','discrete','integer'}
            
            ticks=get(ax(end,c),'XTick');
            omag=round(log10(abs(ticks)));
            %set really small numbers=0
            ticks(omag<-15)=0;
            omag(omag<-15)=0;
            %only reformat if ANY omag is 10^3 or greater.
            % if sum(omag>2)>=(numel(omag)-1)
            if any(omag>2)
%                 ticks=ticks/10^max(omag);
                if strcmp(handles.prefs.interpreter,'tex')
                    xlegendmultipliertext{c} = [' x10^{',num2str(-max(omag)),'}'];
                else
                    xlegendmultipliertext{c} = [' x10^',num2str(-max(omag))];
                end
            elseif  sum(omag<-2)>=(numel(omag)-1)
                if strcmp(handles.prefs.interpreter,'tex')
                    xlegendmultipliertext{c} = [' x10^{',num2str(-min(omag)),'}'];
                else
                    xlegendmultipliertext{c} = [' x10^',num2str(-min(omag))];
                end
            else %no scaling necessary
                xlegendmultipliertext{c} ='';
            end
        otherwise
            xlegendmultipliertext{c}='';
    end
    
    switch udata.view.layout
        case {'square','rectangle','lower'}
            ravemaketextticks(ax(end,c),[],xcol(c),[],[],[])
        case 'upper'
            if c>1
                ravemaketextticks(ax(c,c),[],xcol(c),[],[],[])
            end
    end
end

%shrink tick labels:
xticklabels=findall(ax,'tag','xticklabel');
if iscell(xticklabels);xticklabels=cell2mat(xticklabels);end
fs=round(0.8*handles.prefs.fonts.graphsize);
fs=max(fs,6);
set(xticklabels,'fontsize',fs);


% Y TICKS:
ylegendmultipliertext=cell(1,numel(ycol)); %first row is xcol, 2nd is ycol.

for r=1:numel(ycol)
    ticks=get(ax(r,1),'YTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    omag(omag<-15)=0;
    %only reformat if all omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
    if any(omag>2)
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{r} = [' x10^{',num2str(-max(omag)),'}'];
        else
            ylegendmultipliertext{r} = [' x10^',num2str(-max(omag))];
        end
    elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{r} = [' x10^{',num2str(-min(omag)),'}'];
        else
            ylegendmultipliertext{r} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        ylegendmultipliertext{r} ='';
    end

    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            ravemaketextticks(ax(r,1),[],[],ycol(r),[],[])
        otherwise %upper
            if r<numel(ycol)
                ravemaketextticks(ax(r,r+1),[],[],ycol(r),[],[])
            end
    end
end

%shrink tick labels:
yticklabels=findall(ax,'tag','yticklabel');
if iscell(yticklabels);yticklabels=cell2mat(yticklabels);end
fs=round(0.8*handles.prefs.fonts.graphsize);
fs=max(fs,6);
set(yticklabels,'fontsize',fs);




%{
rotationz=90;
halign='right';
valign='default';


% X TICKS:
xlegendmultipliertext=cell(1,numel(xcol)); %first row is xcol, 2nd is ycol.

for q=1:numel(xcol)
    ticks=get(ax(end,q),'XTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if ANY omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)
    if any(omag>2)
        omag=max(omag);
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif  sum(omag<-2)>=(numel(omag)-1)
        omag=min(omag);
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            xlegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            xlegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        xlegendmultipliertext{q} ='';
        omag=0;
    end

    set(ax(end,q),'XTickLabel',[]);
    set(fakeax(end,q),'XTickLabel',[]);
    set(fakerax(end,q),'XTickLabel',[]);

    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            switch handles.datatype{d}{xcol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(end,q),'xtick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)

                        if ticks(w)*10^omag > (xlim(end,q,2)-0.1*(xlim(end,q,2)-xlim(end,q,1))) %within 10% of endpoint...
                            usethesevalues{w}='';
                        else
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end



                    end

                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end

                    else %too many, just treat as continuous
                        putthemhere=get(ax(end,q),'xtick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));

                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end


            ylimits=get(ax(end,1),'Ylim');
            ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));

            th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
                'HorizontalAlignment',halign,'rotation',rotationz,'parent',ax(end,q),'hittest','off',...
                'verticalalignment',valign,'handlevisibility','off','tag','xticklabel',...
                'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph); %#ok<NASGU>

        otherwise %upper
            switch handles.datatype{d}{xcol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(1,q),'xtick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end

                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end

                    else %too many, just treat as continuous
                        putthemhere=get(ax(1,q),'xtick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));

                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end


            ylimits=get(ax(1,1),'Ylim');
            ypos=ylimits(2)+.02*(ylimits(2)-ylimits(1));

            th=text(putthemhere,ypos*ones(1,length(putthemhere)),usethesevalues,...
                'HorizontalAlignment','left','rotation',rotationz,'parent',ax(1,q),'hittest','off',...
                'verticalalignment',valign,'handlevisibility','off','tag','xticklabel',...
                'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph); %#ok<NASGU>
    end
end

% Y TICKS:
ylegendmultipliertext=cell(1,numel(ycol)); %first row is xcol, 2nd is ycol.

for q=1:numel(ycol)
    ticks=get(ax(q,1),'YTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    ticks(omag<-15)=0;
    omag(omag<-15)=0;
    %only reformat if all omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
    if any(omag>2)
        omag=max(omag);
        ticks=ticks/10^max(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-max(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-max(omag))];
        end
    elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
        omag=min(omag);
        ticks=ticks/10^min(omag);
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{q} = [' x10^{',num2str(-min(omag)),'}'];
        else
            ylegendmultipliertext{q} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        omag=0;
        ylegendmultipliertext{q} ='';
    end
    set(ax(q,1),'YTickLabel',[]);
    set(fakeax(q,1),'YTickLabel',[]);
    set(fakerax(q,1),'YTickLabel',[]);

    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            switch handles.datatype{d}{xcol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(q,1),'ytick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        if ticks(w)*10^omag > (ylim(q,1,2)-0.1*(ylim(q,1,2)-ylim(q,1,1))) %within 10% of endpoint...
                            usethesevalues{w}='';
                        else
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end

                    end
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                    else %too many, just treat as continuous
                        putthemhere=get(ax(q,1),'ytick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));

                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end

            xlimits=get(ax(q,1),'xlim');
            xpos=xlimits(1)-.02*(xlimits(2)-xlimits(1));

            th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
                'HorizontalAlignment','right','rotation',0,'parent',ax(q,1),'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag','yticklabel',...
                'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph); %#ok<NASGU>

        otherwise %upper
            switch handles.datatype{d}{xcol(q)}
                case {'continuous','constant','integer'}
                    putthemhere=get(ax(q,end),'ytick');
                    usethesevalues=cell(numel(ticks),1);
                    for w=1:numel(ticks)
                        usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                    end
                case 'discrete'
                    discretez=handles.discretevalues{d}{a}{xcol(q)};
                    if numel(discretez)<=12;
                        putthemhere=discretez;
                        usethesevalues=cell(numel(discretez),1);
                        for w=1:numel(discretez)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',discretez(w)));
                        end
                    else %too many, just treat as continuous
                        putthemhere=get(ax(q,end),'ytick');
                        usethesevalues=cell(numel(ticks),1);
                        for w=1:numel(ticks)
                            usethesevalues{w}=strtrim(sprintf('%-6.4g',ticks(w)));
                        end
                    end
                case 'string'
                    usethesevalues=handles.discretevalues{d}{a}{xcol(q)};
                    putthemhere=unique(handles.alldata{d}(:,xcol(q)));

                case 'logical'
                    usethesevalues={handles.prefs.logicalfalse;handles.prefs.logicaltrue};
                    putthemhere=[0 1];
            end

            xlimits=get(ax(end,end),'xlim');
            xpos=xlimits(2)+.02*(xlimits(2)-xlimits(1));

            th=text(xpos*ones(1,length(putthemhere)),putthemhere,usethesevalues,...
                'HorizontalAlignment','left','rotation',0,'parent',ax(q,end),'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag','yticklabel',...
                'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph); %#ok<NASGU>

    end
end



%}

%% make variable name labels:

switch udata.view.layout
    case {'square';'upper';'lower'}
        for k=1:numel(udata.view.x)
            %            set(h.main(k,k,:),'visible','off','tag','nevershow');
            %            set(selected(k,k),'visible','off','tag','nevershow');
            lims=get(ax(k,k),'xlim');
            if ~isempty(xlegendmultipliertext{k})
                tt(k)=text(mean(lims),mean(lims),{xnames{k};xlegendmultipliertext{k}},...
                    'parent',ax(k,k),...
                    'horizontalAlignment','center','verticalAlignment','bottom','interpreter',handles.prefs.interpreter,...
                    'backgroundcolor',get(ax(1),'color'),'tag','axeslabel',...
                    'fontsize',handles.prefs.fonts.graphsize+1,'fontname',handles.prefs.fonts.graph);
            else %avoid extra white space that obscures profiler
                tt(k)=text(mean(lims),mean(lims),xnames{k},...
                    'parent',ax(k,k),...
                    'horizontalAlignment','center','verticalAlignment','bottom','interpreter',handles.prefs.interpreter,...
                    'backgroundcolor','none','tag','axeslabel','fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph);
            end

        end
    otherwise %rectangle
        %make x/y labels on ax(r,c)
        for k=1:numel(udata.view.x)
            set(get(ax(end,k),'xlabel'),'string',{' ';xnames{k}},'interpreter',handles.prefs.interpreter,...
                'verticalalignment','top');
        end

        for k=1:numel(udata.view.y)
            set(get(ax(k,1),'ylabel'),'string',{ynames{k};' '},'interpreter',handles.prefs.interpreter,...
                'verticalalignment','bottom');
        end


end

%% turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;


%select the subaxes in position (1,2) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1,2);

%store handles to the data sets:

%resize axes so axes labels/ticks fit inside the bounding box.

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end


%for upper/lower triangles, hide appropriate axes/data
switch udata.view.layout
    case 'upper'
        hidefilter=logical(tril(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
    case 'lower'
        hidefilter=logical(triu(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
end

%update da handles
guidata(src,handles)
set(handles.fakeax(handles.activeindex),'userdata',udata);

drawnow;

%fix colorbar:
if udata.format.colorbar
    ravemakecolorbar(handles.fig,[]);
end

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
