function handles=raveupdatehyperslicexdials(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%handles=guidata(src);
getudata;
ai=handles.activeindex;
xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;


%prepare data for gridding:
numpts=udata.view.smooth;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
xdials=handles.xdials{d}{a};

[xsteps totalrows]=ravecalculatexsteps(handles.fig,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(numel(xcol),numel(xcol),3);

constraints=udata.view.constraints(udata.view.constraints~=0);
if ~isempty(constraints)
    alltheconstraints=cell(numel(xcol),numel(xcol),numel(constraints));
end

for r=numel(ycol):-1:1,
    for c=numel(ycol):-1:1,
        if r==c;continue;end

        thisxdata=xsteps{c};
        thisydata=xsteps{r};
        
        [xgrid,ygrid]=meshgrid(thisxdata,thisydata);
        
        xcalldata=xgrid(:);
        ycalldata=ygrid(:);
        
        constants=repmat(xdials,[length(xcalldata),1]);
        constants(:,xcol(c))=xcalldata;
        constants(:,xcol(r))=ycalldata;
        
        %evaluate the functions:
        
        zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);
        
        allthedata{r,c,1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
        allthedata{r,c,3}=reshape(zvalues,numel(thisydata),numel(thisxdata));
        
        for q=1:numel(constraints)
            constraintvalues=raveevaluateconstraint(handles,d,a,constraints(q),constants,false(1,size(constants,2)));
            alltheconstraints{r,c,q}=reshape(constraintvalues,numel(thisydata),numel(thisxdata));
        end
    end
end

zdataz=allthedata(:,:,3);
zdataz=zdataz(:);
zdataz=zdataz(~cellfun(@isempty,zdataz));

lowestvalue=min(zdataz{1}(:));
highestvalue=max(zdataz{1}(:));
for b=2:numel(zdataz)
    lowestvalue=min(lowestvalue,min(zdataz{b}(:)));
    highestvalue=max(highestvalue,max(zdataz{b}(:)));
end

%check against existing limits:
doupdate=false;
if lowestvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=lowestvalue;doupdate=true;end
if highestvalue>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=highestvalue;doupdate=true;end
if doupdate
    guidata(src,handles);
end
lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

udata.view.clim=[lowestvalue highestvalue];
set(handles.fakeax(handles.activeindex),'userdata',udata);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

ax=handles.subax{handles.activeindex};
fakeax=handles.subfakeax{ai};

for r=numel(xcol):-1:1
    for c=numel(xcol):-1:1
         if r==c %make prediction profiler plot:
            thisxdata=xsteps{c};
            constants=repmat(xdials,[length(thisxdata),1]);
            constants(:,xcol(c))=thisxdata;
            vvalues=raveevaluatefunction(src,d,udata.view.z,constants);
            %normalize vvalues to fit scale of box:
            vvalues=((vvalues-lowestvalue)./(highestvalue-lowestvalue))*(max(xsteps{r})-min(xsteps{r}))+min(xsteps{r});

            delete(findobj(fakeax(r,c),'tag','junkk'));
            line(thisxdata,vvalues,'parent',fakeax(r,c),'tag','junkk')
         else

            zplotdata=allthedata{r,c,3};
            if ~udata.format.reversecolors
                zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            else
                zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
            end

            C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),zplotdata,levelz);
            h=udata.view.mainhandles{r,c};
            contourplot(C,h,'update');
            
%             delete(handles.profilers.main{ai}{r,c});
%             mainhandles{r,c}=contourplot(C,'style','line','parent',ax(r,c),'linewidth',udata.format.linesize,'hittest','off');

            %draw constraints:
            for q=1:numel(constraints)
                constraintdata=alltheconstraints{r,c,q};
                if ~udata.format.reversecolors
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                else
                    constraintdata=(constraintdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                    clevelz=128-(0-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
                end

                C=contourc(allthedata{r,c,1}(1,:),allthedata{r,c,2}(:,1),constraintdata,[clevelz clevelz]);
                ravedrawconstraintcore(ax(r,c),handles.activeindex,C,constraints(q),xcol(c),xcol(r),1);
            end


            %use single precision axes limits for correct positioning on screen:
            xlimz=single([allthedata{r,c,1}(1,1) allthedata{r,c,1}(1,end)]);
            ylimz=single([allthedata{r,c,2}(1,1) allthedata{r,c,2}(end,1)]);

            set(ax(r,c),'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual','xgrid','off','ygrid','off','layer','top')

            %update tracers
            
            set(udata.view.xhair{r,c},'xdata',[xdials(xcol(c)) xdials(xcol(c))],'ydata',[xmin(xcol(r)) xmax(xcol(r))])
            set(udata.view.yhair{r,c},'xdata',[xmin(xcol(c)) xmax(xcol(c))],'ydata',[xdials(xcol(r)) xdials(xcol(r))])

         end
    end
end

clear contourplot %clear persistent vars;

%handles.profilers.main{ai}=mainhandles;
guidata(src,handles); %save now in case an error happens later.

%udpate colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[lowestvalue highestvalue];
    udata.view.clim=newlim;
    ylim=get(cbar,'ylim');
    if doupdate
        ravemakecolorbar(handles.fig,[],ai);
        legdata=get(handles.fakerax(ai),'userdata');
        cbar=findobj(legdata,'tag','Colorbar');
    end

    %determine current c value:
    thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis});
    
    thiscoord=interp1(newlim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
end

