function handles=raveupdatescattergrad3Dvisiblerows(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;
scol=udata.view.s;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);
z=handles.alldata{d}(:,zcol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};


%% get constantvalues:
% if color/sizecol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the column.

if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
    end
else
    %c=zeros(length(x),1);
    if udata.format.ncolor
        c=1920+(handles.rowcolors{d}{a})+ rand -.5; %noisy colors
    else
        c=1920+(handles.rowcolors{d}{a}); %non-noisy colors cdata=0.5
    end
end

%% normalize sdata to fit a reasonable range of marker sizes:
if scol~=0
    s=handles.alldata{d}(:,scol);
    minsize=udata.format.minmarkersize;
    maxsize=udata.format.maxmarkersize;
    if min(s)~=max(s) %avoid divide by zero
        s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;
        s=round(s)+1;
    else
        s=minsize.*ones(size(s));
    end
else
    s=ones(length(x),1)*udata.format.minmarkersize;
end

s=sqrt(s); %for consistency with other matlab functions


%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);

%make the patches:
patchidx=udata.view.patchidx;
filtpatchhandles=udata.view.filthandles;
mainpatchhandles=udata.view.mainhandles;
selpatchhandles=udata.view.selhandles;

%filtered handles
if ~udata.view.uselines
    for j=1:npatches
        filtidx=patchidx==j & nothidden;
        if any(filtidx)
            set(filtpatchhandles(j),'xdata',x(filtidx),'ydata',y(filtidx),'zdata',z(filtidx),'visible','on');
        else
            set(filtpatchhandles(j),'xdata',x(1),'ydata',y(1),'zdata',z(1),'visible','off');
        end
    end
else
    if any(nothidden)
        set(filtpatchhandles,'xdata',x(nothidden),'ydata',y(nothidden),'zdata',z(nothidden),'visible','on');
    else
        set(filtpatchhandles,'xdata',x(1),'ydata',y(1),'zdata',z(1),'visible','off');
    end
end

%main handles
if ~udata.view.uselines
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            set(mainpatchhandles(j),'xdata',x(mainidx),'ydata',y(mainidx),'zdata',z(mainidx),'cdata',c(mainidx),'visible','on');
        else
            set(mainpatchhandles(j),'xdata',x(1),'ydata',y(1),'zdata',z(1),'cdata',c(1),'visible','off');
        end
    end
else
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            set(mainpatchhandles(j),'xdata',x(mainidx),'ydata',y(mainidx),'zdata',z(maindix),'visible','on');
        else
            set(mainpatchhandles(j),'xdata',x(1),'ydata',y(1),'zdata',z(1),'visible','off');
        end
    end
end

%selected handles
if ~udata.view.uselines
    for j=1:npatches
        selidx=patchidx==j & isselected & nothidden;
        if any(selidx)
            set(selpatchhandles(j),'xdata',x(selidx),'ydata',y(selidx),'zdata',z(selidx),'visible','on');
        else
            set(selpatchhandles(j),'xdata',x(1),'ydata',y(1),'zdata',z(1),'visible','off');
        end
    end
else
    selidx=isselected & nothidden;
    if any(selidx)
        set(selpatchhandles,'xdata',x(selidx),'ydata',y(selidx),'zdata',z(selidx),'visible','on');
    else
        set(selpatchhandles,'xdata',x(1),'ydata',y(1),'zdata',z(1),'visible','off');
    end
end
