function ravecreatesliders(src,ev)
%create xdials sliders on workspace.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph; %=handles.activegraph
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');
try %#ok<TRYNC>
    delete(udata.view.scroll);
end

%% make magic box:

allnames=handles.allnames{d};

varnames=allnames(xcol);
ranges=[handles.minlimit{d}{a}(xcol)' handles.maxlimit{d}{a}(xcol)'];
xdials=handles.xdials{d}{a}(xcol);
datatypes=handles.datatype{d}(xcol);
%raveresizeaxes(src,[],handles.activeindex)
axpos=get(targetaxes,'position');
nvars=numel(varnames);

rpix=14.7; %pixels per row of text. 
tpix=rpix*(nvars+1); %total number of pixels needed to display all vars.
dpix = axpos(4); %number of pixels visible at any time. 

ndisp=floor(axpos(4)/rpix); %number of rows of text that will fit in the box

set(targetaxes,'units',get(handles.fig,'units'),'position',axpos,...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[1 ndisp*rpix],...
    'xtickmode','manual','ytickmode','manual','xtick',[],'ytick',[],'box','on',...
    'xcolor',[.7 .7 .7 ],'ycolor',[.7 .7 .7]);

set(targetaxes,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')
set(handles.fakerax(handles.activeindex),'visible','off')

if udata.format.usevariablecolors
cmap=handles.variablecolors{d}(xcol,:);
else
   cmap=repmat(udata.format.facecolor,[numel(xcol) 1]) ;
    
end
%initialize variables: this must be done to avoid errors.
L=zeros(1,nvars);
udata.view.blobs=L;
udata.view.valtexts=L;
udata.view.vartexts=L;

%normalize xdials to range used in magicbox:
startvalues=(xdials-ranges(:,1)')./(ranges(:,2)'-ranges(:,1)')*.3+.43;
startvalues(isinf(startvalues))=0.58; %mid range.
startvalues(isnan(startvalues))=0.58;

%make box:
downer=rpix/2;

for t=1:nvars
    thisvar=xcol(end-t+1);
    thistype=datatypes{end-t+1};
    val=xdials(end-t+1);
    
    %make current value texts:
    if ~any(strcmp(thistype,{'string','logical'}))
        valtext=strtrim(sprintf('%5.5g',val));
    elseif strcmp(thistype,'string')
        valtext=handles.discretevalues{d}{a}{thisvar}{val};
    else %logical
        if val
            valtext=handles.prefs.logicaltrue;
        else
            valtext=handles.prefs.logicalfalse;
        end
    end
    
    oor=false; %oor=out of range
    if startvalues(end-t+1)<.43
        startvalues(end-t+1)=.43;
        oor=true;
    elseif startvalues(end-t+1)>.73
        startvalues(end-t+1)=.73;
        oor=true;
    end

    if oor
        startcolor=[1 0 0];
    else
        startcolor=[0 0 0];
    end
 
     udata.view.vartexts(end-t+1)=text(0.38,t/ndisp*dpix,varnames{end-t+1},'horizontalalignment','right','clipping','on','verticalalignment','top','parent',targetaxes,...
         'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph,'interpreter',handles.prefs.interpreter);

     L(t)=line([0.43 .505 .580 .655 .730],repmat(t/ndisp*dpix-downer,[1 5]),'linewidth',1,'color',[0 0 0],'marker','+','parent',targetaxes);
    
    udata.view.blobs(end-t+1)=line(startvalues(end-t+1),t/ndisp*dpix-downer,'linestyle','none','markerfacecolor',cmap(end-t+1,:),'marker','o',...
        'markeredgecolor',[0 0 0],'buttondownfcn',@slidersclick,'userdata',thisvar,'parent',targetaxes);
     
     udata.view.valtexts(end-t+1)=text(0.78,t/ndisp*dpix,valtext,...
         'horizontalalignment','left','clipping','on','verticalalignment','top','userdata',thisvar,...
         'buttondownfcn',@sliderschangevaltexts,'hittest','on','parent',targetaxes,'color',startcolor,...
         'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph,'interpreter',handles.prefs.interpreter);

end

set(targetaxes,'ylim',[tpix-dpix tpix]);

%% make slider:
slpos=[axpos(1)+axpos(3) axpos(2)-1 17 axpos(4)+1];
vscroll=drawvscroll(handles.wspanel,targetaxes,slpos,0,tpix,dpix,tpix,@sliderscroll,1);
udata.view.scroll=vscroll;
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravearrangeobjectssliders(handles.fig,[])

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects

