function ravecreatescatter(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

cla(targetaxes);
legend(targetaxes,'off');

if ~udata.format.persistentlimits
    set(targetaxes,'xlimmode','auto','ylimmode','auto','xtickmode','auto','ytickmode','auto')
else
    set(targetaxes,'xtickmode','auto','ytickmode','auto')
end

%font formatting
fweight='normal';

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

%% get constantvalues:
% if color/sizecol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the column.
if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
    end
else
    %c=zeros(length(x),1);
    if udata.format.ncolor
        c=1920+(handles.rowcolors{d}{a})+ rand -.5; %noisy colors
    else
        c=1920+(handles.rowcolors{d}{a}); %non-noisy colors cdata=0.5
    end
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%set(handles.activegraph,'drawmode','fast')

%% normalize sdata to fit a reasonable range of marker sizes:
if scol~=0
    s=handles.alldata{d}(:,scol);
    minsize=udata.format.minmarkersize;
    maxsize=udata.format.maxmarkersize;
    if min(s)~=max(s) %avoid divide by zero
        s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;
        s=round(s)+1;
    else
        s=minsize.*ones(size(s));
    end
else
    s=ones(length(x),1)*udata.format.minmarkersize;
end

s=sqrt(s); %for consistency with other matlab functions


%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);

%if patches are overkill, use lines:
patchidx=zeros(size(s,1),1);

if npatches==1 && ccol==0
    npatches=10;
    uselines=true;
    patchidx=handles.rowcolors{d}{a};
else
    uselines=false;
    for g=1:size(s,1)
        patchidx(g)=find(usdata==s(g),1,'first');
    end
end

%make the patches:
filtpatchhandles=zeros(npatches,1);
mainpatchhandles=zeros(npatches,1);
selpatchhandles=zeros(npatches,1);
%filtered handles
if ~uselines
    for j=1:npatches
        filtidx=patchidx==j & nothidden;
        if any(filtidx)
            filtpatchhandles(j)=line(x(filtidx),y(filtidx),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
        else
            filtpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
        end
    end
else
    if any(nothidden)
        filtpatchhandles=line(x(nothidden),y(nothidden),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','on','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
    else
        filtpatchhandles=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
    end
end

%main handles
if ~uselines
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            mainpatchhandles(j)=patch(x(mainidx),y(mainidx),c(mainidx),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
                'buttondownfcn',@pointselectscatter,'linestyle','none');
        else
            mainpatchhandles(j)=patch(x(1),y(1),c(1),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
                'buttondownfcn',@pointselectscatter,'linestyle','none');
        end
    end
else
    colors=handles.prefs.rowcolors;
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            mainpatchhandles(j)=line(x(mainidx),y(mainidx),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                'markeredgecolor','none','hittest','off',...
                'buttondownfcn',@pointselectscatter,'linestyle','none');
        else
            mainpatchhandles(j)=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                'markeredgecolor','none','visible','off','hittest','off',...
                'buttondownfcn',@pointselectscatter,'linestyle','none');
        end
    end
end

%selected handles
if ~uselines
    for j=1:npatches
        selidx=patchidx==j & isselected & nothidden;
        if any(selidx)
            selpatchhandles(j)=line(x(selidx),y(selidx),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','on','tag','selected','linewidth',2,'hittest','off');
        else
            selpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','off','tag','selected','linewidth',2,'hittest','off');
        end
    end
else
    selidx=isselected & nothidden;
    if any(selidx)
        selpatchhandles=line(x(selidx),y(selidx),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','on','tag','selected','linewidth',2,'hittest','off');
    else
        selpatchhandles=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','off','tag','selected','linewidth',2,'hittest','off');
    end
    
end

udata.view.filthandles=filtpatchhandles;
udata.view.mainhandles=mainpatchhandles;
udata.view.selhandles=selpatchhandles;
udata.view.patchidx=patchidx;
udata.view.uselines=uselines;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%set buttondownfcns:
set(targetaxes,'hittest','on')
ravesetselectionmodescatter(handles.fig,handles.activeindex);

%format for fillstyle:
fillstyle=udata.format.markerfill;
if ~uselines
    if fillstyle==1
        set(udata.view.mainhandles,'markerfacecolor','none','markeredgecolor','flat');
        set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
    elseif fillstyle==2
        set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor','none');
        set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
    else %fillstyle=3
        set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
        set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
    end
else
    for j=1:10
        if fillstyle==1
            set(udata.view.mainhandles(j),'markerfacecolor','none','markeredgecolor',colors(j,:));
            set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
        elseif fillstyle==2
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor','none');
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        else %fillstyle=3
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor',[0 0 0]);
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        end
    end
end
set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))

udata.format.xlim=get(targetaxes,'xlim');
udata.format.ylim=get(targetaxes,'ylim');
set(targetaxes,'xlimmode','manual','ylimmode','manual');

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',handles.prefs.fonts.graphsize,...
        'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

%do mouseovers
if udata.explore.mouseover
    l1=line(nan,nan,'marker',udata.format.marker,'markerfacecolor','none','markeredgecolor',handles.colors.mouseoverhighlight,'linewidth',1.5,...
        'visible','off','parent',targetaxes,'hittest','off','linestyle','none');
    t1=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t2=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t3=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t4=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t5=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'fontsize',14);
    
    udata.explore.mouseoverhandles=[l1 t1 t2 t3 t4 t5];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[]; 
    
    handles.mouseoverhandles=[handles.mouseoverhandles;targetaxes;filtpatchhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverscatteraxes,handles.activeindex}};repmat({{@ravemouseoverscatterdata,handles.activeindex}},numel(filtpatchhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(filtpatchhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverscatteraxes ravemouseoverscatterdata
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
%update handles
guidata(src,handles)
ravemakecolorbar(handles.fig,[]);

%make ticks:

%clear existing ticks: (these may be recreated during this function by
%axeschangelistener, so clear them here at the end:
labels=findall(targetaxes,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

ravemaketextticks(targetaxes,[],xcol,ycol,[],[]);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects

raveupdateexploremethods(handles.fig,[],handles.activeindex,'recreate');

