function ravecreatedensity(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%stop warning us about this (for when all data has been filtered)
warning('off','MATLAB:hg:patch:CannotUseFaceVertexCDataOfSize0')

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%backwards compatibility:
if ~isfield(udata.format,'emphedges');
    udata.format.emphedges=0;
end

%clear existing graph
labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

cla(targetaxes);
legend(targetaxes,'off');

%set(targetaxes,'userdata','density','box','on','color','none');
set(targetaxes,'xlimmode','auto','ylimmode','auto');
set(targetaxes,'xtickmode','auto','ytickmode','auto');
set(targetaxes,'layer','top')

%font formatting
fweight='normal';
labelsize=handles.prefs.fonts.graphsize;

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

%Normalize Data:
if isempty(udata.format.xlim);
    minx=min(x);
    maxx=max(x);
else
    minx=udata.format.xlim(1);
    maxx=udata.format.xlim(2);
end

if isempty(udata.format.ylim);
    miny=min(y);
    maxy=max(y);
else
    miny=udata.format.ylim(1);
    maxy=udata.format.ylim(2);
end

rangex=maxx-minx;
rangey=maxy-miny;

%normalize data before using inpolygon:
x=x-minx;
x=x./rangex;

y=y-miny;
y=y./rangey;

%% generate grid:
% For hexagons....
% Generate hexagonal grid

switch udata.view.gridshape
    case 'hexagon'
        
        %get axes pixel size:
        pos=get(targetaxes,'position');
        
        ypix=pos(4);
        xpix=pos(3);
        
        ny=round(udata.view.gridsize); %n must be even
        nx=round(ny*xpix/ypix)+1;
        
        udata.view.nx=nx;
        udata.view.ny=ny;
        
        X=repmat(linspace(0,1,nx),ny,1);
        
        Y=linspace(0,1,2*ny);
        Y=reshape(Y,2,[])';
        
        if rem(nx,2)==0 %nx is even
            Y=repmat(Y,1,nx/2);
        else
            Y=repmat(Y,1,(nx-1)/2+1);
        end
        
        rx=diff(X(1,1:2))*(2/3);
        ry=diff(Y(1:2,1))/sqrt(3);
        
        X=X(:);
        Y=Y(:);
        
        thetas=[0 60 120 180 240 300]*pi/180;
        
        %calculate in-ness:
        inhere=false(numel(x),numel(X));
        
        notaccountedfor=true(size(x));
        %notaccountedfor(~nothidden)=false; %exclude hidden points
        
        %presort data:
        [x,b]=sort(x);
        y=y(b);
        
        for c=1:numel(X)
            xhex=rx*cos(thetas)+X(c);
            yhex=ry*sin(thetas)+Y(c);
            
            if ~any(notaccountedfor);break;end
            
            try
                inhere(notaccountedfor,c)=raveinsidepoly(x(notaccountedfor),y(notaccountedfor),xhex,yhex);
                notaccountedfor(inhere(notaccountedfor,c))=false;
            catch %#ok<CTCH> %when mex is incompatible...
                inhere(notaccountedfor,c)=inpolygon(x(notaccountedfor),y(notaccountedfor),xhex,yhex);
                notaccountedfor(inhere(notaccountedfor,c))=false;
            end
            
        end
        
        [junk,aaa]=sort(b);
        inhere=inhere(aaa,:);
        x=x(aaa);
        y=y(aaa);
        
        udata.view.inhere=inhere;
        
        inherefiltered=bsxfun(@and,inhere,nothidden);
        thismanyfiltered=sum(inherefiltered,1);
        
        inhere=bsxfun(@and,inhere,~isfiltered);
        inhere=bsxfun(@and,inhere,nothidden);
        thismany=sum(inhere,1);
        
        inhereselected=bsxfun(@and,inherefiltered,nothidden);
        inhereselected=bsxfun(@and,inhereselected,isselected);
        thismanyselected=sum(inhereselected,1);
end

%dont let any tiles get drawn at less than 0.1 scale
tilescale=sqrt(thismanyfiltered)./max((sqrt(thismanyfiltered)));
tilescale(tilescale<0.15)=0.15;

%% assign colors:
if ccol==-2 %use count to color
    c=thismany;
    minc=1;
    maxc=max(thismanyfiltered);
    
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        facecolor=(c-minc)/(maxc-minc)*127+1+128*(udata.format.mcmap-1);
    else
        facecolor=-(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap);
    end
elseif ccol==-1 %use most populous row color
    c=zeros(numel(X),1);
    for h=1:numel(X)
        c(h)=mode(handles.rowcolors{d}{a}(inhere(:,h)));
    end
    
    facecolor=c+1920;
    
elseif ccol==0 %use constant color
    facecolor=repmat(udata.format.facecolor,numel(X),1);
else %color according to avg of selected variable.
    c=zeros(numel(X),1);
    for h=1:numel(X)
        c(h)=sum(handles.alldata{d}(inhere(:,h),ccol))./thismany(h); %faster than calling "mean"
    end
    
    minc=min(handles.alldata{d}(:,ccol));
    maxc=max(handles.alldata{d}(:,ccol));
    
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        facecolor=(c-minc)/(maxc-minc)*127+1+128*(udata.format.mcmap-1);
    else
        facecolor=-(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap);
    end
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%% SIZE TILES
if scol==-1 %Do density sizing
    sizescale=tilescale;
elseif scol==0 %Do uniform sizing
    sizescale = ones(numel(X),1);
else %size by avg of selected variable.
    s=zeros(numel(X),1);
    for h=1:numel(X)
        s(h)=mean(handles.alldata{d}(inhere(:,h)',scol));
    end
    sizescale=s-min(s);
    sizescale=sizescale./max(sizescale);
end

%% Draw the patches:
%denormalize data:
if ~isempty(x)
    
    verts=zeros(numel(X)*6,2);
    sverts=verts;
    fverts=verts;
    faces=(1:(6*numel(X)))';
    
    isvacant=false(numel(X),1);
    sisvacant=false(numel(X),1);
    fisvacant=false(numel(X),1);
    
    for h=1:numel(X)
        
        %filtered data:
        if (~udata.view.preciseoutliers && thismanyfiltered(h)>0) || (udata.view.preciseoutliers && thismanyfiltered(h)>1)
            fxhex=sizescale(h)*rx*cos(thetas)+X(h);
            fyhex=sizescale(h)*ry*sin(thetas)+Y(h);
        elseif thismanyfiltered(h)==1
            thisone=find(inpolygon(x,y,rx*cos(thetas)+X(h),ry*sin(thetas)+Y(h)));
            if numel(thisone)==1 %even if first check says 1 point, sometimes there are more than one (not sure why...)
                fxhex=sqrt(thismanyfiltered(h))/sqrt(thismany(h))*sizescale(h)*rx*cos(thetas)+x(thisone);
                fyhex=sqrt(thismanyfiltered(h))/sqrt(thismany(h))*sizescale(h)*ry*sin(thetas)+y(thisone);
            else
                fxhex=sizescale(h)*rx*cos(thetas)+X(h);
                fyhex=sizescale(h)*ry*sin(thetas)+Y(h);
                
            end
            
        else
            fisvacant(h)=true;
            fxhex=zeros(6,1);
            fyhex=zeros(6,1);
        end
        
        fverts((h-1)*6+1:h*6,1)=fxhex(:);
        fverts((h-1)*6+1:h*6,2)=fyhex(:);
        
        
        %main data:
        if (~udata.view.preciseoutliers && thismany(h)>0) || (udata.view.preciseoutliers && thismany(h)>1)
            
            if scol==-1 %density sizing can draw smaller main data series than filterd data set.
                xhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+X(h);
                yhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+Y(h);
            else
                xhex=sizescale(h)*rx*cos(thetas)+X(h);
                yhex=sizescale(h)*ry*sin(thetas)+Y(h);
            end
        elseif thismany(h)==1
            thisone=find(inpolygon(x,y,rx*cos(thetas)+X(h),ry*sin(thetas)+Y(h)));
            if scol==-1
                if numel(thisone)==1
                    xhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+x(thisone);
                    yhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+y(thisone);
                else
                    xhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+X(h);
                    yhex=sqrt(thismany(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+Y(h);
                end
            else
                if numel(thisone)==1
                    xhex=sizescale(h)*rx*cos(thetas)+x(thisone);
                    yhex=sizescale(h)*ry*sin(thetas)+y(thisone);
                else
                    xhex=sizescale(h)*rx*cos(thetas)+X(h);
                    yhex=sizescale(h)*ry*sin(thetas)+Y(h);
                end
            end
            
        else
            isvacant(h)=true;
            xhex=zeros(6,1);
            yhex=zeros(6,1);
        end
        
        verts((h-1)*6+1:h*6,1)=xhex(:);
        verts((h-1)*6+1:h*6,2)=yhex(:);
        
        %selected data:
        if thismanyselected(h)>0 && (~udata.view.preciseoutliers && thismanyfiltered(h)>0) || (udata.view.preciseoutliers && thismanyfiltered(h)>1)
            sxhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+X(h);
            syhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+Y(h);
        elseif thismanyselected(h)>0 && thismanyfiltered(h)==1
            thisone=find(inpolygon(x,y,rx*cos(thetas)+X(h),ry*sin(thetas)+Y(h)));
            
            if numel(thisone)==1
                sxhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+x(thisone);
                syhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+y(thisone);
            else
                sxhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+X(h);
                syhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+Y(h);
                
            end
            
        else
            sisvacant(h)=true;
            sxhex=zeros(6,1);
            syhex=zeros(6,1);
        end
        
        sverts((h-1)*6+1:h*6,1)=sxhex(:);
        sverts((h-1)*6+1:h*6,2)=syhex(:);
        
        
        %grid data:
        gridxhex=rx*cos(thetas)+X(h);
        gridyhex=ry*sin(thetas)+Y(h);
        
        gridverts((h-1)*6+1:h*6,1)=gridxhex(:);
        gridverts((h-1)*6+1:h*6,2)=gridyhex(:);
    end
end

%restore original scale:
fverts(:,1)=fverts(:,1)*rangex+minx;
fverts(:,2)=fverts(:,2)*rangey+miny;

verts(:,1)=verts(:,1)*rangex+minx;
verts(:,2)=verts(:,2)*rangey+miny;

sverts(:,1)=sverts(:,1)*rangex+minx;
sverts(:,2)=sverts(:,2)*rangey+miny;

gridverts(:,1)=gridverts(:,1)*rangex+minx;
gridverts(:,2)=gridverts(:,2)*rangey+miny;


%crop to stay within plot bounds:
fverts(fverts(:,1)<minx,1)=minx;
fverts(fverts(:,1)>maxx,1)=maxx;
fverts(fverts(:,2)<miny,2)=miny;
fverts(fverts(:,2)>maxy,2)=maxy;

verts(verts(:,1)<minx,1)=minx;
verts(verts(:,1)>maxx,1)=maxx;
verts(verts(:,2)<miny,2)=miny;
verts(verts(:,2)>maxy,2)=maxy;

sverts(sverts(:,1)<minx,1)=minx;
sverts(sverts(:,1)>maxx,1)=maxx;
sverts(sverts(:,2)<miny,2)=miny;
sverts(sverts(:,2)>maxy,2)=maxy;

gridverts(gridverts(:,1)<minx,1)=minx;
gridverts(gridverts(:,1)>maxx,1)=maxx;
gridverts(gridverts(:,2)<miny,2)=miny;
gridverts(gridverts(:,2)>maxy,2)=maxy;

%make filtered data series:
[fuverts,junk,uidx]=unique(fverts,'rows');
fpatchdata.vertices=fuverts;
facemat=reshape(faces(uidx),6,[])';
fpatchdata.faces=facemat(~fisvacant,:);
filteredhandles=patch(fpatchdata,'parent',targetaxes,'facecolor',handles.colors.filtered,'edgecolor','none','hittest','off','tag','filtered');

%make main data series:
[uverts,junk,uidx]=unique(verts,'rows');
patchdata.vertices=uverts;
facemat=reshape(faces(uidx),6,[])';
patchdata.faces=facemat(~isvacant,:);
mainhandles=patch(patchdata,'parent',targetaxes,'facecolor','flat','edgecolor',[0 0 0],'cdata',facecolor(~isvacant),'hittest','off','tag','main');

%make selected data series:
[suverts,junk,suidx]=unique(sverts,'rows');
spatchdata.vertices=suverts;
sfacemat=reshape(faces(suidx),6,[])';
spatchdata.faces=sfacemat(~sisvacant,:);
selectedhandles=patch(spatchdata,'parent',targetaxes,'facecolor','none','edgecolor',handles.colors.selectionhighlight,'hittest','off','tag','selected');

%make grid data series:
[griduverts,junk,griduidx]=unique(gridverts,'rows');
gridpatchdata.vertices=griduverts;
gridfacemat=reshape(faces(griduidx),6,[])';
gridpatchdata.faces=gridfacemat;
gridhandle=patch(gridpatchdata,'parent',targetaxes,'facecolor','none','edgecolor',[.8 .8 .8],'hittest','off','tag','grid');


if udata.format.grid==4
    set(gridhandle,'edgecolor',get(handles.fakerax(handles.activeindex),'xcolor'))
    uistack(gridhandle,'bottom')
elseif udata.format.emphedges
    set(gridhandle,'edgecolor',get(handles.fakerax(handles.activeindex),'color'))
else
    set(gridhandle,'visible','off')
end

udata.view.mainhandles=mainhandles;
udata.view.selectedhandles=selectedhandles;
udata.view.filteredhandles=filteredhandles;
udata.view.sizescale=sizescale;

set(handles.fakeax(handles.activeindex),'userdata',udata);

if ~udata.format.showedges;set(mainhandles,'edgecolor','none');end


%store axis limits. This needs to be done because patches don't clip when
%they go outside axes bounds.

if isempty(udata.format.xlim)
    udata.format.xlim=[min(handles.alldata{d}(:,xcol)) max(handles.alldata{d}(:,xcol))];
end

if isempty(udata.format.ylim)
    udata.format.ylim=[min(handles.alldata{d}(:,ycol)) max(handles.alldata{d}(:,ycol))];
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
try %error if xcol or ycol is constant
    set(targetaxes,'xlim',udata.format.xlim,'ylim',udata.format.ylim);
end

set(targetaxes,'buttondownfcn',@buttondownfcndensity,'hittest','on');

%%
%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.prefs.fonts.graph)

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',handles.prefs.fonts.graphsize,...
        'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

guidata(src,handles)
ravemakecolorbar(handles.fig,[]);
ravemaketextticks(targetaxes,[],xcol,ycol,[],[])

drawnow;
raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects

