function handles=parallelcoordsapplyfilter(src,ev,ai,handles)
%update parallel coordinate line visibilities based on current filter
%locations.
%NOTE: for speed, this function takes in and returns handles (to avoid slow
%guidata). Make sure functions that call this send handles as the 4th
%input.

udata=get(handles.fakeax(ai),'userdata');

allkidz=udata.view.mainhandles;
%allydata=cell2mat(get(allkidz,'ydata'));

alldata=handles.alldata{udata.view.dataset};

if numel(udata.view.highfilter)<size(alldata,2)
    n=numel(udata.view.highfilter);
    newhighs=handles.maxfilter{udata.view.dataset}{udata.view.analysis}((n+1):end);
    newlows=handles.minfilter{udata.view.dataset}{udata.view.analysis}((n+1):end);
    udata.view.highfilter=[udata.view.highfilter, newhighs];
    udata.view.lowfilter=[udata.view.lowfilter, newlows];
    udata.view.varmins=handles.minlimit{udata.view.dataset}{udata.view.analysis};
    udata.view.varranges=handles.maxlimit{udata.view.dataset}{udata.view.analysis}-handles.minlimit{udata.view.dataset}{udata.view.analysis};

    set(handles.fakeax(ai),'userdata',udata);
end
  
maxoutlier=bsxfun(@gt,alldata,udata.view.highfilter);
minoutlier=bsxfun(@lt,alldata,udata.view.lowfilter);

nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
nothidden=nothidden(udata.view.maxlineindex);

outliers=minoutlier | maxoutlier;

if ismember('partialfilter',udata.explore.method)  % DO PARTIAL FILTERING
    
    %count how many filters each point violates:
    violations=sum(outliers,2);
    
    violations=violations(udata.view.maxlineindex);
    
    violations(violations>udata.explore.partialfilter.levels)=udata.explore.partialfilter.levels;
    
    dim=violations./udata.explore.partialfilter.levels;
    dim=dim.^0.5    ; %make it so that the first violation is more strongly emphasized than subsequent violations.
    
    %Get original data colors:
    index=udata.view.maxlineindex;
    ccol=udata.view.c;
    allcolorz=get(handles.fig,'colormap');
    if ccol~=0
        c=handles.alldata{udata.view.dataset}(index,ccol);
        %normalize cdata to desired colormap:
        c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.lcmap-1)+1;
        if udata.format.reversecolors
            c=(1-(c-min(c))/(max(c)-min(c)))*127+128*(udata.format.lcmap-1)+1;
        end
        
        c(isnan(c))=128*(udata.format.lcmap-1)+1;
        cdata=allcolorz(floor(c),:);
        
    else %USE ROW COLORS
        c=1920+(handles.rowcolors{udata.view.dataset}{udata.view.analysis}(index))'; %non-noisy colors cdata=0.5
        cdata=allcolorz(c,:);
        if udata.format.ncolor %do noisy colors
            noise=0.3; %noise multiplier
            cdata=cdata+(rand(size(cdata))-.5)*noise;
            cdata(cdata<0)=0;
            cdata(cdata>1)=1;
        end
    end
    
    %average colors with filtered colors:
    
    fcdata=repmat(handles.colors.filtered,size(cdata,1),1);
    newcdata=bsxfun(@times,cdata,1-dim)+bsxfun(@times,fcdata,dim);
    newcdata=mat2cell(newcdata,ones(size(newcdata,1),1),3);
    
    set(udata.view.mainhandles,'visible','on');
    set(udata.view.mainhandles,{'color'},newcdata);
    
    %stack so that less-filtered points come to the front:
    allkidz=get(handles.allgraphs(ai),'children');
    mainidx=ismember(allkidz,udata.view.mainhandles);
    [junk,newidx]=sort(violations,'ascend');
    allkidz(mainidx)=udata.view.mainhandles(newidx);
    set(handles.allgraphs(ai),'children',allkidz);
    
    outliers=any(outliers,2);
    outliers(~nothidden)=true;
    handles.isfiltered{udata.view.dataset}{udata.view.analysis}=outliers;
    
    if ~handles.prefs.filteredselectable
        handles.isselected{udata.view.dataset}{udata.view.analysis}(outliers)=false;
    end
    
    %guidata(handles.fig,handles);
    
else
    outliers=any(outliers,2);
    outliers(~nothidden)=true;
    handles.isfiltered{udata.view.dataset}{udata.view.analysis}=outliers;
    
    if ~handles.prefs.filteredselectable
        handles.isselected{udata.view.dataset}{udata.view.analysis}(outliers)=false;
    end
    
    %guidata(handles.fig,handles);

    %dim infeasible rows:
    outliers=outliers(udata.view.maxlineindex);
    vises=cell(numel(allkidz),1);
    vises(:)={'on'};
    vises(outliers)={'off'};
    
    set(allkidz,{'visible'},vises)
    set(udata.view.hiddenhandles(outliers),'visible','on');
    set(udata.view.hiddenhandles(~outliers),'visible','off');
    set(udata.view.hiddenhandles(~nothidden),'visible','off');
end




