function ravecreatehistogram(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
xcol=udata.view.x;
a=udata.view.analysis; 
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
legend(targetaxes,'off');
try
    delete(handles.subax{handles.activeindex})
    delete(handles.subfakeax{handles.activeindex})
    delete(handles.subfakerax{handles.activeindex})
end

handles.subax{handles.activeindex}=[];
handles.subfakeax{handles.activeindex}=[];
handles.subfakerax{handles.activeindex}=[];
guidata(handles.fig,handles);

%% make the graph.

%how many graphs to draw?
numvars=length(xcol);
set(targetaxes,'visible','off','xticklabel','','yticklabel','');

%% %% %%
margin=20;
pos = get(targetaxes,'Position');%+[30 30 -30 -30]; %shrink a little to fit tick labels.

xlabelspace=20;% pixels to move up axes to make room for labels at the bottom.
ylabelspace=32;% pixels to allow between graphs to make room for axes labels. (note scattermatrix doesnt have this problem.)
%note y labelspace used for horizontal stack, xlabelspace used for vertical stack.

if  pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
    width = (pos(3)-ylabelspace*(numvars-1)-10)/numvars;
    height = pos(4)-xlabelspace-margin;
    space = .00; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];

else %vertical stacking
    width=pos(3)-ylabelspace-margin;
    height=pos(4)/numvars-xlabelspace;
    space = .03; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
end

if width<3;width=3;end
if height<3;height=3;end

%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

for c=1:numvars
    [maincount,selectedcount,filtcount,nbars,edges]=ravehistogramcount(src,[],d,a,xcol(c),true(size(handles.alldata{d},1),1),udata.view.nbars,udata.view.discreteascategory);
    udata.view.edges{c}=edges;
    
    if udata.view.normalize %normalize counts (make pdfs)
        nrows=size(handles.alldata{d},1);
        maincount=maincount./nrows;
        selectedcount=selectedcount./nrows;
        filtcount=filtcount./nrows;
    end

    if  pos(3) > pos(4) % &&  ~udata.view.stack  %horizontal stacking
        axPos = [pos(1)+(c-1)*(width+ylabelspace)+ylabelspace pos(2)+margin ...
            width-ylabelspace height*(1-space)]; %subtract margin to make space for statistics
    else %vertical stacking
        axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
            width-ylabelspace height*(1-space)-20]; %subtract margin to make space for statistics
    end
    
    axPos(axPos<1)=1;
    
    fakerax(c) = axes('units','pixels','Position',axPos,'parent',parent,'color',bgcolor,...
        'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,...
        'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize); %#ok<AGROW>
    fakeax(c) = axes('units','pixels','Position',axPos,'parent',parent,...
        'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid,'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize); %#ok<AGROW>
    ax(c) = axes('units','pixels','Position',axPos,'parent',parent,...
        'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnhistogram,...
        'userdata',handles.activeindex,'color','none','fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
        'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %#ok<AGROW> %userdata contains the index of the bigax for this subgraph.
    
    %for robustness to deletion errors, save these on every iteration:
    handles.subax{handles.activeindex}=ax;
    handles.subfakeax{handles.activeindex}=fakeax;
    handles.subfakerax{handles.activeindex}=fakerax;
    guidata(handles.fig,handles);

    if strcmp(udata.format.orient,'horizontal') 
        view(ax(c),90,90);
        view(fakeax(c),90,90);
        view(fakerax(c),90,90);
    end
    
    if udata.format.drawedge
        fedgecolor=udata.format.edgecolor;
        edgecolor=udata.format.edgecolor;
        sedgecolor=udata.format.edgecolor;
    else
        fedgecolor=handles.colors.filtered;
        edgecolor=udata.format.facecolor;
        sedgecolor=handles.colors.selectionhighlight;
    end
    
    %filtered data series:
    filteredhandle=bar(ax(c),edges,filtcount,udata.view.barwidth);
    set(filteredhandle,'facecolor',handles.colors.filtered,'hittest','on','tag','filtered','edgecolor',fedgecolor,'buttondownfcn',@pointselecthistogram);

    %main data series:
    mainhandle=bar(ax(c),edges,maincount,udata.view.barwidth);
    set(mainhandle,'facecolor',udata.format.facecolor,'hittest','on','tag','main','edgecolor',edgecolor,'buttondownfcn',@pointselecthistogram);
    
    %selected data series
    selectedhandle=bar(ax(c),edges,selectedcount,udata.view.barwidth);
    set(selectedhandle,'facecolor',handles.colors.selectionhighlight,'hittest','off','tag','selected','edgecolor',sedgecolor);

    axis(ax(c),'tight');
    set(ax(c),'ylimmode','auto','tickdir','in','layer','top');
    xlim{c} = get(ax(c),'xlim');  %#ok<AGROW>
    ylim{c} = get(ax(c),'ylim'); %#ok<AGROW>
    
    yticks=get(ax(c),'yticklabel');
    if ~iscell(yticks);
       yticks=mat2cell(yticks,ones(size(yticks,1),1),size(yticks,2));
       yticks=cellfun(@strtrim,yticks,'uniformoutput',0);
    end

    ravemaketextticks(ax(c),[],xcol(c),0,[],[],[],yticks) 
    set([fakeax(c) fakerax(c)],'xtick',get(ax(c),'xtick'),'xticklabel','','ytick',get(ax(c),'ytick'),'yticklabel','','ztick',get(ax(c),'ztick'),'zticklabel','')
    xlabel(ax(c),handles.allnames{d}{xcol(c)},'interpreter',handles.prefs.interpreter,'fontsize',handles.prefs.fonts.graphsize,'fontname',handles.prefs.fonts.graph)
end

%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
guidata(handles.fig,handles);

set(handles.fakeax(handles.activeindex),'userdata',udata);

%make listeners:
delete(handles.axeslisteners{handles.activeindex});
handles.axeslisteners{handles.activeindex}=[];

%must store first listener outside of loop to initialize the variable type
%as "listener";

handles.axeslisteners{handles.activeindex}=addlistener(ax(1,1),'YTick','PreSet',@subaxeschangelistener);
handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(1,1),'XTick','PreSet',@subaxeschangelistener);
for c=2:numvars
    handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(c),'XTick','PreSet',@subaxeschangelistener);
    handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(c),'YTick','PreSet',@subaxeschangelistener);
end

guidata(handles.fig,handles);

%fix axes limts:
for r=1:numvars
    set(fakerax(r),'xlim',xlim{r},'ylim',ylim{r})
    set(fakeax(r),'xlim',xlim{r},'ylim',ylim{r})
end

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%% %% %%
set(ax,'ylimmode','manual','xlimmode','manual');
set(handles.activegraph,'userdata','histogram');

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

%select the subaxes in position (1) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%update handles
guidata(src,handles)

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects


