function viewtabhistogram(src,eventdata) %#ok<INUSD>
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');

%backward compatibility:
if ~isfield(udata.view,'commonaxis')
    udata.view.commonaxis=0;
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

if ~isfield(udata.view,'sharedaxis')
    udata.view.sharedylim=0;
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

%% text headers

projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title);

optionstext=text(.15-.10317,.20*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',18*handles.prefs.fonts.titlescale,'fontname',handles.prefs.fonts.title,'color',handles.colors.title,'visible','off');

%% text labels

analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

xaxistext=text(.25-.10317,.72*1.0317-.0317,'Choose Variables:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

yaxistext=text(.25-.10317,.37*1.0317-.0317,'Y-Axis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',udata.view.analysis);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',udata.view.dataset);

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+235 180 200],...
    'style','listbox','string',handles.allnames{1},'value',udata.view.x,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',50,'callback',@xaxismenu_callback);

%% buttons

scalebutton=uicontrol('position',[origin(1)+55 origin(2)+065 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg,'visible','off');

countbutton=uicontrol('position',[origin(1)+92 origin(2)+204 100 30],...
    'style','radiobutton','string','count',...
    'backgroundcolor',handles.colors.tabbg,'value',~udata.view.normalize,'callback',@countbutton_callback,'tag','countbutton');
probbutton=uicontrol('position',[origin(1)+152 origin(2)+204 100 30],...
    'style','radiobutton','string','frequency',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.normalize,'callback',@probbutton_callback,'tag','probbutton',...
    'value',0);

sharedyaxis=uicontrol('position',[origin(1)+35 origin(2)+174 200 30],...
    'style','checkbox','string','All axes have same Y-Axis range',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.sharedylim,'callback',@sharedaxis_callback,'tag','probbutton',...
    'value',0);

sharedyaxis=uicontrol('position',[origin(1)+35 origin(2)+150 220 30],...
    'style','checkbox','string','Treat Discrete Variables as Categorical',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.discreteascategory,'callback',@discretebutton_callback,'tag','probbutton',...
    'value',0);


outlierscheck=uicontrol('position',[origin(1)+55 origin(2)+170 200 30],...
    'style','checkbox','string','Exclude outliers beyond       sigma',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.outliers,'callback',@outlierscheck_callback,'tag','outlierscheck','visible','off');
outliersvalue=uicontrol('position',[origin(1)+191 origin(2)+176 16 20],...
    'style','edit','string','3',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.stddevs,'callback',@outliersvalue_callback,'tag','outliersvalue','visible','off');

%% group items and finish up
handles.tabobjects=[datasettext xaxistext datasetmenu xaxismenu,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext yaxistext,...
    countbutton probbutton outlierscheck outliersvalue sharedyaxis];

handles.activetab='view';
guidata(src,handles);


%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function datasetmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=get(src,'value');
if d==udata.view.dataset;return;end
udata.view.dataset=d;

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
set(xaxismenu,'string',handles.allnames{d})

if numel(handles.allnames{d})<max(udata.view.x);
    udata.view.x=1;
    set(xaxismenu,'value',1);
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

ravecreatehistogram(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehistogram(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata) %#ok<INUSD,INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.x=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehistogram(src,[]);

function countbutton_callback(src,eventdata) %#ok<INUSD> %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handles=guidata(src);

set(src,'value',1)

probbutton=findobj(handles.tabobjects,'tag','probbutton');
set(probbutton,'value',0)

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.normalize=0; %exclude constant check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);

%re-draw graph
ravecreatehistogram(handles.fig,[])



function probbutton_callback(src,eventdata) %#ok<INUSD> %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handles=guidata(src);

set(src,'value',1)

countbutton=findobj(handles.tabobjects,'tag','countbutton');
set(countbutton,'value',0)

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.normalize=1; %exclude constant check box value
set(handles.fakeax(handles.activeindex),'userdata',udata);

%re-draw graph
ravecreatehistogram(handles.fig,[])

function outlierscheck_callback(src,eventdata) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%not implemented yet.

function sharedaxis_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.sharedylim=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehistogram(src,[]);


function discretebutton_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.discreteascategory=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatehistogram(src,[]);



