function ravecreatebarcategorical(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;


%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

leg=get(handles.fakerax(handles.activeindex),'userdata');
if ~isempty(leg)
    delete(leg(ishandle(leg)));
end

cla(targetaxes);
legend(targetaxes,'off');

set(targetaxes,'xtickmode','auto','ytickmode','auto')

if xcol==0
    text(0.5,0.5,{'Data set must contain at least';'one discrete or string variable';'in order to create bar chart.'},'hor','center','ver','mid','parent',handles.activegraph);
    return
end




%get data

if strcmp(handles.datatype{d}{xcol},'string')
    xvalues=1:numel(handles.discretevalues{d}{a}{xcol});
else
    xvalues=handles.discretevalues{d}{a}{xcol}; %discrete
end

if zcol>0
    
    if strcmp(handles.datatype{d}{zcol},'string')
        zvalues=1:numel(unique(handles.discretevalues{d}{a}{zcol}));
    
    else
        zvalues=unique(handles.discretevalues{d}{a}{zcol});
    end
    
    
    
elseif zcol==0 %row color.
    zvalues=1:10;
else
    zvalues=1;
end

if ycol>0
    y=handles.alldata{d}(:,ycol);
else
    y=ones(size(handles.alldata{d},1),1);
end

x=handles.alldata{d}(:,xcol);

if zcol>0
    z=handles.alldata{d}(:,zcol);
end

%make bar data:
bardata=zeros(numel(xvalues),numel(zvalues));

if zcol>0 %stack by variable
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            bardata(r,c)=aggregate(y(z==zvalues(c) & x==xvalues(r)),udata.view.aggregateby);
        end
    end
elseif zcol==0 %stack by row color
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            bardata(r,c)=aggregate(y(handles.rowcolors{d}{a}==c & x==xvalues(r)),udata.view.aggregateby);
        end
    end
    
elseif zcol==-1 %no stacking
    
    for r=1:numel(xvalues)
        bardata(r)=aggregate(y(x==xvalues(r)),udata.view.aggregateby);
    end
    
    
end

if udata.view.stack
    layoutstring='stacked';
else
    layoutstring='grouped';
end

if numel(xvalues)==1;
    set(handles.infobar,'string','Sorry, I can''t make a bar graph when the x-axis data is constant');
    return;
    
else
    hh=bar(targetaxes,xvalues,bardata,'barlayout',layoutstring,'barwidth',udata.view.barwidth);
end


nstacks=numel(hh);

if zcol~=0
    allcolors=handles.prefs.barcolors;
else
    allcolors=handles.prefs.rowcolors;
end


ncolors=size(allcolors,1);

reps=ceil(nstacks/ncolors);
allcolors=repmat(allcolors,reps,1);

for n=1:nstacks
    set(get(hh(n),'children'),'facecolor',allcolors(n,:),'edgecolor',[0 0 0]);
    
end


if strcmp(udata.format.orient,'horizontal')
    view(targetaxes,90,90);
    view(handles.fakeax(handles.activeindex),90,90);
    view(handles.fakerax(handles.activeindex),90,90);
end


set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))


if strcmp(udata.format.orient,'horizontal')
    view(handles.activegraph,90,90);
    view(handles.fakeax(handles.activeindex),90,90);
    view(handles.fakerax(handles.activeindex),90,90);
end

set(targetaxes,'color','none');


%% make tick marks:

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize+2,...
    'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)

if ycol>0 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.prefs.fonts.graphsize+2,'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','Count',...
        'fontsize',handles.prefs.fonts.graphsize+2,'fontweight','normal','interpreter','none','fontname',handles.prefs.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

set(targetaxes,'userdata','barcategorical');

%update da handles
guidata(src,handles)

if zcol>0
    legstrings=handles.discretevalues{d}{a}{zcol};
    if isnumeric(legstrings)
        legstrings=num2cell(legstrings);
        for c=1:numel(legstrings);
            legstrings{c}=num2str(legstrings{c});
        end
    end

    leg1=legend(handles.activegraph,hh,legstrings,'interpreter',handles.prefs.interpreter);
    set(handles.fakerax(handles.activeindex),'userdata',leg1);
    legtitle=get(leg1,'title');
    
    if zcol>1
        set(legtitle,'string',handles.allnames{d}{zcol});
    else
        set(legtitle,'string','Row Color');
    end

    kidz=get(leg1,'children');
    r=1;
    for k=(numel(kidz)-1):-2:1
        ptch=get(kidz(k),'children');
        set(ptch,'facecolor',allcolors(r,:),'edgecolor','none');
        r=r+1;
    end
end

ravemaketextticks(targetaxes,[],xcol,[],[],[])
raveresizeaxes(handles.fig,[],handles.activeindex);
%ravearrangeobjects(handles.fig,[])
clear ravearrangeobjects

function y=aggregate(yvec,mode)

if mode==1
    y=sum(yvec);
elseif mode==2
    y=mean(yvec);
else
    y=numel(yvec);
end



