function ravecreateternscatter(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'vis','off');

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);
zcol=udata.view.z(1);
ccol=udata.view.c(1);
scol=udata.view.s(1);
%pscales=udata.view.pscale;
%gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

set(targetaxes,'drawmode','fast');

%clear existing graph
cla(targetaxes);
delete(setdiff(findall(targetaxes),targetaxes));
legend(targetaxes,'off');


%set axes properties so that ravemaketernaxes formats everything correctly:


set(targetaxes,'color',[1 1 1],'fontsize',handles.prefs.fonts.graphsize);
set(targetaxes,'xtick',[0:0.1:1],'ytick',[0:0.1:1],'ztick',[0:0.1:1]);

%make new ternary axes:
ravemaketernaxes(handles.fig,[],handles.activeindex);

%get data
x=handles.alldata{d}(:,xcol);
y=handles.alldata{d}(:,ycol);
z=handles.alldata{d}(:,zcol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    if ~udata.format.reversecolors
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
    else
        c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
    end
else
    %c=zeros(length(x),1);
    if udata.format.ncolor
        c=1920+(handles.rowcolors{d}{a})+ rand -.5; %noisy colors
    else
        c=1920+(handles.rowcolors{d}{a}); %non-noisy colors cdata=0.5
    end
end


set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

if scol~=0 && ~isempty(x)
    s=handles.alldata{d}(:,scol);
    isallonesize=0;
    minsize=udata.format.minmarkersize;
    maxsize=udata.format.maxmarkersize;
    s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;
else
    s=ones(length(x),1)*udata.format.minmarkersize;
end

s=sqrt(s); %for consistency with other matlab functions


%% normalize data to fit ternary axes requirements:
[x,y]=terncoords(x,z,y);




%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);
patchidx=zeros(size(s,1),1);

%assign points to patches
for g=1:size(s,1)
    patchidx(g)=find(usdata==s(g),1,'first');
end
%make the patches:

filtpatchhandles=zeros(npatches,1);
mainpatchhandles=zeros(npatches,1);
selpatchhandles=zeros(npatches,1);
%filtered handles
for j=1:npatches
    filtidx=patchidx==j & nothidden;
    if any(filtidx)
        filtpatchhandles(j)=line(x(filtidx),y(filtidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','hittest','off','linestyle','none');
    else
        filtpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','off','hittest','off','linestyle','none');
    end
end

%main handles

for j=1:npatches
    mainidx=patchidx==j & ~isfiltered & nothidden;
    if any(mainidx)
        mainpatchhandles(j)=patch(x(mainidx),y(mainidx),c(mainidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
            'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
            'buttondownfcn',@pointselectscatter3d);
    else
        mainpatchhandles(j)=patch(x(1),y(1),c(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
            'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
            'buttondownfcn',@pointselectscatter3d);
    end
end

if strcmp(handles.selectionmode,'point')
    set(mainpatchhandles,'hittest','on');
end

%selected handles
for j=1:npatches
    selidx=patchidx==j & isselected & nothidden;
    if any(selidx)
        selpatchhandles(j)=line(x(selidx),y(selidx),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','on','tag','selected','linewidth',2,'hittest','off');
    else
        selpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','off','tag','selected','linewidth',2,'hittest','off');
    end
end

udata.view.filthandles=filtpatchhandles;
udata.view.mainhandles=mainpatchhandles;
udata.view.selhandles=selpatchhandles;
udata.view.patchidx=patchidx;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%format for fillstyle:
fillstyle=udata.format.markerfill;
if fillstyle==1
    set(udata.view.mainhandles,'markerfacecolor','none','markeredgecolor','flat');
    set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
elseif fillstyle==2
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor','none');
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
else %fillstyle=3
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
end

% set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
% set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
% 

%link axes so subsequent limit changes affect all axes:
% linkaxes([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xy');
 set(targetaxes,'xlimmode','manual','ylimmode','manual','zlimmode','manual');

%set axes labels

fweight='normal';


set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.prefs.fonts.graphsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',handles.prefs.fonts.graphsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
        
% set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
%     'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
% set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
%         'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
% set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
%         'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)  
%     
% set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
%     'fontweight',fweight,'interpreter','none','fontname','arial')
% set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
%         'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)
% set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
%         'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.prefs.fonts.graph)   
    
%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
%     set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
%     set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
%     set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==2 %major grid
%     set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
%     set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
%     set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
elseif udata.format.grid==3; %minor grid
%     set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','zminorgrid','on')
%     set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
%     set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
end

%reposition camera for iso view:

% set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
%     'cameraposition',[1 1 1]);
% 
%  view(targetaxes,3);
%  view(handles.fakeax(handles.activeindex),3);
%  view(handles.fakerax(handles.activeindex),3);


%make all axes have same limits
%Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

%{
set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));
set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));
set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')
set(Q,'ticklength',[0 0]);
%}
%do mouseovers

%{
if udata.explore.mouseover
    l1=line(nan,nan,nan,'marker',udata.format.marker,'markerfacecolor','none','markeredgecolor',handles.colors.mouseoverhighlight,'linewidth',1,...
        'visible','off','parent',targetaxes,'hittest','off','linestyle','none');
    t1=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal');
    t2=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal');
    t3=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes);
    udata.explore.mouseoverhandles=[l1 t1 t2 t3];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[]; 
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    handles.mouseoverhandles=[handles.mouseoverhandles;targetaxes;filtpatchhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverscatter3Daxes,handles.activeindex}};repmat({{@ravemouseoverscatter3Ddata,handles.activeindex}},numel(filtpatchhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(filtpatchhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverscatter3Daxes ravemouseoverscatter3Ddata
end
%}

%update da handles
guidata(handles.fig,handles)

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.


ravemakecolorbar(handles.fig,[]);     

bg=findall(targetaxes,'tag','axesbackground');

set(bg,'buttondownfcn',@buttondownfcnternscatter,'hittest','on')

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




