function ravecreateoverlayscatter(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');
format=udata.format;

targetaxes=handles.activegraph;
targetindex=find(handles.allgraphs==targetaxes);

filename=udata.view.filename;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index


%quit if no file selected:
if udata.view.invalid || isempty(filename)
    text(.5,.5,'No File Selected','parent',targetaxes,'fontname','arial','fontsize',18,...
        'horizontalalignment','center','verticalalignment','middle','color',[0 0 0],'tag','junk');
    guidata(src,handles)
   
    return
end

numpaths=numel(filename);

legend(targetaxes,'off');

junk=findobj(targetaxes,'tag','junk');
delete(junk);

%clear existing graph
crap=findobj(targetaxes,'tag','main','-or','tag','selected');
delete(crap);
legend(targetaxes,'off');


%% make the graph. %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%combine path and filenames to get full paths:
fullpath=udata.view.filename;

%load image
if strcmpi(fullpath(end-2:end),'png')
    imdata=imread(fullpath,'BackgroundColor',handles.colors.wsbg);
else
    imdata=imread(fullpath);
end

udata.view.imdata=imdata;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%determine how to resize axes:
maxsize=get(handles.allblockers(handles.activeindex),'position');
maxsize=maxsize(3:4)-[24 20];
maxratio=maxsize(1)/maxsize(2);

ratio=size(imdata,2)/size(imdata,1);

if ratio>maxratio %width constrained
   
    newheight=maxsize(1)/ratio;
    
    if newheight>size(imdata,1)
        newheight=size(imdata,1);
    end
    
    heightdiff=maxsize(2)-newheight;
%    oldpos=get(handles.activegraph,'position');
      oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
    if oldpos(3)>size(imdata,2)
        widthdiff=oldpos(3)-size(imdata,2);
        oldpos(3)=size(imdata,2);
    else
        widthdiff=0;
    end
    set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 oldpos(3) newheight])
    
else %height constrained

    newwidth=maxsize(2)*ratio;
    
    if newwidth>size(imdata,2)
        newwidth=size(imdata,2);
    end
    
    widthdiff=maxsize(1)-newwidth;
    %oldpos=get(handles.activegraph,'position');
      oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
    if oldpos(4)>size(imdata,1)
        heightdiff=oldpos(4)-size(imdata,1);
        oldpos(4)=size(imdata,1);
    else
        heightdiff=0;
    end
    
    set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 newwidth oldpos(4)])
end

%resize image if necessary:
axpos=get(handles.activegraph,'position');
if ~udata.view.fastresize && ~isempty(which('imresize')) && size(imdata,1)>axpos(4)
   imdata=imresize(imdata,[axpos(4) axpos(3)]); 
end
    

i=image(imdata,'parent',handles.activegraph);
uistack(i,'bottom')
set(handles.activegraph,'ydir','reverse')
axis(handles.activegraph,'image');
set(handles.activegraph,'visible','off','userdata','overlayscatter','hittest','on','xlimmode','manual','ylimmode','manual')
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'visible','off')

xlim=get(handles.activegraph,'xlim');
ylim=get(handles.activegraph,'ylim');

if isempty(udata.view.limits) %only make these the first time the image is drawn. 
udata.view.limits=patch([xlim(1) xlim(1) xlim(2) xlim(2)],[ylim(1) ylim(2) ylim(2) ylim(1)],[1 1 1 1],'linewidth',1,'edgecolor',[0 1 .5],...
    'parent',handles.activegraph,'hittest','on','buttondownfcn','','facecolor','none','visible','off','clipping','off');
%make movegraph grabbers:
udata.view.grabbers(1)=line(xlim(1)-3,ylim(2)+4,'marker','s','markersize',5,'markerfacecolor',[0 1 .5],'markeredgecolor',[0 0 0],...
    'tag','SW','parent',handles.activegraph,'buttondownfcn',@ravedefinecoordinates,'visible','off','clipping','off');
udata.view.grabbers(2)=line(xlim(1)-3,ylim(1)-3,'marker','s','markersize',5,'markerfacecolor',[0 1 .5],'markeredgecolor',[0 0 0],...
    'tag','NW','parent',handles.activegraph,'buttondownfcn',@ravedefinecoordinates,'visible','off','clipping','off');
udata.view.grabbers(3)=line(xlim(2)+3,ylim(1)-3,'marker','s','markersize',5,'markerfacecolor',[0 1 .5],'markeredgecolor',[0 0 0],...
    'tag','NE','parent',handles.activegraph,'buttondownfcn',@ravedefinecoordinates,'visible','off','clipping','off');
udata.view.grabbers(4)=line(xlim(2)+3,ylim(2)+4,'marker','s','markersize',5,'markerfacecolor',[0 1 .5],'markeredgecolor',[0 0 0],...
    'tag','SE','parent',handles.activegraph,'buttondownfcn',@ravedefinecoordinates,'visible','off','clipping','off');
end

%% MAKE SCATTER PLOT

if isempty(udata.view.tform)
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    guidata(src,handles)
    return
end

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
% pscales=udata.view.pscale;
% gscales=udata.view.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
crap=findobj(targetaxes,'tag','main','-or','tag','selected');
delete(crap);
legend(targetaxes,'off');

%get data
u=handles.alldata{d}(:,xcol);
v=handles.alldata{d}(:,ycol);

%transform data to image coordinate system:
[x, y]=tforminv(udata.view.tform,u,v);

%maintain selection:

isselected=handles.isselected{udata.view.dataset}{udata.view.analysis};
%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};

%% get constantvalues:
% if color/sizecol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the column. 

if ccol~=0
    c=handles.alldata{d}(:,ccol);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap-1)+1;    
else    
    %c=zeros(length(x),1);
    if format.ncolor
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'+10*rand(length(x),1)-.5; %noisy colors
    else
        c=handles.prefs.rowcolors(handles.rowcolors{udata.view.dataset}{udata.view.analysis})'; %non-noisy colors cdata=0.5      
    end
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%set(handles.activegraph,'drawmode','fast')

%% normalize sdata to fit a reasonable range of marker sizes:
if scol~=0
    s=handles.alldata{d}(:,scol);
    isallonesize=0;
    minsize=format.minmarkersize;
    maxsize=format.maxmarkersize;
    s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;

else
    s=ones(length(x),1);
    isallonesize=1;
    
end

%format for fillstyle:
fillstyle=format.markerfill;
if fillstyle==1
    pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
elseif fillstyle==2
    pvpairs={'markerfacecolor','flat','markeredgecolor','none'};
else %fillstyle=3
    pvpairs={'markerfacecolor','flat','markeredgecolor',[0 0 0]};
end

h=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),...
    'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,pvpairs{:},'clipping','on');
set(h,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
%h.refresh
plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
h=double(h); %i have no idea what this line does, but i think it's important.
set(h,'tag','main')


if any(nothidden & isselected)
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(isselected & nothidden),'ydata',y(isselected & nothidden),...
        'sizedata',s(isselected & nothidden),'cdata',c(isselected & nothidden),'hittest','off','marker',format.marker,'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on');
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','on','userdata',isselected)
    %NOTE: userdata for selected, is vector of which points are currently selected.
else
    H=specgraph.scattergroup('parent',targetaxes,'xdata',x(nothidden),'ydata',y(nothidden),...
        'sizedata',s(nothidden),'cdata',c(nothidden),'hittest','off','marker',format.marker,'markerfacecolor','none','markeredgecolor',[1 0 0],'clipping','on');
    set(H,'refreshmode','auto'); %i have no idea what this line does, but i think it's important.
    plotdoneevent(targetaxes,h); %i have no idea what this line does, but i think it's important.
    H=double(H); %i have no idea what this line does, but i think it's important.
    set(H,'tag','selected','markeredgecolor',[1 0 0],'visible','off','userdata',false(length(x),1))
    %NOTE: userdata for selected, is vector of which points are currently selected.
end

%check markersize:
minsize=format.minmarkersize;
if isallonesize
   set(get(h,'children'),'markersize',sqrt(minsize)) ;
   set(get(H,'children'),'markersize',sqrt(minsize)) ; 
end

%make all axes have same limits
%set(targetaxes,'nextplot','add','xlimmode','manual','ylimmode','manual','zlimmode','manual')

%update da handles
guidata(src,handles)

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.
%if udata.format.colorbar && udata.view.c;         
        ravemakecolorbar(handles.fig,[]);     
%end


set(handles.fakeax(handles.activeindex),'userdata',udata);

%update da handles
guidata(src,handles)
raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
