function settings=ravemakemodelrbf(src,ev,settings)
%creates an RSE based on the settings matrix.
%must do three things:
%1) Fit the model.
%2) Assess the model against all rows and calculate predicted values. Store
%   these in settings.predictedvalues.
%3) Make a cell array of strings to be printed in a text file to make a
%   valid matlab function to calculate this model. (this is the hard part).

handles=guidata(src);

xcol=settings.inputs;
d=settings.dataset;

xfitdata=handles.alldata{d}(settings.fitrows,xcol);
yfitdata=handles.alldata{d}(settings.fitrows,settings.output);

xmin=min(xfitdata);
xrange=max(xfitdata)-min(xfitdata);

ymin=min(yfitdata);
yrange=max(yfitdata)-min(yfitdata);



xfitdata=bsxfun(@minus,xfitdata,xmin);
xfitdata=bsxfun(@rdivide,xfitdata,xrange);

yfitdata=bsxfun(@minus,yfitdata,ymin);
yfitdata=bsxfun(@rdivide,yfitdata,yrange);



%determine unique points:
[xfd,idx]=unique(xfitdata,'rows');
xfitdata=xfitdata(idx,:);
yfitdata=yfitdata(idx,:);


%calculate ipdm:
r=ipdm(xfitdata);

%transform by basis function:

fcn=settings.basisfcn;
fcn=strrep(fcn,'*','.*');
fcn=strrep(fcn,'^','.^');
fcn=strrep(fcn,'/','./');
fcn=strrep(fcn,'ln','log');

k=settings.k;

try
r=eval(fcn);
catch
    errordlg(['There was a problem evaluating the basis function ',fcn,'. This must be a scalar function of r and k.'])
end


r(isnan(r))=0;
%calculate weights:

w=r\yfitdata;

%predict data:
alldata=handles.alldata{d}(:,xcol);
alldata=bsxfun(@minus,alldata,xmin);
alldata=bsxfun(@rdivide,alldata,xrange);

r=ipdm(alldata,xfitdata); %#ok<NASGU> %distance from all data points to basis data points
r=eval(fcn);

r(isnan(r))=0;
r(isinf(r))=0;

y=r*w;
settings.predictedvalues=y.*yrange+ymin;


%make model string:
varnames=handles.allnames{d}(xcol);
varnames=ravemakevariablenames(varnames);

line1='xdata = [';
for v=1:numel(varnames);
    line1=[line1,varnames{v},' ']; %#ok<AGROW>
end

str={};
str{end+1} = [line1(1:end-1),'];'];
str{end+1} = 'persistent w xfitdata xmin xrange ymin yrange';
str{end+1} = 'if isempty(w)';
str{end+1} = ['    w=',mat2str(w),';'];
str{end+1} = ['    xfitdata=',mat2str(xfitdata),';'];
str{end+1} = ['    xmin=',mat2str(xmin),';'];
str{end+1} = ['    xrange=',mat2str(xrange),';'];
str{end+1} = ['    ymin=',mat2str(ymin),';'];
str{end+1} = ['    yrange=',mat2str(yrange),';'];
str{end+1} = 'end';
str{end+1} = '';
str{end+1} = 'xdata=bsxfun(@minus,xdata,xmin);';
str{end+1} = 'xdata=bsxfun(@rdivide,xdata,xrange);';
str{end+1} = 'r=ipdm(xdata,xfitdata);';
str{end+1} = ['r=',fcn,';'];
str{end+1} = 'r(isnan(r))=0;';
str{end+1} = 'r(isinf(r))=0;';
str{end+1} = 'y=r*w;';
str{end+1} = 'y=bsxfun(@times,y,yrange);';
str{end+1} = 'y=bsxfun(@plus,y,ymin);';

settings.modelstring=str';













