function movie2gif(mov, gifFile, varargin)
%Copyright (c) 2008, Nicolae Cindea
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
%
%
% Movie2Gif ver. 1.0
% ==================
% Matlab movie to GIF Converter.
%
% Syntax: movie2gif(mov, gifFile, prop, value, ...)
% =================================================
% The list of properties is the same like for the command 'imwrite' for the
% file format gif:
%
% 'BackgroundColor' - A scalar integer. This value specifies which index in 
%                     the colormap should be treated as the transparent 
%                     color for the image and is used for certain disposal 
%                     methods in animated GIFs. If X is uint8 or logical, 
%                     then indexing starts at 0. If X is double, then 
%                     indexing starts at 1.
%
% 'Comment' - A string or cell array of strings containing a comment to be
%             added to the image. For a cell array of strings, a carriage 
%             return is added after each row.
%
% 'DelayTime' - A scalar value between 0 and 655 inclusive, that specifies 
%               the delay in seconds before displaying the next image. 
%
% 'DisposalMethod' - One of the following strings, which sets the disposal 
%                    method of an animated GIF: 'leaveInPlace', 
%                    'restoreBG', 'restorePrevious', or 'doNotSpecify'.
%
% 'LoopCount' - A finite integer between 0 and 65535 or the value Inf (the
%               default) which specifies the number of times to repeat the
%               animation. By default, the animation loops continuously. 
%               For a value of 0, the animation will be played once. For a 
%               value of 1, the animation will be played twice, etc. 
%
% 'TransparentColor' - A scalar integer. This value specifies which index 
%                      in the colormap should be treated as the transparent
%                      color for the image. If X is uint8 or logical, then 
%                      indexing starts at 0. If X is double, then indexing
%                      starts at 1
%
% *************************************************************************
% Copyright 2007 Nicolae CINDEA

if (nargin < 2)
  error('Too few input arguments');
end

if (nargin == 2)
    h = waitbar(0, 'Generating GIF file');
    set(findobj(h,'type','patch'),'facecolor',[0 .5 1],'edgecolor',[0 0 0]);
    set(h,'name','Please Wait...')
    frameNb = size(mov, 2);
    isFirst = true;
    for i = 1:frameNb
        waitbar((i-1)/frameNb, h);
        [RGB, colMap] = frame2im(mov(i)); 
        %[IND, map] = aRGB2IND(RGB); 
        [IND, map] = rgb2ind(RGB,256);
        if isFirst
            imwrite(IND, map, gifFile, 'gif'); 
            isFirst=false;
        else
            imwrite(IND, map, gifFile, 'gif', 'WriteMode', 'append'); 
        end 
    end
    close(h);
end

if (nargin > 2) 
    h = waitbar(0, 'Generating GIF file');
    set(findobj(h,'type','patch'),'facecolor',[0 .5 1],'edgecolor',[0 0 0]);
    set(h,'name','Please Wait...')
    frameNb = size(mov, 2);
    isFirst = true;
    for i = 1:frameNb
        waitbar((i-1)/frameNb, h);
        [RGB, colMap] = frame2im(mov(i)); 
        %[IND, map] = aRGB2IND(RGB);
        [IND, map] = rgb2ind(RGB,256);
        if isFirst
            imwrite(IND, map, gifFile, 'gif', varargin{:}); 
            isFirst=false;
        else
            imwrite(IND, map, gifFile, 'gif', 'WriteMode', 'append', ...
                varargin{:}); 
        end 
    end
    close(h);
end


function [X, map] = aRGB2IND(RGB)

% writted by Nicolae CINDEA

m = size(RGB, 1);
n = size(RGB, 2);
X = zeros(m, n);
map(1,:) = RGB(1, 1, :)./255;

for i = 1:m
    for j = 1:n
        RGBij = double(reshape(RGB(i,j,:), 1, 3)./255);
        isNotFound = true;
        k = 0;
        while isNotFound && k < size(map, 1)
            k = k + 1;
            if map(k,:) ==  RGBij
                isNotFound = false;
            end
        end
        if isNotFound
            map = [map; RGBij];
        end
        X(i,j) = double(k);
    end
end
map = double(map);
