function results=raveoptnbi(src,ev,settings)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%need two settings structures, one for nbi and one for the single objective
%optimizer:
nbisettings=settings;
d=nbisettings.d;
a=nbisettings.a;

%clear existing animations;
delete(findobj(nbisettings.plotaxes,'tag','nbiopthandles'));
delete(findobj(nbisettings.plotaxes,'tag','nbigrid'));
delete(findobj(nbisettings.plotaxes,'tag','nbisearchdir'));
delete(findobj(nbisettings.plotaxes,'tag','nbifinalpoint'));

%start animation:
raveinitializeoptimizeranimation;

%prepare single obj opt:
alg=nbisettings.subalgorithm;

%this is going to overwrite the settings structure:
if ~isempty(which(['optimizersettings',alg]))
    eval(['optimizersettings',alg]);
else
    errordlg(['No optimizer named "',alg,'" was found in your Rave directory'])
end

clear settings; %clear settings to avoid using it later since it's name is now ambiguous.

%load settings for single objective optimizer:

[tabsettingnames,tabsettingvalues]=raveparseoptimizerobjectivetab(src,ev);


if strcmp(nbisettings.settingsfile,'default')
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
else
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
    [fdisplaynames,fsettingvalues]=raveparseoptimizersettingsfile(dest,headerindex);
    fileidx=[];
    for f=1:numel(fdisplaynames)
        idx=find(strcmp(fdisplaynames{t},displaynames));
        if ~isempty(idx)
            fileidx(end+1)=idx; %#ok<AGROW>
        end
    end
    settingnames=displaynames(fileidx);
    settingvalues=fsettingvalues;
end

settingnames=[tabsettingnames(:);settingnames(:)];
settingvalues=[tabsettingvalues(:);settingvalues(:)];

%turn into a structure:
namevaluepairs=[settingnames settingvalues]';
namevaluepairs=namevaluepairs(:)';
subsettings=struct(namevaluepairs{:});

%fix things that don't get autofilled correctly:
subsettings.method=alg;
subsettings.stopbutton=nbisettings.stopbutton;
%subsettings.animation=0;
subsettings.showinfo=0;

%solve for univariate optima:
optfun=str2func(['raveopt',alg]);

numobj=numel(nbisettings.y);

L=zeros(numobj,4);

infotext=findobj(nbisettings.plotaxes,'tag','optinfotext');
totalfunz=0; %total function evals
 
for i=1:numobj
    subsettings.y=nbisettings.y(i);
    subsettings.targets=nbisettings.targets(i);
    
    %get variables for this functions:
    indexx=handles.funvars{d}{subsettings.y};
    
    %also variables for any constraints:
    activecons=subsettings.activeconstraints(subsettings.activeconstraints>0);
    for ac=1:numel(activecons)
        indexx=union(indexx,handles.constraintvars{d}{a}{ac});
    end
    
    %get lowest level inputs:
    while any(handles.isfunction{d}(indexx))
        arefuns=find(handles.isfunction{d}(indexx),1,'last');
        thisfun=indexx(arefuns);
        newvars=handles.funvars{d}{thisfun};
        indexx=union(indexx,newvars);
        indexx=setdiff(indexx,thisfun);
    end
    
    isindexx=ismember(nbisettings.funvars,indexx);
    
    %keep only settings for these variables:
    subsettings.funvars=nbisettings.funvars(isindexx);
    subsettings.x0=nbisettings.x0(isindexx);
    subsettings.normalizationfactors=nbisettings.normalizationfactors(:,isindexx);
    overload.alreadyanimating=1;
    
    if nbisettings.showinfo
        set(infotext,'string',{['Finding univariate optima: ',num2str(i),'/',num2str(numobj)]});
    end
    
    %do optimizer:
    subresults{i}=optfun(src,[],subsettings,overload);
    subresults{i}.independentvars=indexx;
    
    %prevent animations from being deleted:
    if nbisettings.plotprogress
        L(i,1)=findobj(nbisettings.plotaxes,'tag','optinfeasible');
        L(i,2)=findobj(nbisettings.plotaxes,'tag','optfeasible');
        L(i,3)=findobj(nbisettings.plotaxes,'tag','opthistory');
        L(i,4)=findobj(nbisettings.plotaxes,'tag','optcurrent');
        set(L,'tag','nbiopthandles');
    end
end


%form weight grid:
weights=ravetrigridn(nbisettings.ndivs,numobj);

%hide progress so far:
set(L(:,1:3),'visible','off');

%show weights on graph:
univariateoptima=zeros(numobj);
for i=1:numobj
    xidx=subresults{i}.independentvars;
    %thesevars=ismember(nbisettings.funvars,xidx) ;
    
    currentpoint(nbisettings.funvars)=nbisettings.x0;
    currentpoint(xidx)=subresults{i}.finalinputs;
    
    mins(nbisettings.funvars)=nbisettings.normalizationfactors(1,:);
    ranges(nbisettings.funvars)=nbisettings.normalizationfactors(2,:);
    
    %denormalize:
    currentpoint=bsxfun(@times,currentpoint,ranges);
    currentpoint=bsxfun(@plus,currentpoint,mins);
    
    yout=raveevaluatefunction(src,d,nbisettings.y,currentpoint,handles);
    univariateoptima(:,i)=yout(:);
end

%make search base points:
for j=size(weights,1):-1:1
    vtx(:,j)=univariateoptima*(weights(j,:)');
end

ravenbiplotweightgrid(nbisettings,vtx);
drawnow;

%shift points to have the utopia point at the origin:
utopiapoint=diag(univariateoptima);
phi=bsxfun(@minus,univariateoptima,utopiapoint(:));
quasinorm=-phi*ones(size(phi,1),1);
%beta=bsxfun(@minus,vtx,utopiapoint(:));

beta=weights';

%form objective function. THis objective function takes in x and t, and
%returns only t. This ensures that the optimizer uses all variables in
%the x vector (they are needed to calculate constraints)
%
%The format for the vector of independent variables will be [x1 x2 ... xn t],
%so the objective function just returns the last element of this
%vector, multipled by -1 since we want to maximize t.
obj=@(X) -X(end);

%form vector design analysis function:
F=@(xdata) raveevaluatefunction(src,d,nbisettings.y,xdata,handles)';

%form equality constraint function: (b is a column of beta, the barycentric
%coordinates)
eqconstraint= @(xdata,b,t) phi*b + t*quasinorm - (F(xdata)-utopiapoint);

%add a column to necessary fields of subsettings to represent the t
%variable:
%subsettings.normalizationfactors
subsettings.funvars=[nbisettings.funvars NaN];

%start the search at the first univariate optima (this is the first entry
%in the weights array):

%make overload structure:
overload.objectivefcn=obj;
overload.mins=[zeros(1,numel(nbisettings.funvars)) -inf];
overload.maxs=[ones(1,numel(nbisettings.funvars)) inf];
overload.normalizationfactors=[nbisettings.normalizationfactors [0;1]];
overload.alreadyanimating=1;


%make starting point of independent variables:
lastbest=zeros(1,size(handles.alldata{d},2));
lastbest(subresults{1}.independentvars)=[subresults{1}.finalinputs];
lastbest=lastbest(nbisettings.funvars); %keep only the columns needed for nbi optimization
lastbest=[lastbest 0]; %append extra column for t.

%minz(nbisettings.funvars)=nbisettings.normalizationfactors(1,:);
%rangez(nbisettings.funvars)=nbisettings.normalizationfactors(2,:);

%test:
%xtest=bsxfun(@times,xtest,rangez);
%xtest=bsxfun(@plus,xtest,minz);
%F(xtest)

isfinished=false(size(beta,2),1);

%do optimizations:
disp('Beginning optimizations along normal vectors...');
for j=1:size(beta,2)
    
    %draw search direction:
    LL=ravenbiplotsearchdirection(nbisettings,vtx,quasinorm,j);

    %make constraint function specific to this iteration:
    subsettings.normalizationfactors=[nbisettings.normalizationfactors [0;1]]; %used by the constraint
    anoncon = @(X) ravenbi_constraint_calc(X,handles,subsettings,eqconstraint,beta(:,j));
    overload.constraintfcn=anoncon;

    subsettings.x0=lastbest; %start searching at final point of last iteration.
    
    if nbisettings.showinfo
        set(infotext,'string',{['Solving 1-D search: ',num2str(j),'/',num2str(size(beta,2))]});
    end
    
    %do optimizer:
    nbiresults{j}=optfun(src,[],subsettings,overload);
    
    %prevent animations from being deleted:
    if nbisettings.plotprogress
        L(1)=findobj(nbisettings.plotaxes,'tag','optinfeasible');
        L(2)=findobj(nbisettings.plotaxes,'tag','optfeasible');
        L(3)=findobj(nbisettings.plotaxes,'tag','opthistory');
        L(4)=findobj(nbisettings.plotaxes,'tag','optcurrent');
        delete(L(1:3))
        set(L(4),'tag','nbifinalpoint','hittest','off');
       
        ravenbiplotfinalpoint(nbisettings,LL,vtx,L(4),j); 
    end

    isfinished(j)=true;
    
    %start next iteration at final point that is nearest to the next iteration's weight vector:
    if j<size(beta,2)
       nextbeta=beta(:,j+1)';
       finishedbetas=beta(:,1:j)';
       
       dists=ipdm(nextbeta,finishedbetas);
       idx=find(dists==min(dists),1);
       
       lastbest=nbiresults{idx}.finalinputs;
    end
end

%finish animation:
settings=nbisettings; %<-- ravefinishanimation needs this to be called settings.
ravefinishoptimizeranimation;

%export results:
results.finalinputs=zeros(size(beta,2),numel(nbisettings.funvars));
results.finalobjective=zeros(size(beta,2),numel(nbisettings.y));
results.settings=settings;
results.name=['NBI, ',num2str(nbisettings.ndivs),' pt grid, '];
for r=1:numel(nbisettings.y)
    results.name=[results.name,handles.allnames{d}{nbisettings.y(r)}];
    if r<numel(nbisettings.y)
        results.name=[results.name,' '];
    end
end

for b=1:size(beta,2);
    results.finalinputs(b,:)=nbiresults{b}.finalinputs(1:end-1);
end

%denormalize:
results.finalinputs=bsxfun(@times,results.finalinputs,nbisettings.normalizationfactors(2,:));
results.finalinputs=bsxfun(@plus,results.finalinputs,nbisettings.normalizationfactors(1,:));

%evaluate objectives:
xdata=zeros(size(beta,2),size(handles.alldata{d},2));
xdata(:,nbisettings.funvars)=results.finalinputs;
results.finalobjectives=raveevaluatefunction(src,d,nbisettings.y,xdata,handles);


set(LL,'tag','optplots')
set(L(4),'tag','optplots')