function [objective cviolation]=evaluate_population(P,M,nbits,nvars,paretotarget,ranges,rangestep,indexx,handles,udata,settings,dofloor)
%converts binary values into real numbers, and reformats numbers such that max is always best.
%also evaluates all constraints and outputs the total, normalized constraint violation of each point.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

if handles.prefs.gpgpu
    N=size(P,1);

    P3d=reshape(P,[N,nbits,nvars]);
    
    Prowindex=gones(N,size(ranges,2));
    
    
    colnums=(size(P3d,2):-1:1)-1;
    %colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    
    if nbits<=16.1
        colz=gsingle(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    elseif nbits<=32.1
        colz=gsingle(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    else
        colz=gsingle(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    end
    
    
    
else
    N=size(P,1); %note, this might be N or 2N depending on where in the GA you are.
    
    %convert BINARY to decimal:
    P3d=reshape(P,[size(P,1),nbits,nvars]);
    
    Prowindex=ones(N,size(ranges,2));
    
    colnums=(size(P3d,2):-1:1)-1;
    %colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    
    if nbits<=16.1
        colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    elseif nbits<=32.1
        colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    else
        colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    end
    
end

prodz=P3d.*colz;

%base=guint32(2);

expoz=(2.^prodz).*P3d;
sumz=sum(expoz,2);

Prow2D=permute(sumz,[1 3 2]);

Prowindex(:,indexx)=Prow2D;


if ~handles.prefs.gpgpu
    
    Preal=bsxfun(@times,Prowindex,rangestep);
    Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population
    
    Preal(:,dofloor)=floor(Preal(:,dofloor));
    
    %round discrete vars to nearest allowed value:
%    tttt=1;
%    types=handles.datatype{settings.d}(settings.funvars);
    
%     for t=1:numel(types)
%        switch types{t}
%            case {'discrete','string'}
%                val=Preal(:,settings.funvars(t));
%                allowed=repmat(handles.discretevalues{settings.d}{settings.a}{settings.funvars(t)}(:)',size(Preal,1),1);
%                absdiff=abs(bsxfun(@minus,allowed,val));
%                index=bsxfun(@eq,absdiff,min(absdiff,[],2));
%                allowed=allowed';               
%                Pnew=allowed(index');
%                Preal(:,settings.funvars(t))=Pnew;
%            case {'integer','logical'}
%                Preal(settings.funvars(t))=floor(Preal(settings.funvars(t))); 
%        end        
%     end    
    
else
    rangesteparray=gsingle(repmat(rangestep,[size(Prowindex,1) 1]));
    Preal=Prowindex.*rangesteparray;
    rangeadderarray=gsingle(repmat(ranges(1,:),[size(Prowindex,1) 1]));
    Preal=Preal+rangeadderarray;   
    
end


%NOTE: Preal has extra columns at the end for the objective functions, but these are meaningless.
%(and incorrect)
%Only the "objective" variable is used in this function.

%evaluate objective functions at these values:

objective=raveevaluatefunction(handles.fig,settings.d,settings.y,Preal);



%% Reformat data such that a larger number is always best.
for j=1:M
    if paretotarget(j)==inf
        objective(:,j)=objective(:,j); %max is best
    elseif paretotarget(j)==-inf
        objective(:,j)=-objective(:,j); %min is best
    elseif ~isnan(paretotarget(j))
        objective(:,j)=-abs(objective(:,j)-paretotarget(j)); %nominal is best
    end
end

%% evaluate constraints:

if ~handles.prefs.gpgpu
    cviolation=zeros(length(objective(:,1)),numel(handles.temp.activeconstraints));
else
    cviolation=gzeros(length(objective(:,1)),numel(handles.temp.activeconstraints));
end


if isempty(handles.temp.activeconstraints)
    cviolation=zeros(length(objective(:,1)),1);
    return
end



for c=1:numel(handles.temp.activeconstraints)
    thisconstraint=handles.temp.activeconstraints(c);
    ignore=0;
    cviolation(:,c)=raveevaluateconstraint(handles,settings.d,settings.a,thisconstraint,Preal,ignore);
    
end

cviolation(cviolation<0)=0; %set feasible points to 0 violation
scalers=handles.constraintscalers{settings.d}{settings.a}(handles.temp.activeconstraints);
cviolation=bsxfun(@rdivide,cviolation,scalers);
cviolation=sum(cviolation,2);



