function results=raveoptsimulannealbnd(src,ev,settings)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

raveinitializeoptimizeranimation;

d=settings.d;
a=settings.a;


%% make objective/constraint functions:
[functionname,index,mins,maxs]=ravemakeminimizefunction(src,ev,settings);
FITNESSFCN=str2func(functionname);

%% make options structure

options=saoptimset('simulannealbnd');
options.AnnealingFcn=str2func(settings.annealfun);
options.TemperatureFcn=str2func(settings.tempfun);
options.AcceptanceFcn=str2func(settings.acceptfun);
options.TolFun=settings.tolfun;
options.StallIterLimit=settings.stalliter;
options.MaxFunEvals=settings.maxfun;
options.TimeLimit=settings.timelimit;
options.MaxIter=settings.maxiter;
options.ObjectiveLimit=settings.objlimit;
options.InitialTemperature=settings.tinit;
options.ReannealInterval=settings.reannealinterval;


if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';

%% do optimizer:
%tic
handles.temp.crapwindow=rand;
guidata(handles.fig,handles);
[X,FVAL,EXITFLAG,OUTPUT] = simulannealbnd(FITNESSFCN,handles.xdials{d}{a}(settings.funvars),zeros(size(mins)),ones(size(mins)),options); %#ok<NASGU>


%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));

sah=findobj(settings.plotaxes,'tag','annealhist','-or','tag','optbest');
set(sah,'tag','optplots');


handles=guidata(handles.fig);
if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
indexx=settings.funvars;

population=X;

finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
finalpop(:,indexx)=population;

%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});
objective=raveevaluatefunction(src,settings.d,respvars,finalpop);
finalpop(:,respvars)=objective;

%guidata(handles.fig,handles);

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

ravefinishoptimizeranimation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%generate name:
objs=handles.allnames{settings.d}{settings.y};

results.name=['Simulated Annealing ',objs,'   (',handles.datasetnames{settings.d},')'];
[sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
