function [newindex,reverse]=ravechoosecolormap(src,ev,oldindex,reverse) %#ok<INUSL>
%choose a colormap for a specific graph. this should be the callback to a pushbutton that displays
%the current colormap. this callback propmts the user to select new colormap and applies it to the
%graph.
%third input, newindex, is index of currently selected colormap.
%output newindex is the index into handles.cmapnames of the colormap to use next.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

%udata=get(handles.fakeax(handles.activeindex),'userdata');

if nargin==3
    reverse=0;
end

olddat.index=oldindex;
olddat.reverse=reverse;
set(0,'userdata',olddat);


wholecmap=get(handles.fig,'colormap');
cmaprange=(oldindex-1)*128+1:(oldindex)*128;
cmap=wholecmap(cmaprange,:);


uicontrol(handles.dummyobj)

screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-300 screensize(2)+screensize(4)/2-225 600 450],...
    'numbertitle','off','menubar','none','name','Select a Color Map','color',handles.colors.frame,...
    'resize','off','windowstyle','modal','colormap',cmap,'userdata',olddat);

titleax=axes('units','pixels','parent',askbox,'position',[10 410 500 30],'visible','off');
text(0,.4,'Change Graph Colormap','parent',titleax,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[60 380 250 30],'string','Click to select:',...
    'horizontalalignment','left','fontsize',10,'fontweight','bold')

uicontrol('style','pushbutton','string','Ok','parent',askbox,'units','pixels',...
    'position',[360 10 100 30],'callback',@colorpopup_callback,'fontsize',12);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[480 10 100 30],'callback',@colorpopup_callback,'fontsize',12);

for i=1:numel(handles.cmapnames);
    uicontrol('style','pushbutton','units','pixels','parent',askbox,'position',[80 390-22*i 120 20],...
        'cdata',ravemakecdata(handles.fig,[],handles.cmapnames{i}),'callback',@updatecolormap,'userdata',i,...
        'backgroundcolor',[0 0 0])
    uicontrol('style','text','units','pixels','parent',askbox','position',[210 386-22*i 80 20],...
        'string',handles.cmapnames{i},'backgroundcolor',get(askbox,'color'),'horizontalalignment','left');
    if i>=13
        uicontrol('style','pushbutton','units','pixels','parent',askbox,'position',[15 390-22*i 55 20],...
            'string','Modify','userdata',i,'callback',{@ravecolormapeditor,0})
    end
end

uicontrol('style','checkbox','string','Reverse color order','parent',askbox,'units','pixels',...
    'position',[80 33 150 22],'fontsize',10,'tag','reversecheck','backgroundcolor',get(askbox,'color'),'value',reverse);

aa=axes('units','pixels','parent',askbox,'position',[330 130 220 220],'box','on');
scatter(rand(1,50),rand(1,50),rand(1,50)*100+60,rand(1,50),'parent',aa,'facecolor','flat','marker','o','markeredgecolor',[0 0 0])
set(aa,'box','on')

nameax=axes('units','pixels','parent',askbox,'position',[330 65 220 50],'visible','off');
text(.5,.4,handles.cmapnames{oldindex},'parent',nameax,'fontname',handles.prefs.fonts.title,'fontsize',20*handles.prefs.fonts.titlescale,...
    'color',handles.colors.title,'horizontalalignment','center','verticalalignment','baseline','tag','cmapname');

guidata(askbox,handles);

uiwait(askbox);

udat=get(0,'userdata');
newindex=udat.index;
reverse=udat.reverse;
set(0,'userdata','');

function updatecolormap(src,ev)
handles=guidata(src);

index=get(src,'userdata');
wholecmap=get(handles.fig,'colormap');
cmaprange=(index-1)*128+1:(index)*128;
cmap=wholecmap(cmaprange,:);
udat=get(get(src,'parent'),'userdata');
udat.index=index;

set(get(src,'parent'),'colormap',cmap,'userdata',udat);

t=findobj(get(src,'parent'),'tag','cmapname');
set(t,'string',handles.cmapnames{index});


function colorpopup_callback(src,ev)

askbox=get(src,'parent');
reversecheck=findobj(askbox,'tag','reversecheck');
udat=get(askbox,'userdata');
udat.reverse=get(reversecheck,'value');

if strcmp(get(src,'string'),'Ok')
    set(0,'userdata',udat)
end

delete(askbox);


