function hhh=ravedefinevariablecolors(src,ev)
%make a dialog box for the user to specify a fixed color for each variable in a data set. 
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


%note, initial colors are set by managedatagui to hsv(numvars).

handles=guidata(src);
load raveprefs


d=get(handles.datasetmenu,'value'); %selected data set.
handles.d=d;
handles.thisvar=1;
origin=get(handles.indata.fig,'position');


hhh=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-240 origin(2)+origin(4)/2-300 600 475],'color',raveprefs.colors.frame,...
    'name','Change the default line colors for each variable','numbertitle','off','resize','off','windowstyle','modal');
handles.fig=hhh;

titleaxes=axes('units','pixels','position',[20 425 400 40],'parent',hhh,'visible','off');
text(0,.4,'Define Variable Colors','parent',titleaxes,'fontname',raveprefs.fonts.title,'fontsize',20*raveprefs.fonts.titlescale,...
    'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

uicontrol('style','text','parent',hhh,'units','pixels','position',[35 405 400 22],'string','Click a variable in the list below to change its color.',...
    'fontsize',10,'backgroundcolor',raveprefs.colors.frame,'horizontalalignment','left','fontweight','bold')

varnamez=handles.indata.allnames{d};
numvarz=numel(varnamez);

handles.variablecolors=handles.indata.variablecolors{d};

%convert decimal colors to HTML strings:
variablecolors=round(handles.variablecolors*255);
R=dec2hex(variablecolors(:,1));
G=dec2hex(variablecolors(:,2));
B=dec2hex(variablecolors(:,3));

htmlz=[R G B];

%construct monster html formated listbox string:
strng=cell(numvarz,1);
for g=1:numvarz
    strng{g}=['<HTML><B><FONT COLOR="#',htmlz(g,:),'">',varnamez{g},'</FONT></B></HTML>'];    
end

%make java listbox:
handles.jlist=javax.swing.JList(strng);
handles.jlist.setSelectionBackground(java.awt.Color.gray);
handles.jlist.setBackground(java.awt.Color.gray);
handles.jlist.setDoubleBuffered(1);

set(handle(handles.jlist,'callbackproperties'),'MouseClickedCallback',{@listclicked_callback,hhh});
handles.jscroll=javax.swing.JScrollPane(handles.jlist);
[handles.mlist,handles.hglist]=javacomponent(handles.jscroll,[50 105 200 290]);


%make background color buttons:

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[155 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',0,...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[180 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',0,...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[205 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',1,...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[230 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',0,...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.bgblobs=[bgwhite bglight bgdark bgblack];


% make variable info:
if handles.indata.isresponse{d}(1)
    thisvarrole='Response';
else
    thisvarrole='Independent';
end

if handles.indata.isfunction{d}(1)
    thisvarmodel='Yes';
else
    thisvarmodel='No';
end


uicontrol('style','text','position',[290 350 200 22],'string','Variable Name:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame)
uicontrol('style','text','position',[290 320 200 22],'string','Role:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame)
uicontrol('style','text','position',[290 290 200 22],'string','Modeled:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame)
uicontrol('style','text','position',[290 260 200 22],'string','Range:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame)

handles.thisvarname=uicontrol('style','text','position',[400 350 200 22],'string',varnamez{1},'fontname','arial','fontweight','normal','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame);
handles.thisvarrole=uicontrol('style','text','position',[334 320 200 22],'string',thisvarrole,'fontname','arial','fontweight','normal','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame);
handles.thisvarmodel=uicontrol('style','text','position',[360 290 200 22],'string',thisvarmodel,'fontname','arial','fontweight','normal','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame);
handles.thisvarrange=uicontrol('style','text','position',[346 260 200 22],'string',[num2str(min(handles.indata.alldata{d}(:,1))),' to ',num2str(max(handles.indata.alldata{d}(:,1)))],...
    'fontname','arial','fontweight','normal','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame);



uicontrol('style','text','position',[290 230 200 22],'string','Color (click to change):','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',raveprefs.colors.frame)

handles.colorbutton=uicontrol('units','pixels','style','pushbutton','position',[445 235 100 20],'backgroundcolor',handles.variablecolors(1,:),...
    'callback',@colorbutton_callback);

%done button:
uicontrol('units','pixels','style','pushbutton','position',[300 19 133 40],...
    'callback',@donebutton_callback,'string','Done','fontsize',16);


uicontrol('units','pixels','style','pushbutton','position',[450 19 133 40],...
    'callback',@cancelbutton_callback,'string','Cancel','fontsize',16);


guidata(hhh,handles);

set(0,'userdata',[]);


function bgblob_callback(src,ev)
%make listbox background this color:
handles=guidata(src);
set(handles.bgblobs,'value',0);
set(src,'value',1);
newcolor=get(src,'backgroundcolor');
handles.jlist.setBackground(java.awt.Color(newcolor(1),newcolor(2),newcolor(3)));
handles.jlist.setSelectionBackground(java.awt.Color(newcolor(1),newcolor(2),newcolor(3)));

function listclicked_callback(src,ev,hObject) %#ok<INUSL>
handles=guidata(hObject);
d=handles.d;

thisvar=handles.jlist.getSelectedIndex+1;
handles.thisvar=thisvar;
set(handles.colorbutton,'backgroundcolor',handles.variablecolors(thisvar,:));

if handles.indata.isresponse{d}(thisvar)
    thisvarrole='Response';
else
    thisvarrole='Independent';
end

if handles.indata.isfunction{d}(thisvar)
    thisvarmodel='Yes';
else
    thisvarmodel='No';
end

set(handles.thisvarname,'string',handles.indata.allnames{d}{thisvar});
set(handles.thisvarrole,'string',thisvarrole);
set(handles.thisvarmodel,'string',thisvarmodel);
set(handles.thisvarrange,'string',[num2str(min(handles.indata.alldata{d}(:,thisvar))),' to ',num2str(max(handles.indata.alldata{d}(:,thisvar)))]);

guidata(hObject,handles)

function colorbutton_callback(src,ev)
handles=guidata(src);
d=handles.d;

col=ravecolorchooser(handles.fig,handles.indata.variablecolors{handles.d}(handles.thisvar,:),'Pick a Color');

drawnow; %MUST have this to avoid java crash. 

if numel(col)==1
    return    
end


varnamez=handles.indata.allnames{d};
numvarz=numel(varnamez);

set(src,'backgroundcolor',col)
handles.indata.variablecolors{handles.d}(handles.thisvar,:)=col;

%update jlist:
handles.variablecolors=handles.indata.variablecolors{d};
variablecolors=round(handles.variablecolors*255);
R=dec2hex(variablecolors(:,1));
G=dec2hex(variablecolors(:,2));
B=dec2hex(variablecolors(:,3));
htmlz=[R G B];

%construct monster html formated listbox string:
strng=cell(numvarz,1);
for g=1:numvarz
    strng{g}=['<HTML><B><FONT COLOR="#',htmlz(g,:),'">',varnamez{g},'</FONT></B></HTML>'];    
end

%update jlist:
handles.jlist.setListData(strng);

%delete(handles.jlist)
%delete(handles.jscroll)
delete(handles.hglist);

%make java listbox:
handles.jlist=javax.swing.JList(strng);
handles.jlist.setSelectionBackground(java.awt.Color.gray);
handles.jlist.setBackground(java.awt.Color.gray);
handles.jlist.setDoubleBuffered(1);
set(handle(handles.jlist,'callbackproperties'),'MouseClickedCallback',{@listclicked_callback,handles.fig});
handles.jscroll=javax.swing.JScrollPane(handles.jlist);
[handles.mlist,handles.hglist]=javacomponent(handles.jscroll,[50 105 200 290]);

%refresh to avoid draw errors:
drawnow;

guidata(src,handles)



function donebutton_callback(src,ev)
handles=guidata(src);
set(0,'userdata',handles.indata.variablecolors);
close(handles.fig);


function cancelbutton_callback(src,ev)
answer=questdlg('This will cancel all changes. Are you sure you want to do that?','Cancel changes');
if strcmpi(answer,'yes')
handles=guidata(src);
set(0,'userdata',[]);
close(handles.fig);
end


