function ravedeletegraph(src,eventdata,showprompt) %#ok<INUSL>
%3rd input argument is 0 or 1 to show dialog confirming delete.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

if ishandle(handles.fakeax(handles.activeindex))
    udata=get(handles.fakeax(handles.activeindex),'userdata');
end

if ishandle(handles.fakerax(handles.activeindex))
    legdata=get(handles.fakerax(handles.activeindex),'userdata');
else
    legdata=[];
end

%check if graphs exist:
if isempty(handles.activeindex)
    return
end

if showprompt
    %prompt for deletion:
    set(0,'userdata','cancel');
    origin=get(handles.fig,'position');

    askbox=figure('units','pixels','position',[origin(1)+origin(3)/2-350/2 origin(2)+origin(4)/2-150/2 350 150],...
        'numbertitle','off','menubar','none','name','Delete Graph?','color',handles.colors.frame,...
        'resize','off','windowstyle','modal');

    uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
        'position',[20 65 300 40],'string','Delete selected graph permanently? (Use "Put Away" to temporarily hide graphs)','horizontalalignment','left','fontsize',12)
    uicontrol('style','pushbutton','string','Delete','parent',askbox,'units','pixels',...
        'position',[110 10 100 30],'callback',@ravedeletepopup_callback,'fontsize',10);
    uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
        'position',[220 10 100 30],'callback',@ravedeletepopup_callback,'fontsize',10);

    uiwait(askbox);
    answer=get(0,'userdata');

    if ~strcmp(answer,'Delete')
        return
    end
end

%delete subaxes as necessary: (delete these first in case they are children of a uipanel in fakerax)
alltrash=[handles.subax{handles.activeindex}(:);handles.subfakeax{handles.activeindex}(:);handles.subfakerax{handles.activeindex}(:);...
    handles.allgraphs(handles.activeindex);handles.fakeax(handles.activeindex);handles.fakerax(handles.activeindex);handles.grabbers{handles.activeindex}(:)];
delete(alltrash(ishandle(alltrash)));


% try
% delete(handles.subax{handles.activeindex});
% delete(handles.subfakeax{handles.activeindex});
% delete(handles.subfakerax{handles.activeindex});
% end
% 
% delete(handles.allgraphs(handles.activeindex));
% delete(handles.fakeax(handles.activeindex));
% delete(handles.fakerax(handles.activeindex));
% % delete(handles.allblockers(handles.activeindex)); %note, don't delete this, just hide it. 
% % delete(handles.expblockers(handles.activeindex));
% delete(handles.grabbers{handles.activeindex});

try
    delete(handles.axeslisteners{handles.activeindex});
end

%delete colorbar:
if ~isempty(legdata)
    legdata=legdata(ishandle(legdata));
    delete(legdata);
end

set(handles.allblockers(handles.activeindex),'visible','off')
set(handles.expblockers(handles.activeindex),'visible','off')

%delete other graph-specific objects:
if isfield(handles.graphinfo.(handles.activetype),'deletehandles')
    deletehandles=handles.graphinfo.(handles.activetype).deletehandles;
    for g=1:numel(deletehandles)
        thesehandles=eval(deletehandles{g});
        thesehandles=thesehandles(ishandle(thesehandles));
        delete(thesehandles);
    end
end

%delete explore methods objects:
if isfield(udata,'explore');
    exps=fieldnames(udata.explore);
    exps=setdiff(exps,'method');
    for i=1:numel(exps)
        if isfield(udata.explore.(exps{i}),'deletehandles')
            thesehandles=udata.explore.(exps{i}).deletehandles;
            delete(thesehandles(ishandle(thesehandles)));
        end
    end
end

%make another graph the active graph:
newgraph=[];
for s=1:length(handles.allgraphs)
    if ishandle(handles.allgraphs(s))&& ~ismember(s,handles.putaway) && handles.sheet(s)==handles.activesheet %this graph hasnt been deleted
        newgraph=handles.allgraphs(s);
        break
    end
end

handles.activeindex=[];
handles.activegraph=[];
guidata(src,handles);

if ~isempty(newgraph) %graphs exist
    raveselectgraph(newgraph,[]) %make this graph current.
else %graphs dont exist
    managetab(handles.fig,[]);
end


function ravedeletepopup_callback(src,ev) %#ok<INUSD>
set(0,'userdata',get(src,'string'));
close(gcbf);

