function raveinstall
%This function installs RAVE. It does not put the RAVE directory on your search path. You must do that yourself.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


prefspath=which('raveprefs.mat');
if ~isempty(prefspath)
    load raveprefs
else
    raveprefs=ravemakedefaultprefs;
end



%% save and move on...
save('raveprefstemp.mat','raveprefs')


%%
screen0;

function screen0

figg=findobj(0,'name','RAVE Setup');
if ~isempty(figg);return;end; %don't allow multiple instances to run

a=which('raveprefs.mat');
if ~isempty(a);msgbox('It looks like you''ve already set up RAVE. If you complete this setup again your current settings will be overwritten.','Notice - raveprefs.mat already exists');end;

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal');

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton0_Callback);


uicontrol('style','text','string','Thank you for choosing RAVE for your visualization, exploration, optimization, and metamodeling needs. Before you start using RAVE, you need to specify a few options using this interface.',...
    'position',[30 400 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left');

uicontrol('style','text','string','Step 0: Choose Installation Type',...
    'position',[30 350 640 24],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');

uicontrol('style','text','string','Select the desired installation option below.',...
    'position',[30 330 640 24],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');

uicontrol('style','radiobutton','position',[30 310 600 24],'tag','localcheck',...
    'fontsize',10,'backgroundcolor',raveprefs.colors.frame,'callback',@localcheck_callback,...
    'string','Local Installation: Select this to install Rave locally on this computer.','value',1)
uicontrol('style','radiobutton','position',[30 290 600 24],'callback',@portablecheck_callback,...
    'fontsize',10,'backgroundcolor',raveprefs.colors.frame,'tag','portablecheck',...
    'string','Portable Installation: Select this to install Rave to a USB or other portable storage device.')

uicontrol('style','text','string','Portable Installation Note: If you have not already done so, move the "rave" directory to the location (drive) where you want Rave installed. Then quit this installer, and re-run it from Rave''s new location. Alternatively, you can finish this installer and then move the rave directory to its final location.',...
    'position',[30 200 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal','visible','off','tag','portablewarning');

if ~isempty(a);uistack(askbox,'down');end;


function screen1
figg=findobj(0,'name','RAVE Setup');
if ~isempty(figg);return;end; %don't allow multiple instances to run

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal');

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton1_Callback);


uicontrol('style','text','string','Step 1: Choose a default directory for RAVE files.',...
    'position',[30 340 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');

uicontrol('style','text','string','When RAVE needs to create a new file, it will put it in this folder. This mostly happens when RAVE needs to convert a plain text function to a MATLAB function file. It is recommended that you make a new folder specifically for this purpose. Name it "ravefiles" or anything you like. This folder should be below your usual MATLAB working directory, not below the RAVE toolbox directory.',...
    'position',[30 280 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');



bb=uicontrol('style','pushbutton','string','Browse...','position',[30 200 80 24],'callback',@browsebutton_Callback);
de=uicontrol('style','edit','string','','position',[120 200 540 24],'callback','','hor','left','back',[1 1 1],'tag','defaultdiredit');

if raveprefs.isportable
    uicontrol('style','text','string','Note: Since you selected Portable Installation, you cannot pick this directory, it MUST be the "ravefiles" folder under your main "rave" directory. Click "Next" to continue.',...
        'position',[30 226 640 60],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    mkdir(raveroot,'ravefiles');
    set(de,'string',fullfile(raveroot,'ravefiles'),'enable','off');
    set(bb,'enable','off')
    
end





function screen2

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
    'resize','on','windowstyle','normal','resizefcn',@windowresize_Callback);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton2_Callback);

uicontrol('style','text','string','Step 2: Specify size and position of RAVE window.',...
    'position',[30 380 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');

uicontrol('style','text','string','Resize and drag *this window* to the desired size and position that you wish the RAVE window to appear on your screen. It is recommended that you simply maximize this window on the monitor on which you wish RAVE to appear. In order to function properly, the window must be at least 850 pixels high. Note, the RAVE window is not resizable during use.',...
    'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');

uicontrol('style','text','string','Current size is too small. Make this window at least 400 pixels taller. Click "Next" when you are finished.',...
    'position',[30 260 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal','foregroundcolor',[1 0 0],'tag','windowsizetext');


function screen3
hhh=raveeditthemefontsgui([],1);
uiwait(hhh);
screen4

function screen4
% hhh=raveeditthemecolorsgui([],1);
% uiwait(hhh);
screen5


function screen5

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
    'resize','on','windowstyle','normal','resizefcn',@windowresize_Callback);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton5_Callback);

uicontrol('style','text','string','Step 5: Set up file type associations',...
    'position',[30 380 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');


if ispc && ~raveprefs.isportable
    uicontrol('style','text','string','If you would like, the file extensions used by rave (.rve and others) can be associated with MATLAB so that they have RAVE icons and can be opened by double clicking. This requires adding some information to your registry. If you would like to do this, click the button below. Otherwise, click Next to continue.',...
        'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
    
    uicontrol('style','push','pos',[250 250 200 24],'string','Associate .rve file type with MATLAB','callback',@windowsfiletypes);
    
elseif raveprefs.isportable
    uicontrol('style','text','string','File type associations cannot be set up for a Portable Installation. Click "Next" to continue.',...
        'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
    
else
    uicontrol('style','text','string','Unfortunately I don''t know how to automatically set up file associations for your operating system. If you know how to do this from MATLAB, please contact mdaskilewicz@asdl.gatech.edu. Click Next to skip this step and continue.',...
        'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
end


function screen6

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
    'resize','on','windowstyle','normal','resizefcn',@windowresize_Callback);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Finish','fontsize',12,'callback',@nextbutton6_Callback);

uicontrol('style','text','string','Step 6: Almost done...',...
    'position',[30 380 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');

uicontrol('style','text','string','There are a few more options that you must set up manually if you''d like to use them:',...
    'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');

uicontrol('style','text','string','1. If you want optimizers to be able to send you text messages when they finish running, edit the function ravesetuptexting, update your information, and then run the function. (You may also need to edit ravesendtxtmessage.m if you don''t have a major US cell carrier)',...
    'position',[30 280 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');

uicontrol('style','text','string','2. If you want to export graphs as pdfs, you must have ghostscript installed and on your operating system path. See the RAVE install manual for more info.',...
    'position',[30 200 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');





%enable texting?
%check that office web componenets is installed.
%set up PDF export

function browsebutton_Callback(src,ev)

mypath=uigetdir(pwd,'Choose a directory RAVE can use to store any files it creates.');
editbox=findobj(gcf,'tag','defaultdiredit');
set(editbox,'string',mypath);



function nextbutton0_Callback(src,ev)

prefspath=which('raveprefstemp.mat');
load raveprefstemp

raveprefs.isportable=get(findobj(gcf,'tag','portablecheck'),'value');
save(prefspath,'raveprefs');


%% add rave directories to path:

if ~raveprefs.isportable
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    addpath(genpath(raveroot));
    writable=savepath;
    if ~writable
        savepath(which('pathdef.m'));        
    end
end

close(gcf);
screen1;



function nextbutton1_Callback(src,ev)
editbox=findobj(gcf,'tag','defaultdiredit');
mypath=get(editbox,'string');



if isdir(mypath)
    prefspath=which('raveprefstemp.mat');
    load raveprefstemp
    raveprefs.defaultdir=mypath;
    save(prefspath,'raveprefs');
    path(mypath,path);
    savepath;
    rehash;
    close(gcf);
    screen2;
else
    errordlg('You must enter a valid path.','Try again');
end

function nextbutton2_Callback(src,ev)
prefspath=which('raveprefstemp.mat');
load raveprefstemp
raveprefs.preferredposition=get(gcf,'position');
save(prefspath,'raveprefs');
close(gcf);
screen3;

function cancelbutton_Callback(src,ev)
close(gcf);

function windowresize_Callback(src,ev)
windowsizetext=findobj(src,'tag','windowsizetext');
if isempty(windowsizetext);return;end;

pos=get(src,'position');

height=pos(4);

if height>=850
    set(windowsizetext,'string','Window is large enough. Click "Next" when you are finished sizing.','foregroundcolor',[0 .8 0]);
else
    set(windowsizetext,'string',['Current size is too small. Make this window at least ',num2str(850-height),' pixels taller. Click "Next" when you are finished.'],'foregroundcolor',[1 0 0]);
end

drawnow;



function windowsfiletypes(src,ev)

raveroot=which('rave');
raveroot=raveroot(1:end-7);
mlroot=matlabroot;

fid=fopen(fullfile(raveroot,'rve.reg.txt'));
raveroot=strrep(raveroot,'\','\\');
mlroot=strrep(mlroot,'\','\\');

fid2=fopen('rve.reg','wt');

tline = fgetl(fid); %skip 1st line
tline = fgetl(fid);
while ischar(tline)
    tline=strrep(tline,'XXXXXX',raveroot);
    tline=strrep(tline,'YYYYYY',mlroot);
    if strcmp(computer,'PCWIN64')
        tline=strrep(tline,'win32','win64');
    end
    fprintf(fid2,'%s\n',tline);
    tline = fgetl(fid);
end

fclose all

regpath=which('rve.reg');
system(['regedit "',regpath,'"']);
delete(regpath);

function nextbutton5_Callback(src,ev)
regpath=which('rve.reg');
if ~isempty(regpath);delete(regpath);end;
close(gcf);
screen6;


function nextbutton6_Callback(src,ev)
load raveprefstemp

if ~raveprefs.isportable
    path=uigetdir(pwd,'Choose where to put the RAVE preferences file that was just created. The best place is in your default MATLAB working directory.');
    if ~ischar(path);return;end;
    prefspath=fullfile(path,'raveprefs.mat');
else
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    prefspath=fullfile(raveroot,'raveprefs.mat');    
end

save(prefspath,'raveprefs');
delete('raveprefstemp.mat');
close(gcf);
rave;

function localcheck_callback(src,ev)
set(findobj(gcf,'tag','portablecheck'),'value',0);
set(src,'value',1);
set(findobj(gcf,'tag','portablewarning'),'visible','off')

function portablecheck_callback(src,ev)
set(findobj(gcf,'tag','localcheck'),'value',0);
set(src,'value',1);
set(findobj(gcf,'tag','portablewarning'),'visible','on')
