function th=ravemakecustomticks(src,axis,ticks,strings)
%src must be handle of axes for which to apply text labels.
%axis is 'x','y',or 'z' for which axis to make ticks along.
%vals is the numerical locations along axis where ticks should be placed
%strings is the cell array of strings to put at each location.
%orient is 'horizontal' or 'vertical' indicating whether to rotate text.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

th=[];

fakeax=handles.fakeax(handles.activeindex);
fakerax=handles.fakerax(handles.activeindex);

ticks=ticks(:)'; %ensure row vector

if isfield(udata.format,'bg')
    txtcolor=handles.colors.axes{udata.format.bg}(5,:);
else
    txtcolor=[0 0 0];
end

switch axis
    case 'x'
        xticks=findall(src,'tag','xticklabel');
        delete(xticks)
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xtick',ticks,'xminortick','off','xticklabel','');

        for s=numel(strings):-1:1
            th(s)=text(nan,nan,nan,strings{s},...
                'HorizontalAlignment','center','rotation',0,'parent',src,'hittest','off',...
                'verticalalignment','top','handlevisibility','off','tag',[axis,'ticklabel'],...
                'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
                'color',txtcolor,'interpreter',handles.prefs.interpreter);
        end
        
    case 'y'
        yticks=findall(src,'tag','yticklabel');
        delete(yticks)
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',ticks,'yticklabel','','yminortick','off');

        for s=numel(strings):-1:1
            th(s)=text(nan,nan,nan,strings{s},...
                'HorizontalAlignment','right','rotation',0,'parent',src,'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag',[axis,'ticklabel'],...
                'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
                'color',txtcolor,'interpreter',handles.prefs.interpreter);
        end
        
    case 'z'
        zticks=findall(src,'tag','zticklabel');
        delete(zticks)
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',ticks,'zminortick','off');

        for s=numel(strings):-1:1
            th(s)=text(nan,nan,nan,strings{s},...
                'HorizontalAlignment','right','rotation',0,'parent',src,'hittest','off',...
                'verticalalignment','middle','handlevisibility','off','tag',[axis,'ticklabel'],...
                'fontname',handles.prefs.fonts.graph,'fontsize',handles.prefs.fonts.graphsize,...
                'color',txtcolor,'interpreter',handles.prefs.interpreter);
        end
end


