function ravemaketernaxes(src,ev,ai)
%turns the axes for graph ai into a ternary axes. This entails:
% set all axes visible off
% draw a triangle patch
% draw outer axis borders
% draw tick marks
% draw text tick labels
% draw axes labels
%these are set/formatted according to the relevant axes properties' values, so set
%those values before calling this function.
%       z
%    x / \ z
%     /   \
%    x --- y
%       y
handles=guidata(src);


targetaxes=handles.allgraphs(ai);
set(targetaxes,'xlimmode','manual','ylimmode','manual','zlimmode','manual','xticklabel','','yticklabel','','zticklabel','');
set(targetaxes,'visible','off')
set(handles.fakeax(ai),'visible','off')
set(handles.fakerax(ai),'visible','off')

fn=get(targetaxes,'fontname');
fs=get(targetaxes,'fontsize');


%get positions/scales:
pos=get(targetaxes,'position'); %pixel position


%scale xlim/yim so that triangle goes from 0-1 xlim and 0-sqrt(3) ylim:

ar=pos(3)/pos(4); %graph aspect ratio:

if ar>=2/sqrt(3);
    set(targetaxes,'ylim',[0 sqrt(3)/2],'xlim',[0.5-sqrt(3)/4*ar sqrt(3)/4*ar+0.5])
else
    set(targetaxes,'xlim',[0 1],'ylim',[sqrt(3)/4-(1/ar)/2 (1/ar)/2+sqrt(3)/4])
end


%make background patch:
bg=patch([0 1 0.5],[0 0 sqrt(3)/2],[-1 -1 -1],get(targetaxes,'color'),'edgecolor',[0 0 0],'parent',targetaxes,'tag','axesbackground');

if ~get(targetaxes,'box')
    set(bg,'edgecolor','none');
end


%make grid lines:
xtick=get(targetaxes,'xtick');
ytick=get(targetaxes,'ytick');
ztick=get(targetaxes,'ztick');

xtick=xtick(xtick<=1 & xtick>=0);
ytick=ytick(ytick<=1 & ytick>=0);
ztick=xtick(ztick<=1 & ztick>=0);



t=0.015; %ticklength = 1.5 percent



%% grid lines, ticks, and tick labels:

% z axis:
if ~isempty(ztick)
    [x, y] = terncoords(1-ztick, ztick, zeros(size(ztick)));
    [xo,yo]= terncoords(zeros(size(ztick)), ztick, 1-ztick);
    for j=1:numel(x)
        text(x(j)+0.005, y(j),[' ',num2str(ztick(j))],'fontsize',fs,'fontname',fn,'parent',targetaxes,'tag','zticklabel','handlevis','off','hittest','off');
        if ztick(j)~=0
            line([x(j) xo(j)],[y(j) yo(j)],'color',[0.7 0.7 0.7],'linewidth',0.5,'tag','zgrid','parent',targetaxes,'handlevis','off','hittest','off');
            if ztick(j)~=1
                line([x(j) x(j)-t],[y(j) y(j)],'color',[0 0 0],'linewidth',0.5,'tag','zgrid','parent',targetaxes,'handlevis','off','hittest','off');
            end
        end
    end
end

% x axis ticks:
if ~isempty(xtick)
    [x, y] = terncoords(zeros(size(xtick)), 1-xtick, xtick);
    [xo, yo] = terncoords(1-xtick, zeros(size(xtick)), xtick);
    for j=1:numel(x)
        text(x(j)-0.005, y(j)+0.005,num2str(xtick(j)),'fontsize',fs,'fontname',fn,'parent',targetaxes,...
            'hor','right','rot',-60,'handlevis','off','hittest','off');
        if xtick(j)~=0
            line([x(j) xo(j)],[y(j) yo(j)],'color',[0.7 0.7 0.7],'linewidth',0.5,'tag','xgrid','parent',targetaxes,'handlevis','off','hittest','off');
            if xtick(j)~=1;
                line([x(j) x(j)+t/2],[y(j) y(j)-t*sqrt(3)/2],'color',[0 0 0],'linewidth',0.5,'tag','xtick','parent',targetaxes,'handlevis','off','hittest','off');
            end
        end
    end
end

% y axis ticks:
if ~isempty(ytick)
    [x, y] = terncoords(ytick, zeros(size(ytick)), 1-ytick);
    [xo, yo] = terncoords(ytick, 1-ytick, zeros(size(ytick)));
    for j=1:numel(x)
        text(x(j)-0.015, y(j)-0.005,num2str(ytick(j)),'fontsize',fs,'fontname',fn,'parent',targetaxes,...
            'hor','right','rot',60,'vert','top','handlevis','off','hittest','off');
        if ytick(j)~=0
            line([x(j) xo(j)],[y(j) yo(j)],'color',[0.7 0.7 0.7],'linewidth',0.5,'tag','ygrid','parent',targetaxes,'handlevis','off','hittest','off');
            if ytick(j)~=1
                line([x(j) x(j)+t/2],[y(j) y(j)+t*sqrt(3)/2],'color',[0 0 0],'linewidth',0.5,'tag','ytick','parent',targetaxes,'handlevis','off','hittest','off');
            end
        end
        
    end
end

%% axis labels:

xlab=get(targetaxes,'xlabel');
ylab=get(targetaxes,'ylabel');
zlab=get(targetaxes,'zlabel');

%need to explicitly set visible on since axes visible is off.
set(xlab,'position',[0.5 -0.06],'string','xlabel','visible','on','vert','top','hittest','off')
set(ylab,'position',[0.15 0.5*sqrt(3)/2],'string','ylabel','rotation',60,'visible','on','vert','bottom','hor','center','hittest','off')
set(zlab,'position',[0.85 0.5*sqrt(3)/2],'string','zlabel','rotation',-60,'visible','on','vert','bottom','hor','center','hittest','off')




