function graphname=ravenamegraph(src,ev,ai)
%generate a name for the current graph.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
handles=guidata(src);
if nargin<3
    ai=handles.activeindex;
end



udata=get(handles.fakeax(ai),'userdata');

graphname=handles.graphnames{ai};
if ~isempty(graphname);return;end

thistype=get(handles.allgraphs(ai),'userdata');


allnames=handles.allnames{udata.view.dataset}; 

%make size modifier:
pos=get(handles.allgraphs(ai),'position');
graphsize=pos(3)*pos(4);

if graphsize<50000
    size=' really small';
elseif graphsize<200000
    size=' small';
elseif graphsize<500000;
    size=' average-sized';
elseif graphsize<1000000;
    size=' large';
elseif graphsize<1700000;
    size=' very large';
else
    size=' gigantic';
end

%make aspect ratio modifier:

ar=pos(3)/pos(4);
if ar<.3334
    artext=', short';
elseif ar>=3
    artext=', tall';
else
    artext='';
end

%make graph type text:

type=[' ',lower(handles.graphinfo.(thistype).longname)];

%make variable names text:

switch handles.graphinfo.(thistype).axesformat
    case 'single'
        if all([udata.view.x udata.view.y]>0 & [udata.view.x udata.view.y]<=numel(allnames))       
             vars=[' of ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}]; 
        else
            vars='';
        end
        
%     case 'matrix'
%         vars=[' of ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}];
    case '3D'
       if all([udata.view.x udata.view.y udata.view.z]>0 & [udata.view.x udata.view.y udata.view.z]<=numel(allnames))   
        vars=[' of ',allnames{udata.view.z},' vs. ',allnames{udata.view.y},' vs. ',allnames{udata.view.x}];
       else
           vars='';
       end
    otherwise
        allvars=unique([udata.view.x(:);udata.view.y(:);udata.view.z(:);udata.view.c(:)]);
        allvars=allvars(allvars>0 & allvars<=numel(allnames));
        vars='';
        if numel(allvars)<5 && ~isempty(allvars)
            vars=[' of ',allnames{allvars(1)}];
            for n=2:numel(allvars)
               vars=[vars ', ' allnames{allvars(n)}];    
            end
        else
            vars=[' of ',num2str(numel(allvars)),' variables'];
        end
end

graphname=['The',size,artext,type,vars];








