function raveopenstate(varargin)
%
%This function is used to open saved rave projects when they are
%doubleclicked in windows explorer.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%add rave java stuff to path:
if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveTable.jar'));
end

if nargin==1
    %only one input, was double clicked in windows.
    fullname=varargin{1}; 
else
    %two inputs
    src=varargin{1};
    eventdata=varargin{2};
    [name,path,index]=uigetfile({'*.rve','RAVE Project (*.rve)';'*.*','All Files (*.*)'},'Choose project to load:');
    if name==0;return;end
    fullname=fullfile(path,name);
end

[pathstr, name, ext] = fileparts(fullname); 
copyfile(fullname,fullfile(pathstr,['RAVETEMP' name '.fig']),'f')

%warning('off','MATLAB:dispatcher:UnresolvedFunctionHandle');
handy=openfig(fullfile(pathstr,['RAVETEMP' name '.fig']));
set(handy,'windowbuttonmotionfcn','');
warning('on','MATLAB:dispatcher:UnresolvedFunctionHandle');
delete(fullfile(pathstr,['RAVETEMP' name '.fig']))
handles=guidata(handy);

%% size to fit new monitor:
%resize to fill screen

load raveprefs

set(handles.fig,'units','pixels')
handles.prefs.fonts=raveprefs.fonts;
handles.colors=raveprefs.colors;
handles.colors.wsbg=get(handles.mainaxes,'color');

%% for compatibiltiy with older versions:
if ~isfield(handles,'boneyard')
    handles.boneyard=uipanel('parent',handles.fig,'visible','off');
    uistack(handles.boneyard,'bottom');
end

if ~isfield(handles,'ispointofinterest')
    for d=1:numel(handles.datasetnames)
        handles.ispointofinterest{d}=false(size(handles.alldata{d},1),1);        
    end
end

if ~isfield(handles,'discretefilter')
    handles.discretefilter=handles.discretevalues;
end    

if ~isfield(handles,'zoommode')
    handles.zoommode='selection';
end   

handles.mousetimer=timer('TimerFcn',[],'startdelay',handles.prefs.mouseoverdelay,'executionmode','singleshot');


%%

guidata(handles.fig,handles);

set(handles.fig,'position',raveprefs.preferredposition)
msgstate=warning('query','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');
warning('off','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');

raveresizefcn(handles.fig,[]);
handles=guidata(handles.fig); %resizefcn refreshes managetab;

%fix tab fonts:
ftabs=findobj(handles.fronttabaxes,'type','text');
set(ftabs,'fontname',raveprefs.fonts.tab,'fontsize',(18*((handles.prefs.tabscale-1)*2+1))*handles.prefs.tabscale);
btabs=findobj(handles.backtabaxes,'type','text');
set(btabs,'fontname',raveprefs.fonts.tab,'fontsize',(18*((handles.prefs.tabscale-1)*2+1))*handles.prefs.tabscale);

%make please wait notice:
pos=get(handles.fig,'position');
[j,h]=javacomponent('javax.swing.JTextPane',[pos(3)/2-150 pos(4)/2-25 300 50],handles.fig);
j.setBackground(java.awt.Color(1,1,.8));
j.setBorder(javax.swing.border.LineBorder.createBlackLineBorder);
j.setFont(java.awt.Font(raveprefs.fonts.tab,1,24*raveprefs.fonts.tabscale)); %1=bold 
j.setText('Loading, Please Wait...');
drawnow;

%update java graphics stuff:
handles.axiscanvas=findjobj(handles.fig,'class','AxisCanvas'); 


%replace axeslisteners... evidently they are not serializable:
handles=rmfield(handles,'axeslisteners');
handles.axeslisteners={};
for g=1:numel(handles.allgraphs)
    %handles.axeslisteners{g}=[];
    if ishandle(handles.allgraphs(g))
        if isempty(handles.subax{g}) %make single axes listeners
            try %these dont work on 2007b,
                handles.axeslisteners{g}(1)=addlistener(handles.allgraphs(g),'XTick','PreSet',@axeschangelistener);
                handles.axeslisteners{g}(2)=addlistener(handles.allgraphs(g),'YTick','PreSet',@axeschangelistener);
                handles.axeslisteners{g}(3)=addlistener(handles.allgraphs(g),'ZTick','PreSet',@axeschangelistener);
            end
        else %make subax listeners:
            ax=handles.subax{g};
            [rows,cols]=size(ax);
            handles.axeslisteners{g}=addlistener(ax(1,1),'YTick','PreSet',@subaxeschangelistener);
            for r=2:rows
                try %these dont work on 2007b,
                    handles.axeslisteners{g}(end+1)=addlistener(ax(r,1),'YTick','PreSet',@subaxeschangelistener);
                end
            end
            
            for c=1:cols
                try %these dont work on 2007b,
                    handles.axeslisteners{g}(end+1)=addlistener(ax(end,c),'XTick','PreSet',@subaxeschangelistener);
                end
            end
            
        end
    end
end
guidata(handles.fig,handles);
%delete old table and make a new one:
%delete(handles.maintable.handle);

%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

set(handles.maintable.handle,'visible','off');

%create table models for data sets:
a=handles.maintable.a;

for d=1:numel(handles.alldata);
   handles.analysisupdatespending{d}{a}={''};   
end

guidata(handles.fig,handles);

for d=1:numel(handles.alldata);
   [handles.maintable.models{d} handles.maintable.rhmodels{d}]=ravecreatemaintablemodel(handles.fig,[],d,a);    
end

handles.tablemode=~handles.tablemode;

delete(handles.tabobjects(ishandle(handles.tabobjects)));
handles.tabobjects=[];

guidata(handles.fig,handles);

raveupdategraphtypes(handles.fig,[]);
handles=guidata(handles.fig);


%select rows that are selected in this analysis, or else showhideworkspace
%will clear the selection.
% d=handles.maintable.d
% selectedRows=find(handles.isselected{d}{a}); 
% if ~isempty(selectedRows)
%     selectedRows=selectedRows-1;
%     
%     for g=1:numel(selectedRows);
%         selectedRows(g)=handles.maintable.models{d}.viewIndex(selectedRows(g));
%     end
%     ncols=handles.maintable.models{d}.getColumnCount();
%     
%     jtable.setColumnSelectionInterval(0,ncols-1);
%     jtable.setRowSelectionInterval(selectedRows(1),selectedRows(1));
%     for i=2:numel(selectedRows)
%         jtable.addRowSelectionInterval(selectedRows(i),selectedRows(i));
%     end
% end


%fix table visiblity:
showhideworkspace(handles.fig,[]);
handles=guidata(handles.fig);

%finish up:
ravemoveview(handles.fig,[],[0 handles.prefs.wsheight-1+35]);

managetab(handles.fig,[]);
handles=guidata(handles.fig);

warning(msgstate.state,'MATLAB:hg:uicontrol:ParameterValuesMustBeValid');



uistack(handles.pageedit,'top');
uistack(handles.pagecontrols.left,'top');
uistack(handles.pagecontrols.right,'top');

ppp=findjobj(handles.pageedit);
ppp.setBorder(javax.swing.BorderFactory.createEmptyBorder)

%not sure why these turn off, but they do:
set(handles.pagecontrols.left,'hittest','on');
set(handles.pagecontrols.right,'hittest','on');


%drawnow;
%set(handles.fig,'DoubleBuffer','on'); %fix CoDE "out of windows resources" error.
%re-javacomponent the popupmenus on any graphaxes controls. Not sure why
%this needs to be done, but for some reason saving/opening the figure
%resets their parent to []. Javacomponent makes a new
%com.mathworks.hg.HPPanel for their parent:

ag=handles.activegraph;
ai=handles.activeindex;
at=handles.activetype;


for g=1:numel(handles.allgraphs)
    if ~ishandle(handles.allgraphs(g));continue;end
    if strcmp(get(handles.allgraphs(g),'userdata'),'graphaxes')
        udata=get(handles.fakeax(g),'userdata');
        handles.activegraph=handles.allgraphs(g);
        handles.activeindex=g;
        handles.activetype='graphaxes';
        
        guidata(handles.fig,handles);        
        ravecreategraphaxes(handles.fig,[]);
        handles=guidata(handles.fig);
    end   
end

handles.activegraph=ag;
handles.activeindex=ai;
handles.activetype=at;


guidata(handles.fig,handles);
delete(h);

set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)





