function output=raveparetofront(objective,dorank)
%determines pareto front of objective where each objective is to maximize.
% objective is a n x m matrix where each column contains the
% values of one objective.
% if dorank=false (or omited), output is n x 1 logical indicating
% whether that point is on the pareto front.
% if dorank=true, output is a n x 1 vector of integers indicating each
% point's non-domination rank. 1 = on the pareto front.

if nargin==1
    dorank=false;
end

allpts=objective;

%do deb O(MN^2) scheme on pg 43 of Deb book.

S = false(size(allpts,1));
count = zeros(size(allpts,1),1);

for i=1:size(allpts,1);
    testpt=allpts(i,:);
    count(i)=dominancecount(testpt,allpts);
    S(:,i)=finddominatees(testpt,allpts);
end

%assign ranks:
ranks=nan(size(allpts,1),1);
r=1;

isaccounted=false(size(allpts,1),1);

while any(~isaccounted);
    idx=find(count==0 & ~isaccounted);
    isaccounted(idx)=true;
    ranks(idx)=r;
    nsubs=sum(S(:,idx),2);
    count=count-nsubs;
    r=r+1;
end

if dorank
    output=ranks;
else
    output=ranks==1;
end

function [count,idx]=dominancecount(testpt,allpts)
%counts how many points in allpts dominate testpt, assuming all objectives
%are maximize. Optionally also returns the idx of these points (rows in
%allpts). 

idx=finddominators(testpt,allpts);
count=sum(idx);


function idx=finddominatees(testpt,allpts)
%returns a logical index into rows of allpts indicating which of those pts
%are dominated by the testpt.
%
%presumes that each objective is to maximize. 

normmatrix=bsxfun(@minus,allpts,testpt);
idx=all(normmatrix<=0,2) & ~all(normmatrix==0,2);


function idx=finddominators(testpt,allpts)
%returns a logical index into rows of allpts indicating which of those pts
%dominate the testpt.
%
%presumes that each objective is to maximize. 

normmatrix=bsxfun(@minus,allpts,testpt);
idx=all(normmatrix>=0,2) & ~all(normmatrix==0,2);
