function [varnames,data,textdata,discretevalues,istext,islogical]=raveparsedatafile(infile,in)
%parse the data file "infile" (a string) and return variable names and
%data. Infile can also be a string name of a variable in the base
%workspace. In is a structure similar to the one obtained by importdata
%when infile is actually a file. If infile is not a file, you must manually
%create this structure and include it as an input to this function

if isempty(in)
    in=importdata(infile);
end

if length(infile)>3 && strcmp(infile(end-3:end),'.xls')
    try
        sheetname=fieldnames(in.data);
    catch %#ok<CTCH>
        sheetname='';
    end
    if ~isempty(sheetname)
        in.data=in.data.(sheetname{1});
        if isfield(in,'colheaders')
            in.colheaders=in.colheaders.(sheetname{1});
            in.textdata=in.textdata.(sheetname{1})(:,:);
        else
            in.colheaders=in.textdata.(sheetname{1})(1,:);
            in.textdata=in.textdata.(sheetname{1})(2:end,:);
        end
    else
        if isfield(in,'colheaders')
            %do nothing
        else
            in.colheaders=in.textdata(1,:);
            in.textdata=in.textdata(2:end,:);
        end
    end
    varnames=in.colheaders;
else
    varnames=in.textdata(1,:);
end

for v=1:numel(in.textdata);
    if strcmp(in.textdata{v},'NaN')
        in.textdata{v}='';
    end
end

%check for textvars:
istext=false(1,size(in.textdata,2));
thinktext=istext; %variables matlab *thinks* are text
if size(in.textdata,1)>1;
    for g=1:size(in.textdata,2);
        if any(~cellfun(@isempty,in.textdata(2:end,g)))
            thinktext(g)=true;
            %             if isnan(str2double(texttest{g}))
            %                 istext(g)=true;
            %             end
        end
    end
end

numericcols=~thinktext;
istext=thinktext;

%remove columns of in.data that are ALL nans.
loseit=false(1,size(in.data,2));
for g=1:size(in.data,2)
    if all(isnan(in.data(:,g)))
        loseit(g)=true;
    end
end

if any(loseit)
    errordlg('I can''t import a data set that contains columns where EVERY value is NaN')
    set(handles.donebutton,'enable','on','string','Done');
    drawnow;
    return
end

in.data=in.data(:,~loseit);

try
    data=nan(size(in.data,1),size(in.textdata,2));
    data(:,numericcols)=in.data;
catch %#ok<CTCH>
    %l=lasterror;
    %sstring=l.message;
    
    ncol=size(in.data,2);
    ntab=sum(in.textdata{1}==char(9));
    ncomma=sum(in.textdata{1}==',');
    
    errordlg({'Something went wrong. Make sure number of data columns matches number of column headers. MATLAB found:';...
        [num2str(ncol), ' columns of data'];...
        [num2str(ntab),' tab delimiters in first row of file'];...
        [num2str(ncomma),' comma delimiters in first row of file'];...
        'Note: Rave uses the built-in MATLAB function ''importdata'' to read in data sets.'});
    %set(handles.donebutton,'enable','on','string','Done');
    drawnow;
    return
end


numrows=size(data,1);
numvars=size(data,2);

if any(istext)
    textdata=in.textdata(2:end,:);
else
    textdata=cell(numrows,numvars);
end

discretevalues=cell(1,numvars);
%convert text values to numerical indexes:
for j=find(istext)
    discretevalues{j}=unique(textdata(:,j));
    for h=1:numel(discretevalues{j});
        isthisvalue=strcmp(discretevalues{j}{h},textdata(:,j));
        data(isthisvalue,j)=h;
    end
    
    for h=1:numel(discretevalues{j})
        discretevalues{j}=strtrim(discretevalues{j});
    end
end

%check for logical variables:

logicalz={'yes';'no';'true';'false';'on';'off'}; %#ok<NASGU>

islogical=false(1,numvars);

load raveprefs
if raveprefs.autodetectlogicalvars

for j=1:numvars %for each variable:
    youneeks=unique(data(:,j));
    if numel(youneeks)==2
        if youneeks(1)==0 && youneeks(2)==1
            islogical(j)=true;
            discretevalues{j}=[0 1];
        end
    end
end

end