function ravesavestate(src,eventdata)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

set(handles.infobar,'string','')

%ask user for a name for this project.

if handles.tablemode
    msgbox('You cannot save a project while viewing the data table. Switch back to the workspace view, and then save the project.',...
        'Sorry about that...','error');
    return
end



h=ravesavegui([],handles.projectname);
uiwait(h);
name=get(0,'userdata');

%did they cancel?
if isempty(name);return;end

load raveprefs
path=raveprefs.defaultdir;

set(0,'userdata','')

if ~isempty(name)
    
    blanktab(handles.fig,[]); %clear tabs so tabobjects don't get saved.
    handles=guidata(handles.fig);
    
    %delete tray icon:
    
    pause(0.1) %to avoid java crash
    sysTray = java.awt.SystemTray.getSystemTray;
    if (sysTray.isSupported)
        java.awt.SystemTray.getSystemTray.remove(handles.trayIcon);
    end
    pause(0.1)
    handles.trayIcon=[];
    set(handles.fig,'windowbuttonmotionfcn','')
    handles.projectname=name;
    
    handles.optimizetabobjects=[]; %clear java table from optimize tab
    
    guidata(handles.fig,handles);
    
    
  %  msgstate=warning('query','MATLAB:Java:ConvertFromOpaque');
  %  warning('off','MATLAB:Java:ConvertFromOpaque');
    
    if ~isdir(fullfile(path,name))
        mkdir(fullfile(path,name));
    end
    
    filename=fullfile(path,name,[name '.rve']);
    
    hgS = handle2struct(handles.fig);
    SaveVer = '070000'; %if error, change to '080000'
    SaveVars.(sprintf('hgS_%s', SaveVer)) = hgS;
  
    %no, really:
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.jtable=[];
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.defaultRowHeaderRenderer=[];
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.rhmodels=[];
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.models=[];
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.maintable.jscroll=[];
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.mousetimer=[]; %timer objects cant be saved
    SaveVars.hgS_070000.properties.ApplicationData.UsedByGUIData_m.sounds=[]; %audioplayer objects cant be saved
    
    %Remove the java controls associated with graphaxes control types. This
    %is necessary because their delete functions are specific to the MATLAB version used to create them. (I
    %suppose I could also jsut clear their delete functions, but presumably
    %those are doing something important).
    gacontrols=[];
    for c=1:numel(handles.allgraphs);
        if ishandle(handles.allgraphs(c)) && strcmp(get(handles.allgraphs(c),'userdata'),'graphaxes')
            gacontrols=union(gacontrols,handles.fakeax(c));
        end
    end
    
    if ~isempty(gacontrols)
        for g=1:numel(SaveVars.hgS_070000.children)
            if strcmp(SaveVars.hgS_070000.children(g).type,'uipanel')
                %look for hgjavacomponents:
                for j=1:numel(SaveVars.hgS_070000.children(g).children)
                    if  strcmp(SaveVars.hgS_070000.children(g).children(j).type,'hgjavacomponent')
                        mfd(j)=true;
                    else
                        mfd(j)=false;
                    end
                    
                    %also need to remove these from userdata so that we
                    %dont get an invalid handle error next time we try to delete them:
                    if  strcmp(SaveVars.hgS_070000.children(g).children(j).type,'axes')
                        if ismember(SaveVars.hgS_070000.children(g).children(j).handle,gacontrols)
                            SaveVars.hgS_070000.children(g).children(j).properties.UserData.view.X.menujobj=[];
                            SaveVars.hgS_070000.children(g).children(j).properties.UserData.view.Y.menujobj=[];
                            SaveVars.hgS_070000.children(g).children(j).properties.UserData.view.Z.menujobj=[];
                            SaveVars.hgS_070000.children(g).children(j).properties.UserData.view.C.menujobj=[];
                            SaveVars.hgS_070000.children(g).children(j).properties.UserData.view.S.menujobj=[];
                        end
                    end
                end
                SaveVars.hgS_070000.children(g).children(mfd)=[]; %delete this.
            end
        end
    end
    
    drawnow; %cant hurt
    save(filename, '-struct', 'SaveVars');

    handles.activedir=path;
    
    %restore tray icon:   
    if (sysTray.isSupported)
        raveroot=which('rave');
        raveroot=raveroot(1:end-6);
        myIcon = fullfile(raveroot,'images\tinyrave.png');
        iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
        handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
        handles.trayIcon.setToolTip('RAVE is running.');
        sysTray.add(handles.trayIcon)
    else
        handles.trayIcon=[];
    end

    %restore mouse motion function:
    set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)

    ravefinish

    guidata(src,handles);
    managetab(handles.fig,[]);
    handles=guidata(handles.fig);
    set(handles.infobar,'string','Project saved successfully.')
end



guidata(src,handles);