function raveupdateexploremethods(src,ev,AI,refreshtype)
%refreshes all exploremethods related to the graph in ai in response to the
%specified refreshtype. Valid options for refreshtype:
%  recreate - the graph has been recreated by calling ravecreate<type>
%
%Explore methods must declare these refreshtypes in properties.updates in
%order for this function to refresh them. (Except for "recreate" which
%always results in an update.)
%
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
originalindex=handles.activeindex;

for ai=AI
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    
    thistype=get(othergraph,'userdata'); %type of other graph
    udata=get(handles.fakeax(ai),'userdata'); %user data of othergraph
    methods=udata.explore.method;
    if isempty(methods);continue;end
   
    handles.activeindex=ai;
    handles.activegraph=handles.allgraphs(ai);
    handles.activetype=thistype;
    guidata(handles.fig,handles);
    
    
    %check for exploremethods that need to be updated:
    for m=1:numel(methods)
        if ismember({refreshtype},handles.exploreinfo.(methods{m}).updates) || strcmpi(refreshtype,'recreate');
            if ~isempty(which(['raveupdateexplore',methods{m},refreshtype]))
                eval(['raveupdateexplore',methods{m},refreshtype,'(handles.fig,[]);']);
            else
                eval(['raveenable',methods{m},'(handles.fig,[]);']);
            end
        end
    end
    handles=guidata(handles.fig);
    
end     

lastgraph=handles.allgraphs(originalindex);
thistype=get(lastgraph,'userdata'); %type of other graph in this analysis
handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=lastgraph;
guidata(handles.fig,handles);


