function [wrapped,status,nrows,finalwidth]=superiorlinewrap(string,maxpixels,fontname,fontsize,fontstyle,AxisCanvas)
%wrap a string to fit a width of "pixels" using the characterwidth matrix
%charwidths (see also characterwidths.m)
%status is 1 if string was wrapped successfully or did not need wrapping
%          0 if string could not be wrapped.

status=1;
wrapped=string;
nrows=1;
finalwidth=0;

if isempty(string);return;end


%make javafont:
switch lower(fontstyle)
    case 'plain'
        style=java.awt.Font.PLAIN;
    case 'bold'
        style=java.awt.Font.BOLD;
    case 'italic'
        style=java.awt.Font.ITALIC;
    otherwise
        style=java.awt.Font.PLAIN;
end

font=java.awt.Font(fontname,style,ceil(fontsize*4/3)); %4/3 factor accounts for difference between java/matlab scales (72 vs 96 dpi);

%calculate string dimensions in pixels:
graphics=AxisCanvas.getGraphics;
try
    metrics=graphics.getFontMetrics(font);
catch
   clear functions
   metrics=graphics.getFontMetrics(font);
end
bounds=metrics.getStringBounds(string,graphics);
w=bounds.getWidth;

if w<maxpixels %no need to wrap
    finalwidth=w;
    return
end 

% breakafter=[find(ismember(string,' /)?;:_=+-*~]}')),length(string)];
% breakbefore=find(ismember(string,'({['));
%faster version using ismembc:

%look for allowable places to break lines:
brkaft=[32 41 42 43 45 47 58 59 61 63 93 125 126]; % = )*+-/:;=?]}~  ...characters allowed to line break after
brkbfr=[40 91 123];% =([{ ...characters allowed to break before
breakafter=find(ismembc(double(string),brkaft));
breakbefore=find(ismembc(double(string),brkbfr));

breakat=union(breakafter,breakbefore-1); %ok to break after these characters.

if isempty(breakat) %nowhere to break;
    finalwidth=w;
    return
end 


cumwordwidths=zeros(size(breakat));

for p=1:numel(breakat)
    bounds=metrics.getStringBounds(string(1:breakat(p)),graphics);
    cumwordwidths(p)=bounds.getWidth;
end

%cumwordwidths=cumwidth(breakat);
%determine number of rows needed:
breaks=0;

origwordwidths=cumwordwidths;

count=0;

while ~isempty(cumwordwidths) %any(cumwordwidths>0)
    if cumwordwidths(1)<=maxpixels %string can be broken successfully
        bestbreak=breakat(cumwordwidths==max(cumwordwidths(cumwordwidths<=maxpixels)));
        breaks(end+1)=bestbreak;
    else %make first possible break
        bestbreak=breakat(1);
        breaks(end+1)=bestbreak;
        break
    end
    
    idx=breakat>bestbreak;
    cumwordwidths=cumwordwidths(idx);
    breakat=breakat(idx);
    
    for p=1:numel(breakat)
        bounds=metrics.getStringBounds(string(breaks(end)+1:breakat(p)),graphics);
        cumwordwidths(p)=bounds.getWidth;
    end
    count=count+1;

end

if breaks(end)~=length(string)
    breaks=[breaks length(string)];
end

newstr={};
finalwidth=[];
for q=1:numel(breaks)-1;
    st=strtrim(string(breaks(q)+1:breaks(q+1)));
    if ~isempty(st)
        newstr{end+1}=strtrim(string(breaks(q)+1:breaks(q+1)));
        bounds=metrics.getStringBounds(newstr{end},graphics);
        finalwidth(end+1)=bounds.getWidth;
        
    end
end

wrapped=newstr;
finalwidth=max(finalwidth);
if finalwidth>maxpixels
    status=0;
end

if iscell(wrapped)
    nrows=numel(wrapped);
end







%{








while ~isempty(breakat)
if numel(breakat)==1 %only one break, so make it:
    wrapped={string(1:breakat);string(breakat+1,end)};
else %determine whick breaks to make:
pixes=zeros(size(breakat));
cumwordwidths=cumwidth(breakat);

bounds=metrics.getStringBounds(string(1:breakat(1)),graphics); 
pixes(1)=bounds.getWidth;

for p=2:numel(breakat);
    bounds=metrics.getStringBounds(string(1:breakat(p)),graphics);
    pixes(p)=bounds.getWidth;
    
    if
 
    end
    
end
    

%determine width at each allowable break:





%determine number of rows needed:
breaks=[];

origwordwidths=cumwordwidths;

while any(cumwordwidths>0)
    if cumwordwidths(1)<=maxpixels
        bestbreak=breakat(cumwordwidths==max(cumwordwidths(cumwordwidths<=maxpixels)));
    else
        bestbreak=breakat(1);
    end
    breaks(end+1)=bestbreak;
    cumwordwidths=origwordwidths-cumwidth(bestbreak);
end

breaks=[0 breaks];

if breaks(end)~=length(string)
    breaks=[breaks length(string)];
end

newstr={};
finalwidth=[];
for q=1:numel(breaks)-1;
    st=strtrim(string(breaks(q)+1:breaks(q+1)));
    if ~isempty(st)
        newstr{end+1}=strtrim(string(breaks(q)+1:breaks(q+1)));
        finalwidth(end+1)=cumwidth(breaks(q+1))-cumwidth(breaks(q)+1); %note this may be slightly larger than actual string due to strtrim
    end
end

wrapped=newstr;
finalwidth=max(finalwidth);
if finalwidth>maxpixels
    status=0;
end

if iscell(wrapped)
    nrows=numel(wrapped);
end


%}


